function grain = gtCalculateGrain(grain, parameters, varargin)
% GTCALCULATEGRAIN  Calulates diffraction vectors and detector intersection points
%
%     grain = gtCalculateGrain(grain, [parameters], varargin)
%     -----------------------------------------------------------------
%
%     INPUT:
%       grain      = <struct>   grain of interest
%       parameters = <struct>   parameters file {parameters.mat}
%
%     OPTIONAL INPUT (p/v pairs):
%       showfigure = <logical>  show or not the figure {false}
%       color      = <logical>  show or not the prediction spot positions coloured as
%                               omega {false}
%       axis       = <logical>  plot axis, image reference and beam direction in the
%                               image {false}
%       overlay    = <double>   difspots image output from "gtPlaceDifspotinFull" (MxM)
%                               M is the size of the overlay image (typically 2048)
%       markersize = <double>   marker size {10}
%       clims      = <double>   scale of grays {[-300 500]}
%       usestrain  = <logical>  use strain info
%       variants   = <logical>  Use twin variants simulated orientation
%                               stored in 'g_twins', if existing {false}
%
%     OUTPUT:
%       grain      = <struct>   grain of interest. Added fields:
%            .allblobs(fsimID)  = <struct>   Simulated reflections data
%            .used_fam          = <double>   List of used unique HKL families
%            .used_famsp        = <double>   List of all the used HKL families
%
%     Simplified version of gtFedGenerateRandomGrain   15-03-2012  - WLudwig


if (nargin < 1)
    help(mfile)
    error('gtCalculateGrain:wrong_arguments', 'Not enough arguments')
end
if (~exist('parameters', 'var') || isempty(parameters))
    parameters = gtLoadParameters();
end

% set default values for optional arguments
app.color      = false;
app.showfigure = false;
app.axis       = false;
app.markersize = 25;
app.overlay    = []; % zeros(parameters.acq.ydet, parameters.acq.xdet);
app.variants   = false;
app.usestrain  = false;
app.included   = [];
app.ref_omind  = [];
app.detectors  = [];
if (isfield(parameters.fsim, 'clims'))
    app.clims = parameters.fsim.clims;
else
    app.clims = parameters.fsim.Grayscale;
end

[app, rej_pars] = parse_pv_pairs(app, varargin);
if (app.clims == 0)
    app.clims = [];
end

if ~isfield(grain, 'full') || isempty(grain.full)
    grain.full = app.overlay;
end

% If strain data is not available, disable strain
if ~isfield(grain, 'strain') || ~isfield(grain.strain, 'strainT') || ...
    isnan(grain.strain.strainT(1, 1))

    app.usestrain = false;
end

% Rodrigues vector (grain orientation)
R_vec = grain.R_vector;
gcsam = grain.center;

cryst  = parameters.cryst(grain.phaseid);
labgeo = parameters.labgeo;
samgeo = parameters.samgeo;

if (~isfield(labgeo, 'omstep') || isempty(labgeo.omstep))
    labgeo.omstep = gtGetOmegaStepDeg(parameters);
end

%%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % Detector geometry

if (~isfield(parameters, 'detgeo') || isempty(parameters.detgeo))
    parameters.detgeo = gtGeoConvertLegacyLabgeo2Detgeo(labgeo);
end
detgeo = parameters.detgeo;
if (~isempty(app.detectors))
    detgeo = detgeo(app.detectors);
end

%%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % Crystallography from parameters file

%disp('Translating Miller indices into normalized cartesian coordinates for plane normals')
Bmat = gtCrystHKL2CartesianMatrix(cryst.latticepar);

% D-spacings in the reference crystal for the given reflections
dsp_ref = gtCrystDSpacing(double(cryst.hkl(:, cryst.thetatypesp)), Bmat);

% Plane normals in Cartesian CRYSTAL coordinates
plcry = gtCrystHKL2Cartesian(double(cryst.hklsp), Bmat);

% Compute all orientation matrices g (Reminder: Vc = g . Vs)
g = gtMathsRod2OriMat(R_vec');

% Express plane normals in cartesian SAMPLE CS
pl_sam = gtVectorCryst2Lab(plcry', g)';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute diffraction angles and detector intersection points
rotcomp = gtMathsRotationMatrixComp(labgeo.rotdir', 'col');

% Impose strain on grain
if (app.usestrain)
    % Deformation tensor (use strain tensor for now)
    defT = grain.strain.strainT;

    % New deformed plane normal and relative elongation (relative d-spacing)
    [pl_samd, drel] = gtStrainPlaneNormals(pl_sam, defT); % unit vector
else
    pl_samd = pl_sam;
    num_plane_normals = size(pl_sam, 2);
    drel = ones(1, num_plane_normals);
end

% d-spacing after applying the strain tensor
dsp_d = drel .* dsp_ref;

% Bragg angles theta and sin(theta) in deformed state is inversely proportional
% to elongation
[theta, sinth] = gtCrystTheta(dsp_d, parameters.acq.energy);

% The plane normals need to be brought in the Lab reference where the
% beam direction and rotation axis are defined. This only matters
% if the Sample reference at omega=0 does not coincide with the Lab 
% reference (usually not the case, but we should account for the 
% possibility, as it is provided in 'samgeo').
% Use the Sample -> Lab orientation transformation assuming omega=0;
% (vector length preserved for free vectors)
pl_labd = gtGeoSam2Lab(pl_samd', eye(3), labgeo, samgeo, true, false)';

% Four omegas of the plane normal (1-3 and 2-4 are the two Friedel pairs):
% pls is plus or minus pl, and St is the rotation tensor per each omega
[om4, pllab4, pls4, rot_l2s_4, omind4] = gtFedPredictOmegaMultiple( ...
    pl_labd, sinth, labgeo.beamdir', labgeo.rotdir', rotcomp, []);

% Delete those where no reflection occurs
no_fwd_diffr = (isnan(om4(1, :)) | isnan(om4(3, :)));
no_fwd_diffr4 = no_fwd_diffr([1 1 1 1], :);

%%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % Propagating the same common values for all the 4 omegas coming from a
% % single plane normal.
plorig4 = pl_sam(:, :, [1 1 1 1]);

if (size(cryst.hkl, 1) == 3 || size(cryst.hkl, 1) == 4)
    hkl = cryst.hkl;
    hklsp = cryst.hklsp;
else
    hkl = cryst.hkl';
    hklsp = cryst.hklsp';
end
num_hkl_indxs = size(hkl, 1);

thetatypesp = cryst.thetatypesp;
thetatypesp = reshape(thetatypesp, [], 1);

hkl = hkl(:, thetatypesp)';
hklsp = hklsp';

thetatypesp4 = thetatypesp(:, [1 1 1 1]);
hkl4 = hkl(:, :, [1 1 1 1]);
hklsp4 = hklsp(:, :, [1 1 1 1]);
hklsp4(:, :, [3 4]) = -hklsp4(:, :, [3 4]);
sinth4 = sinth([1 1 1 1], :);
theta4 = theta([1 1 1 1], :);
plcry4 = plcry(:, :, [1 1 1 1]);
plsamd4 = pl_samd(:, :, [1 1 1 1]);


if (isempty(app.ref_omind))
    %%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % % Following the convention of the matching output, the omega value
    % % smaller than 180deg (spot "a") is used in the pair data.

    chind4 = om4(1, :) > om4(3, :);

    pllab4(:, chind4, [1, 3]) = pllab4(:, chind4, [3, 1]);
    pls4(:, chind4, [1, 3])   = pls4(:, chind4, [3, 1]);
    om4([1, 3], chind4)       = om4([3, 1], chind4);
    omind4([1, 3], chind4)    = omind4([3, 1], chind4);
    rot_l2s_4(:, :, chind4, [1, 3]) = rot_l2s_4(:, :, chind4, [3, 1]);
    hklsp4(chind4, :, [1, 3]) = hklsp4(chind4, :, [3, 1]);

    chind4 = om4(2, :) > om4(4, :);

    pllab4(:, chind4, [2, 4]) = pllab4(:, chind4, [4, 2]);
    pls4(:, chind4, [2, 4])   = pls4(:, chind4, [4, 2]);
    om4([2, 4], chind4)       = om4([4, 2], chind4);
    omind4([2, 4], chind4)    = omind4([4, 2], chind4);
    rot_l2s_4(:, :, chind4, [2, 4]) = rot_l2s_4(:, :, chind4, [4, 2]);
    hklsp4(chind4, :, [2, 4]) = hklsp4(chind4, :, [4, 2]);

    %%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Four omegas of the plane normal (1-3 and 2-4 are the two Friedel pairs):
    % % But the two Friedel pairs are going to be 1a-1b and 2a-2b.

    pllab4(:, :, [2, 3]) = pllab4(:, :, [3, 2]);
    pls4(:, :, [2, 3])   = pls4(:, :, [3, 2]);
    om4([2, 3], :)       = om4([3, 2], :);
    omind4([2, 3], :)    = omind4([3, 2], :);
    rot_l2s_4(:, :, :, [2, 3]) = rot_l2s_4(:, :, :, [3, 2]);
    hklsp4(:, :, [2, 3]) = hklsp4(:, :, [3, 2]);
end

%%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % Unfolding the 4-fold data structures

no_fwd_diffr = reshape(no_fwd_diffr4, [], 1);

om = reshape(om4, [], 1);
omind = reshape(omind4, [], 1);
sinth = reshape(sinth4, [], 1);
theta = reshape(theta4, [], 1);

plcry = reshape(permute(plcry4, [1 3 2]), 3, []);
plsamd = reshape(permute(plsamd4, [1 3 2]), 3, []);
pllab = reshape(permute(pllab4, [1 3 2]), 3, []);
pls = reshape(permute(pls4, [1 3 2]), 3, []);
plorig = reshape(permute(plorig4, [1 3 2]), 3, []);

rot_l2s = reshape(permute(rot_l2s_4, [1 2 4 3]), 3, 3, []);

thetatypesp = reshape(thetatypesp4', [], 1);
hkl = reshape(permute(hkl4, [3 1 2]), [], num_hkl_indxs);
hklsp = reshape(permute(hklsp4, [3 1 2]), [], num_hkl_indxs);

if (~isempty(app.ref_omind))
    gr_omind_ind = [ ...
        find(omind == 1); find(omind == 2); ...
        find(omind == 3); find(omind == 4) ];
    ref_omind_ind = [ ...
        find(app.ref_omind == 1); find(app.ref_omind == 2); ...
        find(app.ref_omind == 3); find(app.ref_omind == 4) ];

    no_fwd_diffr(ref_omind_ind) = no_fwd_diffr(gr_omind_ind);

    om(ref_omind_ind) = om(gr_omind_ind);
    omind(ref_omind_ind) = omind(gr_omind_ind);
    sinth(ref_omind_ind) = sinth(gr_omind_ind);
    theta(ref_omind_ind) = theta(gr_omind_ind);

    plcry(:, ref_omind_ind) = plcry(:, gr_omind_ind);
    plsamd(:, ref_omind_ind) = plsamd(:, gr_omind_ind);
    pllab(:, ref_omind_ind) = pllab(:, gr_omind_ind);
    pls(:, ref_omind_ind) = pls(:, gr_omind_ind);
    plorig(:, ref_omind_ind) = plorig(:, gr_omind_ind);

    rot_l2s(:, :, ref_omind_ind) = rot_l2s(:, :, gr_omind_ind);

    thetatypesp(ref_omind_ind) = thetatypesp(gr_omind_ind);
    hkl(ref_omind_ind, :) = hkl(gr_omind_ind, :);
    hklsp(ref_omind_ind, :) = hklsp(gr_omind_ind, :);
end

if (~isempty(app.included))
    no_fwd_diffr = no_fwd_diffr(app.included);

    om = om(app.included);
    omind = omind(app.included);
    sinth = sinth(app.included);
    theta = theta(app.included);

    plcry = plcry(:, app.included);
    plsamd = plsamd(:, app.included);
    pllab = pllab(:, app.included);
    pls = pls(:, app.included);
    plorig = plorig(:, app.included);

    rot_l2s = rot_l2s(:, :, app.included);

    thetatypesp = thetatypesp(app.included);
    hkl = hkl(app.included, :);
    hklsp = hklsp(app.included, :);
end

%%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % Computing the remaining physical quantities

% Diffraction vectors % takes coloumn vectors
dveclab = gtFedPredictDiffVecMultiple(pllab, labgeo.beamdir')';
dveclab = gtMathsNormalizeVectorsList(dveclab);

% Eta angles % takes row vector
eta = gtGeoEtaFromDiffVec(dveclab, labgeo);

% Diffraction vector in Sample reference
dvecsam = gtGeoLab2Sam(dveclab, rot_l2s, labgeo, samgeo, true);

% Let's convert grain center Sam -> Lab
rot_s2l = permute(rot_l2s, [2 1 3]);
gcsam_v = gcsam(ones(1, size(rot_l2s, 3)), :);
gclab_v = gtGeoSam2Lab(gcsam_v, rot_s2l, labgeo, samgeo, false);

% Lorentz factor for the given projections (note that eq. 4.1 from page 36
% in Henning's book is not correct)
Lfactor = 1 ./ abs(sind(eta));

grain.allblobs.plcry = plcry';      % undeformed plane normals in the Crystal ref
grain.allblobs.plorig = plorig';    % unsigned undeformed plane normals in Sample ref
grain.allblobs.plsamdef = plsamd';  % unsigned deformed plane normals in Sample ref
grain.allblobs.pl = pls';           % signed (!) deformed plane normals in Sample ref
grain.allblobs.pllab = pllab';      % signed (!) deformed plane normals rotated to diffraction in Lab ref
grain.allblobs.hkl = hkl;           % {hkl} index
grain.allblobs.hklsp = hklsp;       % specific/signed (hkl) index
grain.allblobs.sintheta = sinth;    % sin(Bragg_angle) in deformed state
grain.allblobs.theta = theta;       % Bragg angle in deformed state (deg)
grain.allblobs.eta = eta;           % Eta azimuth angle in deformed state (deg)
grain.allblobs.thetatype = thetatypesp;
grain.allblobs.lorentzfac = Lfactor;% Lorentz factor
grain.allblobs.omega = om;          % omega rotation angles (0..360deg) in degrees
grain.allblobs.omind = omind;       % omega variant index (1..4) 
grain.allblobs.dvec = dveclab;      % diffracted beam dir in Lab ref
grain.allblobs.dvecsam = dvecsam;   % diffracted beam dir in Sample ref
grain.allblobs.srot = rot_l2s;      % Sample->Lab (!) rotation coordinate transform matrix for column (!) vectors
                                    % OR Lab->Sample (!) rotation coordinate transform matrix for row (!) vectors

for n = 1:numel(detgeo)
    % u,v,w coordinates on the detector
    uvw = gtFedPredictUVWMultiple([], dveclab', gclab_v', ...
        detgeo(n).detrefpos', detgeo(n).detnorm', detgeo(n).Qdet, ...
        [detgeo(n).detrefu, detgeo(n).detrefv]', om', labgeo.omstep)';

    % Precompute which reflections will actually fall into the detector
    ondet = ~no_fwd_diffr & ...
        (uvw(:, 1) >= 1) & (detgeo(n).detsizeu >= uvw(:, 1)) & ...
        (uvw(:, 2) >= 1) & (detgeo(n).detsizev >= uvw(:, 2));

    % Initialse output variables
    grain.allblobs.detector(n) = struct('uvw', {uvw}, 'ondet', {ondet});

    % Visual Feedback
    if (app.showfigure)
        sfDisplayFigure(parameters, app, grain, n);
    end
end

grain.full = [];

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% sub-functions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function sfDisplayFigure(parameters, app, grain, det_ind)
    detgeo = parameters.detgeo(det_ind);
    labgeo = parameters.labgeo;

    detusize = detgeo.detsizeu;
    detvsize = detgeo.detsizev;

    if (isempty(app.overlay))
        app.overlay = zeros(detvsize, detusize);
    end

    hf = figure();
    ax = axes('Parent', hf);
    imshow(app.overlay, app.clims, 'Parent', ax);
    hold(ax, 'on');

    if (app.axis)
        % Detector geometry
        beamdir = labgeo.beamdir';
        rotdir  = labgeo.rotdir';
        detdiru = detgeo.detdiru';
        detdirv = detgeo.detdirv';
        Qdet    = gtFedDetectorProjectionTensor(detdiru, detdirv, 1, 1);
        uvorig  = [detgeo.detrefu, detgeo.detrefv]';

        % Image frame
        plot([0 detusize+1], [0 0], 'k')
        plot([0 detusize+1], [detvsize+1 detvsize+1], 'k')
        plot([0 0], [0 detvsize+1], 'k')
        plot([detusize+1 detusize+1], [0 detvsize+1], 'k')

        % Midlines
        plot([0 detusize+1], [detvsize/2+0.5 detvsize/2+0.5], '-.k')
        plot([detusize/2+0.5 detusize/2+0.5], [0 detvsize+1], '-.k')

        % Set figure propeties
        axis equal
%         set(gca, 'YDir', 'reverse') % ?????
        set(ax, 'Position', [0.05 0.05 0.95 0.9])
%         xlim([-detusize*0.1 detusize*1.1])
%         ylim([-detvsize*0.1 detvsize*1.1])
        xlabel('U direction')
        ylabel('V direction')

        % Beam direction in image
        beamuv = Qdet * beamdir;
        % Arrow in figure indicating beam direction
        quiver(uvorig(1), uvorig(2), beamuv(1), beamuv(2), 1000, '-k', 'Linewidth', 3)

        % Rotation axis direction in image
        rotuv = Qdet*rotdir;
        % Arrow in figure indicating rotation axis direction
        quiver(uvorig(1), uvorig(2), rotuv(1), rotuv(2), 1000, '-.k', 'Linewidth', 3)
    end

    if (app.color)
        ondet = grain.allblobs.detector(1).ondet;
        om = grain.allblobs.omega(ondet);
        uvw = grain.allblobs.detector(1).uvw(ondet);
        cm = inputwdefault(['Choose the type of colormap: omega coloring '...
            '(o) or hkl coloring (h): [o]'], 'o');

        if (strcmp(cm, 'h'))
            disp('Using hkl colormap ...')
            thetatype = grain.allblobs.thetatype(ondet);
            theta = grain.allblobs.theta(ondet);
            thetatype = thetatype(theta <= detgeo.detanglemax/2 ...
                                & theta >= detgeo.detanglemin/2);
            uthetatypes = unique(thetatype);
            cmap_hkl = jet(numel(uthetatypes));
            freezeColors(gca)
            colormap(cmap_hkl);
            c = colorbar;
            xlabel(c, 'theta (deg)')
            caxis([min(theta) max(theta)])

            for it = 1:numel(uthetatypes)
                indx = (thetatype == uthetatypes(it));
                h = scatter3(uvw(indx, 1), uvw(indx, 2), uvw(indx, 3), ...
                    app.markersize, cmap_hkl(it, :), 'fill', 'o');
            end
            impixelinfo

        else
            disp('Using omega colormap ...')
            cmap_omega = jet(1001);
            omtoRGB = @(om) cmap_omega(round((om)/(360)*1000+1), :);
            freezeColors(gca)
            colormap(cmap_omega)
            colorbar
            caxis([0 360])
            impixelinfo
            for iom = 1:numel(om)
                h = scatter3(uvw(iom, 1), uvw(iom, 2), uvw(iom, 3), ...
                    app.markersize, omtoRGB(om(iom)), 'fill', 'o');
            end
        end

    else
        colormap(gray);

    end

    set(h, 'MarkerEdgeColor', [0 0 0])
    view(0, 90)
    xlim([0 detusize])
    ylim([0 detvsize])

    hold(ax, 'off');

    ax_title = sprintf('Fsim for grain %d in phase %d', grain.id(1), grain.phaseid);
    title(ax, ax_title, 'FontWeight', 'bold');
end
