function volume = gtAstra3DTV(proj, varargin)
% GTASTRA3DTV  3D TV-min reconstruction
%     volume = gtAstra3DTV(proj, varargin)
%     ---------------------------------
%
%     INPUT: proj structure containing fields:
%                .stack          stack of projections
%                .geom           projection geometry for Astra
%                                geom is a N x 12 matrix, witn N = #projections
%                                Each row is:
%                                ( rayX, rayY, rayZ, dX, dY, dZ, uX, uY, uZ, vX, vY, vZ )
%                                  ray: the ray direction
%                                  d  : the center of the detector
%                                  u  : the vector from detector pixel (0,0) to (0,1)
%                                  v  : the vector from detector pixel (0,0) to (1,0)
%                .num_rows       vertical size of projection images
%                .num_cols       horizontal size of projection images
%                .num_iter       number of iterations
%                .vol_size_x     x size of output volume
%                .vol_size_y     y size of output volume
%                .vol_size_z     z size of output volume
%
%     INPUT (optional): varargin fields:
%                verbose : {false} | true <logical>
%                is_cone : {false} | true <logical>, for cone beam
%                epsilon : {1e-4} <double>. Tells how much confidence we
%                          have on the projection data. -1 would activate
%                          the standard update of the detector.
%
%     OUTPUT:  volume  <double vol_size_x, vol_size_y, vol_size_z>)
%
%
%     Version 002 26-09-2012 by LNervo
%       Formatting, cleaning, commenting

    conf = struct('verbose', false, 'is_cone', false, 'epsilon', 1e-4);

    conf = parse_pv_pairs(conf, varargin);

    if (~gtCheckGpu())
        error('ASTRA:no_GPU', ...
            'gtAstra3D: Submit this job on a GPU machine ! Exiting...');
    end

    if (isfield(proj, 'padding'))
        padding = proj.padding;
    else
        padding = 0;
    end

    use_astra_mex_direct = (exist('astra_mex_direct_c', 'file') == 3);

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % projection geometry (type, #rows, #columns, vectors)
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if (conf.is_cone)
        proj_geom = astra_create_proj_geom('cone_vec', proj.num_rows, proj.num_cols, proj.geom);
    else
        proj_geom = astra_create_proj_geom('parallel3d_vec', proj.num_rows, proj.num_cols, proj.geom);
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % create volume and projection data objects (x, y, z)
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % proj_data should be a 3d "matrix" of size [#columns,#projections,#rows]
    vol_geom = astra_create_vol_geom(proj.vol_size_x, proj.vol_size_y, proj.vol_size_z);
    if (use_astra_mex_direct)
        astra_projector_id = astra_create_projector('cuda3d', proj_geom, vol_geom);

        fp = @(x)astra_mex_direct_c('FP3D', astra_projector_id, x);
        bp = @(x)astra_mex_direct_c('BP3D', astra_projector_id, x);

        bp_weights = bp(ones(size(proj.stack), 'single'));
        fp_weights = fp(ones(size(bp_weights), 'single'));
    else
        proj_id  = astra_mex_data3d('create', '-proj3d', proj_geom, proj.stack);

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % allocate and zero memory for reconstruction
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        vol_id = astra_mex_data3d('create', '-vol', vol_geom, 0);

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % prepare reconstruction algorithm
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        cfg = astra_struct('FP3D_CUDA');
        cfg.ProjectionDataId = proj_id;
        cfg.VolumeDataId = vol_id;
        alg_fp_id = astra_mex_algorithm('create', cfg);

        cfg = astra_struct('BP3D_CUDA');
        cfg.ProjectionDataId = proj_id;
        cfg.ReconstructionDataId = vol_id;
        alg_bp_id = astra_mex_algorithm('create', cfg);

        fp = @(x)projection(alg_fp_id, proj_id, vol_id, x);
        bp = @(x)projection(alg_bp_id, vol_id, proj_id, x);

        fp_weights = fp(1);
        bp_weights = bp(1);
    end

    volume = zeros(size(bp_weights), class(bp_weights));
    volume_enh = volume;
    dual_tv = cat(4, volume, volume, volume);
    dual_det = zeros(size(fp_weights), class(fp_weights));

    sigma = 1 ./ (fp_weights + (fp_weights == 0));
    sigma_1 = 1 ./ (1 + sigma);
    tau = 1 ./ ((bp_weights + (bp_weights == 0)) + 1);
    theta = 1;
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % reconstruct
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    stats = GtTasksStatistics();
    stats.add_task('fp', 'Forward Projection')
    stats.add_task('bp', 'Back Projection')
    stats.add_task('dual_det_update', 'Update of Dual Detector')
    stats.add_task('dual_grad_update', 'Update of Dual Gradient')
    stats.add_task('primal_update', 'Update of Primal')
    stats.add_task('grad_comp', 'Computing the Gradient')
    stats.add_task('div_comp', 'Computing the Divergence')
    res_initial = gtMathsNorm_l2(proj.stack);
    residuals = [1; zeros(proj.num_iter, 1)];
    fprintf('\b\b: ')
    for ii = 1:proj.num_iter
        char_num = fprintf('%02d/%02d (res: %f)', ii-1, proj.num_iter, residuals(ii));

        stats.tic('fp')
        tomo_fp_tmp = fp(volume_enh);
        stats.toc('fp')
        residuals(ii+1) = sqrt(gtMathsNorm_l2(tomo_fp_tmp - proj.stack) / res_initial);
        stats.tic('dual_det_update')
        if (conf.epsilon >= 0)
            temp_dual_det = dual_det + sigma .* (tomo_fp_tmp - proj.stack);
            temp_dual_denom = temp_dual_det + (temp_dual_det == 0);
            dual_det = temp_dual_det .* max(1 - sigma .* conf.epsilon ./ temp_dual_denom, 0);
        else
            dual_det = (dual_det + sigma .* (tomo_fp_tmp - proj.stack)) .* sigma_1;
        end
        stats.toc('dual_det_update')

        stats.tic('grad_comp')
        d = gradient(volume_enh);
        stats.toc('grad_comp')
        stats.tic('dual_grad_update')
        temp_dual_tv = dual_tv + d;
        dual_tv = temp_dual_tv ./ max(1, abs(temp_dual_tv));
        stats.toc('dual_grad_update')

        stats.tic('bp')
        tomo_bp_tmp = bp(dual_det);
        stats.toc('bp')
        stats.tic('div_comp')
        div_tmp = divergence(dual_tv);
        stats.toc('div_comp')
        stats.tic('primal_update')
        volume_new = volume + (div_tmp - tomo_bp_tmp) .* tau;
        volume_new(volume_new < 0) = 0;
%         volume_new([1:padding, end-padding+1:end], :, :) = 0;
%         volume_new(:, [1:padding, end-padding+1:end], :) = 0;
%         volume_new(:, :, [1:padding, end-padding+1:end]) = 0;

        volume_enh = volume_new + theta .* (volume_new - volume);
        volume = volume_new;
        stats.toc('primal_update')

        fprintf(repmat('\b', [1, char_num]))
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % clean up
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if (use_astra_mex_direct)
        astra_mex_projector3d('delete', astra_projector_id);
    else
        astra_mex_algorithm('delete', alg_fp_id);
        astra_mex_algorithm('delete', alg_bp_id);
        astra_mex_data3d('delete', vol_id);
        astra_mex_data3d('delete', proj_id);
    end

    if (conf.verbose)
        stats.printStats()
        figure, semilogy(residuals)
    end

    volume = volume( ...
        padding+1:end-padding, ...
        padding+1:end-padding, ...
        padding+1:end-padding );
end

function output = projection(algo_id, output_id, input_id, input)
    astra_mex_data3d('store', input_id, input);
    astra_mex_algorithm('iterate', algo_id);
    output = astra_mex_data3d('get_single', output_id);
end

function d = gradient(vol)
    dx = cat(1, vol(2:end, :, :) - vol(1:end-1, :, :), -vol(end, :, :));
    dy = cat(2, vol(:, 2:end, :) - vol(:, 1:end-1, :), -vol(:, end, :));
    dz = cat(3, vol(:, :, 2:end) - vol(:, :, 1:end-1), -vol(:, :, end));
    d = cat(4, dx, dy, dz);
end

function vol = divergence(d)
    dx = d(:, :, :, 1);
    dy = d(:, :, :, 2);
    dz = d(:, :, :, 3);
    ddx = cat(1, dx(1, :, :), dx(2:end, :, :) - dx(1:end-1, :, :));
    ddy = cat(2, dy(:, 1, :), dy(:, 2:end, :) - dy(:, 1:end-1, :));
    ddz = cat(3, dz(:, :, 1), dz(:, :, 2:end) - dz(:, :, 1:end-1));
    vol = ddx + ddy + ddz;
end

% function d = gradient2D(vol)
%     dx = cat(1, vol(1:end-1, :, :) - vol(2:end, :, :), vol(end, :, :));
%     dy = cat(2, vol(:, 1:end-1, :) - vol(:, 2:end, :), vol(:, end, :));
%     d = cat(4, dx, dy, dz);
% end
% 
% function vol = divergence2D(d)
%     dx = d(:, :, :, 1);
%     dy = d(:, :, :, 2);
%     ddx = cat(1, -dx(1, :, :), dx(1:end-1, :, :) - dx(2:end, :, :));
%     ddy = cat(2, -dy(:, 1, :), dy(:, 1:end-1, :) - dy(:, 2:end, :));
%     vol = ddx + ddy;
% end
