function proj = gtAstraPrepareAbsorptionStack(parameters)
% GTASTRAPREPAREABSORPTIONSTACK  Assembles the 3D projection stack of absorption
%                                images and saves them to file absorption_stack.mat
%
%     proj = gtAstraPrepareAbsorptionStack(parameters)
%     ------------------------------------------------
%     INPUT:
%       parameters <structure>
%
%     OUTPUT:
%       proj <structure> with fields:
%         - stack    <double X,X,Z>  3D stack of absorption images
%         - geom     <double Z,12 >  Vector describing Astra preojection geometry
%         - Omega    <Double Z>      Rotation angles in degrees
%
%
%     Version 002.1 26-09-2012 by AKing
%       Change sign of omega angles (used by 2DFBP) when axis is RH [0 0 1]
%
%     Version 002 26-09-2012 by LNervo
%       Formatting, cleaning, commenting

acq = parameters.acq;

labgeo = parameters.labgeo;
samgeo = parameters.samgeo;
recgeo = parameters.recgeo;
detgeo = parameters.detgeo;

if (isfield(parameters.rec, 'absorption'))
    rec_abs = parameters.rec.absorption;
else
    % Old style parameters
    warning('Old "rec" parameters')
    rec_abs = struct(... % can be 'SIRT', '2DFBP' or '3DTV'
        'algorithm', parameters.rec.method, ...
        'num_iter', parameters.rec.abs_num_iter, ...
        'interval', parameters.rec.absinterval, ...
        'padding', 5, ...
        'options', []);
end

ang_incr = pi / acq.nproj;

switch(lower(acq.type))
    case '360degree'
        totproj   = 2 * acq.nproj;
        angle_rad = 0 : ang_incr : 2*pi-ang_incr;
    case '180degree'
        totproj   = acq.nproj;
        angle_rad = pi/2 : ang_incr: 3/2*pi-ang_incr;
    otherwise
        error('Scan type not recognised!')
end

% projection geometry... Here we assume that the absorption images are
% shifted such that the axis of rotation is in the center of the images
% The origin of the sample coordinate system is on the rotation axis - in
% the middle of the illuminated area
fprintf('Building geometry..')
imgs_nums = 0 : rec_abs.interval : totproj-1;
Omega_rad = angle_rad(imgs_nums + 1);
Omega_deg = Omega_rad * 180 / pi;

bbpos = repmat(acq.bb, numel(Omega_deg), []);
geom = gtGeoProjForReconstruction([], Omega_deg', [], bbpos, [], ...
    detgeo, labgeo, samgeo, recgeo, 'ASTRA_absorption');
geom(:, 10:12) = round(geom(:, 10:12));

fprintf('\b\b: Done.\nBuilding sinogram: ');
c = tic();
abs_files_dir = fullfile(acq.dir, '1_preprocessing', 'abs');
info = [];
stack_size = [acq.bb(3), totproj/rec_abs.interval, acq.bb(4)];
stack = zeros(stack_size, 'single');

for jj = 1:numel(imgs_nums)
    num_chars = fprintf('%03d/%03d', jj, numel(imgs_nums));

    if (acq.interlaced_turns == 0)
        absname = fullfile(abs_files_dir, sprintf('abs%04d.edf', imgs_nums(jj)));
    else
        absname = fullfile(abs_files_dir, sprintf('abs_renumbered%04d.edf', imgs_nums(jj)));
    end
    [abs_img, info] = edf_read(absname, [], false, info);

    stack(:, jj, :)  = -log(max(0.01,abs_img))';

    fprintf(repmat('\b', 1, num_chars))
end
fprintf('Done in %f seconds.\n', toc(c));

% deal with the sense of the rotation
if all(labgeo.rotdir == [0 0 1])
    % rotation axis is anti-clockwise
    Omega_rad = -Omega_rad;
end

fprintf('...done!\n')

new_stack_size = stack_size + [2, 0, 2] * rec_abs.padding;
new_stack_shifts = [1, 0, 1] * rec_abs.padding;
stack(stack < 0) = 0;
new_stack = gtPlaceSubVolume(zeros(new_stack_size, 'single'), stack, new_stack_shifts);
new_stack(1:rec_abs.padding, :) = new_stack((rec_abs.padding + 1) * ones(rec_abs.padding, 1), :);
new_stack((end - rec_abs.padding + 1):end, :) = new_stack((end - rec_abs.padding) * ones(rec_abs.padding, 1), :);

proj.padding    = rec_abs.padding;
proj.stack      = new_stack;
proj.Omega      = Omega_rad;
proj.geom       = geom;
proj.num_rows   = new_stack_size(3);
proj.num_cols   = new_stack_size(1);
proj.num_iter   = rec_abs.num_iter;
proj.vol_size_x = new_stack_size(1);
proj.vol_size_y = new_stack_size(1);
proj.vol_size_z = new_stack_size(3);

name = fullfile(acq.dir, 'absorption_stack.mat');
disp(['Writing sinogram in ' name]);
save(name, '-struct', 'proj');

end % end of function
