function [output, rotmat, respos, resang] = gtFindSimilarGrains(g1, g2, parameters1, manual_pair, rotmat)
%find similar grains
%given the grain structure generated by sort_grains, containing positions
%and R-vectors, and the parameters files, containing pixel size data...
% can supply a manual pair of aligned grains to help fix the position

% Extract all the Rodrigues vectors into row vectors
map1.R_vec = gtIndexAllGrainValues(g1, 'R_vector', [], 1, 1:3);
map2.R_vec = gtIndexAllGrainValues(g2, 'R_vector', [], 1, 1:3);

% Extract all the grain centers into row vectors
map1.pos = gtIndexAllGrainValues(g1, 'center', [], 1, 1:3);
map2.pos = gtIndexAllGrainValues(g2, 'center', [], 1, 1:3);

% Get symmetry operators
symm = gtCrystGetSymmetryOperators(parameters1.cryst.crystal_system, ...
                                   parameters1.cryst.spacegroup); 

% thresholds for testing misorientation and distance
angle_test = 10; 
distance_test = 0.1;
sprintf('Using %0.1f degrees and %0.3f (lab unit) as thresholds', ...
    angle_test, distance_test)

% Compute all orientation matrices g on each map
all_g1 = gtMathsRod2OriMat(map1.R_vec.');
all_g2 = gtMathsRod2OriMat(map2.R_vec.');

if exist('rotmat', 'var')
   disp 'Applying the supplied rotation to map 2 orientations and positions' 
   for ii = 1:size(all_g2, 3)
       % orientations as g
       all_g2(:, :, ii) = all_g2(:, :, ii)*rotmat; % postmultiply seems to be the right way
       % update the rod vectors also
       map2.R_vec(ii, :) = gtMathsOriMat2Rod(all_g2(:, :, ii));
       % apply to positions
       map2.pos(ii, :) = map2.pos(ii, :)*rotmat;
   end
end

%try to deal with shifts here.
if ~exist('manual_pair', 'var') || isempty(manual_pair)
    map1.pos=map1.pos-repmat(mean(map1.pos,1), length(map1.pos), 1);
    map2.pos=map2.pos-repmat(mean(map2.pos,1), length(map2.pos), 1);
else
    % we have a manually selected pair of grains
    sprintf('use grains %d (map 1) and %d (map 2) to prealign positions', manual_pair)
    delta = map2.pos(manual_pair(2), :) - map1.pos(manual_pair(1), :);
    map2.pos=map2.pos-repmat(delta, length(map2.pos), 1);
end

% Go through grain map 1, looking for matches in grain map 2
output = [];
dpos = []; % keep the deltas in position
drod = []; % keep the deltas in orientation
for ii = 1:size(all_g1, 3)
    g1 = all_g1(:, :, ii);

    % Calculate misorientations
    misorientation = zeros(size(all_g2, 3), 1);
    for jj = 1:size(all_g2, 3)
        g2 = all_g2(:, :, jj);

        mis_angle = gtDisorientation(g1, g2, symm, 'input', 'orimat');
        % In the case of hexagonal crystals
        %[mis_angle] = gtDisorientation(g1, g2, symm, 'input', 'orimat', 'latticepar', latticepar);
        % misorientation is in degrees! Stay in degrees! 
        misorientation(jj) = mis_angle;
    end

    % Calculate distances
    dif = map2.pos-repmat(map1.pos(ii,:),size(map2.pos,1),1);
    distance = sqrt(sum((dif.^2),2));
    % distance is in lab units (usually mm)
    
    pos=find(distance<distance_test & misorientation<angle_test);
    disp([ii pos']);
    
    % keep all the candidates
    output(ii).possibles=pos;
    output(ii).dif=dif(pos,:);
    output(ii).distance=distance(pos);
    output(ii).misorientations=misorientation(pos);
    
    % choose the best candidate
    if ~isempty(pos)
        test = (distance(pos)/distance_test) + (misorientation(pos)/angle_test);
        [~, bestndx] = min(test);
        % keep the best values, relative to the current alignment
        output(ii).best = pos(bestndx);
        output(ii).best_misorientation = misorientation(pos(bestndx));
        output(ii).best_distance = distance(pos(bestndx));
        % save the data for further analysis
        dpos = [dpos; map1.pos(ii,:)-map2.pos(pos(bestndx),:)];
        drod = [drod; map1.R_vec(ii,:)-map2.R_vec(pos(bestndx),:)];
    end
end

% now we have tried to match all grains
% is there a systematic offset remaining?
% do position first
disp('__________________________')
dposm = median(dpos, 1);
disp(sprintf('global offset in position of %0.3f [%0.3f, %0.3f, %0.3f]', norm(dposm), dposm(1), dposm(2), dposm(3)))
% subtract this
dposres = dpos - repmat(dposm, size(dpos,1), 1);
% compare the magnitudes of the offsets
tmp = zeros(size(drod, 1), 2);
for ii = 1:size(dposres, 1)
    tmp(ii,1) = norm(dpos(ii,:));
    tmp(ii,2) = norm(dposres(ii,:));
end
disp(sprintf('Magnitude position error: mean: %0.3f lab unit, median: %0.3f lab unit', mean(tmp(:,1)), median(tmp(:,1))))
disp(sprintf('After subtracting median: mean: %0.3f lab unit, median: %0.3f lab unit', mean(tmp(:,2)), median(tmp(:,2))))
disp('__________________________')
respos = tmp(:,2); % to return

% rotations
drodm = median(drod, 1);
disp(sprintf('global offset in rodriguez of %0.3f [%0.3f, %0.3f, %0.3f]', norm(drodm), drodm(1), drodm(2), drodm(3)))
disp('The global median rodriguez deviation gives:')
rotmat = gtMathsRod2OriMat(drodm)
% if we apply this, what is left
disp('Applying this and calculating residual disorientations')

dmisres = zeros(size(drod, 1), 2);
count = 1;
for ii = 1:size(all_g1, 3)
    if output(ii).best % if we found a candidate
        g1 = all_g1(:, :, ii);
        g2 = all_g2(:, :, output(ii).best);
        [ang,axis] = gtDisorientation(g1, g2, symm, 'input', 'orimat');
        dmisres(count,1) = ang;
        [ang,axis] = gtDisorientation(g1, g2*rotmat, symm, 'input', 'orimat');
        dmisres(count,2) = ang;
        count = count + 1;
    end
end

disp(sprintf('Magnitude of disorientations: mean: %0.3f deg, median: %0.3f deg', mean(dmisres(:,1)), median(dmisres(:,1))))
disp(sprintf('After applying rotmat: remaining disorientations: mean: %0.3f deg, median: %0.3f deg', mean(dmisres(:,2)), median(dmisres(:,2))))
disp('__________________________')
resang = dmisres(:,2); % to return




end % end of function
