function [gvdm, dmvol, gradok] = gtFedTestGenerateRandomDef(fedpars, nn)

    volsize = fedpars.grainsize;

    dmvol = zeros([volsize nn]);

    gradok = false(nn, 1);

    interp_method = 'spline';
%     interp_method = 'linear';

    if (~isfield(fedpars, 'deftype'))
        fedpars.deftype = 'smooth';
    end

    switch (fedpars.deftype)
        case 'smooth'
            [gvdm, gradok] = make_smooth_def(fedpars, volsize, nn, gradok, interp_method);
        case 'realistic'
            [gvdm, gradok] = make_realistic_def(fedpars, volsize, nn, gradok);
    end
    % Let's center on the average!
    center = mean(gvdm, 2);
    gvdm = gvdm - center(:, ones(size(gvdm, 2), 1));

%     max_angle = gtGetMaxDisorientation(gvdm, [], 'minimal');
%     average_comp = sum(fedpars.dcomps .* fedpars.dapplim) / nn;
%     gvdm = gvdm .* (average_comp / max_angle * 200);
%
%     center = mean(gvdm, 2);
%     gvdm = gvdm - center(:, ones(size(gvdm, 2), 1));

    gtGetMaxDisorientation(gvdm, [], 'diameter_average');
    gtGetMaxDisorientation(gvdm, [], 'diameter_zero');
%     gtGetMaxDisorientation(gvdm, [], 'minimal');

    merged = GtOrientationPostProcessing.tile_theo_deviations(gvdm', volsize);
    GtVolView(merged, 'cmap', jet)

    for ii = 1:nn
        dmvol(:, :, :, ii) = reshape(gvdm(ii, :), volsize);
    end

    disp('Mean of deformation components:')
    disp(mean(gvdm, 2))
    disp('Std of deformation components:')
    disp(std(gvdm, 0, 2))
    disp('Bbox of deformation components:')
    disp([max(gvdm, [], 2)'; min(gvdm, [], 2)'])
end % of function

function [vol, gradok, maxgrad] = smooth_vol(vol, gausssize, gradlim)
    for jj = 1:500
        % Smooth until gradients are small enough
        diffx = diff(vol, 1, 1);
        diffy = diff(vol, 1, 2);
        diffz = diff(vol, 1, 3);

        maxgrad = max(abs([diffx(:); diffy(:); diffz(:)]));

        if (maxgrad <= gradlim)
            gradok = true;
            return;
        end

        vol = smooth3(vol, 'gaussian', gausssize);
    end
    fprintf('Failed to minimize gradient!\n');
    gradok = false;
end

function [gvdm, gradok] = make_smooth_def(fedpars, volsize, nn, gradok, interp_method)

    gausssize = [3 3 3];

    gvdm  = zeros(nn, volsize(1)*volsize(2)*volsize(3));
    gradlim = fedpars.dappgradlim;

    for ii = 1:nn
        switch(fedpars.randdefmode)

            case 'smooth'
                % Random values for all voxels
                vol = randn(volsize) * fedpars.dapplim(ii);

            case 'interp'
                % 3x3x3 grid points on the corners of the grain volume;
                % random strain components will be generated on this grid
                mgvec           = [0, 0.5, 1];
                [mg1, mg2, mg3] = meshgrid(mgvec, mgvec, mgvec);

                % Generate random strain components (+-fedpars.dapplim) on the 3x3x3 grid
                mgvals = -randn(3, 3, 3) * fedpars.dapplim(ii);

                % Grid for all voxels in grain volume.
                % Flip real coordinates 1-2 due to meshgrid !!!!
                [mg1i, mg2i, mg3i] = meshgrid( ...
                    (0.5:volsize(2))/volsize(2), ...
                    (0.5:volsize(1))/volsize(1), ...
                    (0.5:volsize(3))/volsize(3) );

                % interpolation using 'interp_method' method.
                vol = interp3(mg1, mg2, mg3, mgvals, mg1i, mg2i, mg3i, interp_method);

            otherwise
                error('Mode for generating random deformation not recognized.')
        end

        % Shall we really smooth things out?
        [vol, gradok(ii), maxgrad] = smooth_vol(vol, gausssize, gradlim(ii));

        fprintf('Maximum gradient for component %d: %g\n', ii, maxgrad)

        gvdm(ii, :) = vol(:);
    end
end

function [gvdm, gradok] = make_realistic_def(fedpars, volsize, nn, gradok)

    num_seeds = 3;
    % Find Seeds first
    seeds = repmat(volsize', 1, num_seeds);
    seeds = round( (seeds - 1) .* rand(3, num_seeds) ) + 1;
    radiuses = [];
    for ii_s = 1:num_seeds
        for ii_s2 = (ii_s+1):num_seeds
            radiuses(end+1) = norm(seeds(:, ii_s) - seeds(:, ii_s2));
        end
    end
    safe_dist = min(radiuses) / 2;

    prob = cell(3, num_seeds);
    [xx, yy, zz] = meshgrid(1:volsize(1), 1:volsize(2), 1:volsize(3));

    for ii_dim = 1:3
%         polariz_fac = rand() * 2 + 1;
        polariz_fac = rand() * 0.4 + 0.6;
        for ii_s = 1:num_seeds
            prob{ii_dim}{ii_s} =  exp( -( ...
                ( ((xx - seeds(1, ii_s)) .^ 2) ./ ((safe_dist * polariz_fac) .^ 2 * 2) ) + ...
                ( ((yy - seeds(2, ii_s)) .^ 2) ./ ((safe_dist * polariz_fac) .^ 2 * 2) ) + ...
                ( ((zz - seeds(3, ii_s)) .^ 2) ./ ((safe_dist * polariz_fac) .^ 2 * 2) ) ...
                ) );
        end
    end

    % Generate random def components (+-fedpars.dapplim) on the 3x3x3 grid
    mgvals = -randn(3, num_seeds) .* fedpars.dapplim(ones(1, num_seeds), 1:3)';

%     gradlim = fedpars.dappgradlim;
%     gausssize = [3 3 3];

    gvdm = zeros(nn, volsize(1)*volsize(2)*volsize(3));

    for ii_dim = 1:3
        sum_weigths = sum(cat(4, prob{ii_dim}{:}), 4);

        vol = mgvals(ii_dim, 1) .* prob{ii_dim}{1};

        for ii_s = 2:num_seeds
            vol = vol + mgvals(ii_dim, ii_s) .* prob{ii_dim}{ii_s};
        end

        vol = vol./ sum_weigths;

        % Shall we really smooth things out?
%         [vol, gradok(ii_dim), maxgrad] = smooth_vol(vol, gausssize, gradlim(ii_dim));
% 
%         fprintf('Maximum gradient for component %d: %g\n', ii_dim, maxgrad)

        gvdm(ii_dim, :) = vol(:);
    end

    figure;
    hold on
    scatter3(gvdm(1, 1:200:end)', gvdm(2, 1:200:end)', gvdm(3, 1:200:end)', 20, 'c')
    scatter3(mgvals(1, :)', mgvals(2, :)', mgvals(3, :)', 20, 'r')
    hold off

    drawnow();
end



