/*
 * gt6DUpdateDualDetector_c.cpp
 *
 *  Created on: Oct 29, 2014
 *
 * Nicola Vigano', 2014, INSA Lyon / ESRF ID11, vigano@esrf.eu
 */

#include "gt6DUpdatePrimalOps.h"

void mexFunction( int nlhs, mxArray * plhs[], int nrhs, const mxArray * prhs[] )
{
  if (nrhs < 4) {
    mexErrMsgIdAndTxt(GT6D::primal_error_id,
        "Not enough arguments!");
    return;
  }
  if (nlhs != 2) {
    mexErrMsgIdAndTxt(GT6D::primal_error_id,
        "Not enough output!");
    return;
  }

  const mxArray * const curr_sol = prhs[0];
  const mxArray * const curr_enh_sol = prhs[1];
  const mxArray * const corrections = prhs[2];
  const mxArray * const tau = prhs[3];

  if (!mxIsNumeric(curr_sol)) {
    mexErrMsgIdAndTxt(GT6D::primal_error_id,
        "The first argument should be an array (old solution variable)");
    return;
  }
  if (!mxIsNumeric(curr_enh_sol)) {
    mexErrMsgIdAndTxt(GT6D::primal_error_id,
        "The first argument should be an array (old enhanced solution variable)");
    return;
  }
  if (!mxIsNumeric(corrections)) {
    mexErrMsgIdAndTxt(GT6D::primal_error_id,
        "The second argument should be an array (Corrections)");
    return;
  }
  if (!mxIsNumeric(tau)) {
    mexErrMsgIdAndTxt(GT6D::primal_error_id,
        "The fourth argument should be a scalar (tau)");
    return;
  }

  double scale;
  if (mxIsDouble(tau)) {
    scale = *mxGetPr(tau);
  } else if (mxIsSingle(tau)) {
    scale = *(float *)mxGetData(tau);
  } else {
    mexErrMsgIdAndTxt(GT6D::primal_error_id,
        "The tau needs to be of a floating point type");
    return;
  }

  if (nrhs >= 6)
  {
    const mxArray * const num_threads = prhs[5];
    initialize_multithreading(mxGetScalar(num_threads));
  }
  else
  {
    initialize_multithreading();
  }

  if (!GT6D::check_primal_arguments(curr_sol, corrections) ||
      !GT6D::check_primal_arguments(curr_enh_sol, corrections))
  {
    return;
  }

  plhs[0] = mxCreateSharedDataCopy(curr_sol);
  plhs[1] = mxCreateSharedDataCopy(curr_enh_sol);
//  GT6D::primal_allocate_output(plhs[0], plhs[1], curr_sol);

  const mxClassID firstClass = mxGetClassID(curr_sol);
  switch (firstClass)
  {
    case mxDOUBLE_CLASS:
    {
      GT6D::update_primal<double> func(scale);
      GT6D::primal_iteration<double>(plhs[0], plhs[1], curr_sol, corrections, func);
      break;
    }
    case mxSINGLE_CLASS:
    {
      GT6D::update_primal<float> func(scale);
      GT6D::primal_iteration<float>(plhs[0], plhs[1], curr_sol, corrections, func);
      break;
    }
    case mxUNKNOWN_CLASS:
    {
      mexErrMsgIdAndTxt(GT6D::primal_error_id,
          "The argument needs to be a non empty Cell array of coherent floating point types");
      return;
    }
    default:
    {
      mexErrMsgIdAndTxt(GT6D::primal_error_id,
          "The argument needs to be a Cell array of floating point numbers");
      return;
    }
  }
}


