/*
 * gt6DBlobsSinogramsTransforms.h
 *
 *  Created on: Oct 30, 2014
 *      Author: vigano
 */

#ifndef ZUTIL_CXX_INCLUDE_GT6DBLOBSSINOGRAMSTRANSFORMS_H_
#define ZUTIL_CXX_INCLUDE_GT6DBLOBSSINOGRAMSTRANSFORMS_H_

#include "internal_cell_defs.h"

#include <algorithm>
#include <vector>

namespace GT6D {

  const char * transforms_error_id = "C_FUN:gt6DBlobsSinogramsTransforms:wrong_argument";

  static const char error_slice_scaled_num_blob_over_hlim[] = "Requested a blob that is outside the range ('blob_offsets' too large)!";
  static const char error_slice_scaled_num_blob_over_llim[] = "Requested a blob that is outside the range ('blob_offsets' less then 1)!";

  static const char error_slice_scaled_wrong_dest_dims[] = "One of the destination arrays is not 3D";
  static const char error_slice_scaled_wrong_src_dims[] = "One of the source arrays is not 3D";

  static const char error_slice_scaled_wrong_dim_0[] = "Blobs and sinos should have same length along 1st dimension";
  static const char error_slice_scaled_wrong_dim_2[] = "Blobs and sinos should have same length along 3rd dimension";

  static const char error_slice_scaled_sino_slice_over_hlim[] = "One of 'sino_offsets' slices indices for the sino is too large!";
  static const char error_slice_scaled_blob_slice_over_hlim[] = "One of 'proj_offsets' slices indices of the related blob is too large!";

  static const char error_slice_scaled_sino_slice_over_llim[] = "One of 'sino_offsets' slices indices for the sino is less than 1!";
  static const char error_slice_scaled_blob_slice_over_llim[] = "One of 'proj_offsets' slices indices of the related blob is less than 1!";

  inline bool
  check_coeffs(const mxArray * const coeff)
  {
    if (!mxIsStruct(coeff))
    {
      mexErrMsgIdAndTxt(transforms_error_id, "Invalid Coeffs: third argument should be a structure!");
      return false;
    }
    const mxArray * const sino_offsets_a = mxGetField(coeff, 0, "sino_offsets");
    const mxArray * const blob_offsets_a = mxGetField(coeff, 0, "blob_offsets");
    const mxArray * const proj_offsets_a = mxGetField(coeff, 0, "proj_offsets");
    const mxArray * const proj_coeffs_a = mxGetField(coeff, 0, "proj_coeffs");

    if (!sino_offsets_a)
    {
      mexErrMsgIdAndTxt(transforms_error_id, "Invalid Coeffs: 'sino_offsets' is missing!");
      return false;
    }
    if (!blob_offsets_a)
    {
      mexErrMsgIdAndTxt(transforms_error_id, "Invalid Coeffs: 'blob_offsets' is missing!");
      return false;
    }
    if (!proj_offsets_a)
    {
      mexErrMsgIdAndTxt(transforms_error_id, "Invalid Coeffs: 'proj_offsets' is missing!");
      return false;
    }
    if (!proj_coeffs_a)
    {
      mexErrMsgIdAndTxt(transforms_error_id, "Invalid Coeffs: 'proj_coeffs' is missing!");
      return false;
    }

    {
      const mwSize num_elems = mxGetNumberOfElements(sino_offsets_a);
      if (num_elems != mxGetNumberOfElements(blob_offsets_a))
      {
        mexErrMsgIdAndTxt(transforms_error_id, "Invalid Coeffs: number of 'blob_offsets' is different from 'sino_offsets'!");
        return false;
      }
      if (num_elems != mxGetNumberOfElements(proj_offsets_a))
      {
        mexErrMsgIdAndTxt(transforms_error_id, "Invalid Coeffs: number of 'proj_offsets' is different from 'sino_offsets'!");
        return false;
      }
      if (num_elems != mxGetNumberOfElements(proj_coeffs_a))
      {
        mexErrMsgIdAndTxt(transforms_error_id, "Invalid Coeffs: number of 'proj_coeffs' is different from 'sino_offsets'!");
        return false;
      }
    }

    {
      if (mxDOUBLE_CLASS != mxGetClassID(sino_offsets_a))
      {
        mexErrMsgIdAndTxt(transforms_error_id, "Invalid Coeffs: type of 'sino_offsets' should be 'double'!");
        return false;
      }
      if (mxDOUBLE_CLASS != mxGetClassID(blob_offsets_a))
      {
        mexErrMsgIdAndTxt(transforms_error_id, "Invalid Coeffs: type of 'blob_offsets' should be 'double'!");
        return false;
      }
      if (mxDOUBLE_CLASS != mxGetClassID(proj_offsets_a))
      {
        mexErrMsgIdAndTxt(transforms_error_id, "Invalid Coeffs: type of 'proj_offsets' should be 'double'!");
        return false;
      }
      if (mxDOUBLE_CLASS != mxGetClassID(proj_coeffs_a))
      {
        mexErrMsgIdAndTxt(transforms_error_id, "Invalid Coeffs: type of 'proj_coeffs' should be 'double'!");
        return false;
      }
    }

    return true;
  }

  inline bool
  check_blobs_and_single_sinogram(const mxArray * const sino,
      const mxArray * const blobs, const mxArray * const coeff)
  {
    const mxArray * const sino_offsets_a = mxGetField(coeff, 0, "sino_offsets");
    const mxArray * const blob_offsets_a = mxGetField(coeff, 0, "blob_offsets");
    const mxArray * const proj_offsets_a = mxGetField(coeff, 0, "proj_offsets");
    const mxArray * const proj_coeffs_a = mxGetField(coeff, 0, "proj_coeffs");

    const double * const sino_offsets = (const double *) mxGetData(sino_offsets_a);
    const double * const blob_offsets = (const double *) mxGetData(blob_offsets_a);
    const double * const proj_offsets = (const double *) mxGetData(proj_offsets_a);
    const double * const proj_coeffs = (const double *) mxGetData(proj_coeffs_a);

    const mwSize num_dims_sino = mxGetNumberOfDimensions(sino);
    if (num_dims_sino != 3)
    {
      mexErrMsgIdAndTxt(transforms_error_id, error_slice_scaled_wrong_dest_dims);
      return false;
    }
    const mwSize * const dims_sino = mxGetDimensions(sino);

    const mwSize num_projs = mxGetNumberOfElements(proj_offsets_a);
    for (mwIndex m = 0; m < num_projs; m++)
    {
      const mwIndex p = (const mwIndex) proj_offsets[m]-1;
      const mwIndex b = (const mwIndex) blob_offsets[m]-1;
      const mwIndex s = (const mwIndex) sino_offsets[m]-1;

      if (b >= mxGetNumberOfElements(blobs))
      {
        mexErrMsgIdAndTxt(transforms_error_id, error_slice_scaled_num_blob_over_hlim);
        return false;
      }

      if (b < 0)
      {
        mexErrMsgIdAndTxt(transforms_error_id, error_slice_scaled_num_blob_over_llim);
        return false;
      }

      const mxArray * const cell_blobs = mxGetCell(blobs, b);

      const mwSize num_dims_blob = mxGetNumberOfDimensions(cell_blobs);
      if (num_dims_blob != 3)
      {
        mexErrMsgIdAndTxt(transforms_error_id, error_slice_scaled_wrong_src_dims);
        return false;
      }

      // Sino and Blob have the same size along 1st and 3rd dimensions
      const mwSize * const dims_blob = mxGetDimensions(cell_blobs);

      if (dims_blob[0] != dims_sino[0])
      {
        mexErrMsgIdAndTxt(transforms_error_id, error_slice_scaled_wrong_dim_0);
        return false;
      }
      if (dims_blob[2] != dims_sino[2])
      {
        mexErrMsgIdAndTxt(transforms_error_id, error_slice_scaled_wrong_dim_2);
        return false;
      }

      if (s >= dims_sino[1])
      {
        mexErrMsgIdAndTxt(transforms_error_id, error_slice_scaled_sino_slice_over_hlim);
        return false;
      }
      if (p >= dims_blob[1])
      {
        mexErrMsgIdAndTxt(transforms_error_id, error_slice_scaled_blob_slice_over_hlim);
        return false;
      }
      if (s < 0)
      {
        mexErrMsgIdAndTxt(transforms_error_id, error_slice_scaled_sino_slice_over_llim);
        return false;
      }
      if (p < 0)
      {
        mexErrMsgIdAndTxt(transforms_error_id, error_slice_scaled_blob_slice_over_llim);
        return false;
      }
    }
    return true;
  }

  inline bool
  check_full_incoming_args_blobs_and_sino(int nrhs, const mxArray * prhs[] )
  {
    if (nrhs != 3) {
      mexErrMsgIdAndTxt(transforms_error_id,
          "Not enough arguments!");
      return false;
    }

    GT6D::check_coeffs(prhs[2]);

    if (!mxIsCell(prhs[0])) {
      mexErrMsgIdAndTxt(transforms_error_id,
          "The first argument should be a Cell array (Blobs)");
      return false;
    }
    if (!mxIsNumeric(prhs[1])) {
      mexErrMsgIdAndTxt(transforms_error_id,
          "The second argument should be an array (Sinogram)");
      return false;
    }

    GT6D::check_blobs_and_single_sinogram(prhs[1], prhs[0], prhs[2]);

    return true;
  }

////////////////////////////////////////////////////////////////////////////////

  template<typename Type>
  inline void
  sino_initialize_sse(mxArray * const array, const Type & val)
  {
    typedef typename SIMDUnrolling<Type>::vVvf vVvf;

    const mwSize unrolling = 8;
    const SIMDUnrolling<Type> simd_unroll(unrolling);
    const SIMDUnrolling<Type> simd(1);

    const vVvf vec_val = Coeff<Type>::get(val);

    AccessAligned<Type> access;

    Type * __restrict data = (Type *) mxGetData(array);
    const mwSize num_elems = mxGetNumberOfElements(array);

  #pragma omp for nowait
    for (mwIndex elem_count = 0; elem_count < simd_unroll.get_unroll(num_elems);
        elem_count += simd_unroll.block)
    {
      access.store(&data[elem_count + 0 * simd_unroll.shift], vec_val);
      access.store(&data[elem_count + 1 * simd_unroll.shift], vec_val);
      access.store(&data[elem_count + 2 * simd_unroll.shift], vec_val);
      access.store(&data[elem_count + 3 * simd_unroll.shift], vec_val);

      access.store(&data[elem_count + 4 * simd_unroll.shift], vec_val);
      access.store(&data[elem_count + 5 * simd_unroll.shift], vec_val);
      access.store(&data[elem_count + 6 * simd_unroll.shift], vec_val);
      access.store(&data[elem_count + 7 * simd_unroll.shift], vec_val);
    }
  #pragma omp for nowait
    for (mwIndex elem_count = simd_unroll.get_unroll(num_elems);
        elem_count < simd.get_unroll(num_elems); elem_count += simd.block)
    {
      access.store(&data[elem_count], vec_val);
    }
  #pragma omp for nowait
    for (mwIndex elem_count = simd.get_unroll(num_elems); elem_count < num_elems; elem_count++)
    {
      data[elem_count] = val;
    }
  }

  inline mxArray *
  allocate_sino_from_dims(const mxArray * const mxdims, const mxClassID & type)
  {
    const mwSize num_dims = mxGetNumberOfElements(mxdims);
    std::vector<mwSize> dims(num_dims);

    switch(mxGetClassID(mxdims))
    {
      case mxDOUBLE_CLASS:
      {
        const double * dims_d = (double *) mxGetData(mxdims);
        for(mwIndex dim = 0; dim < num_dims; dim++)
        {
          dims[dim] = (mwSize) dims_d[dim];
        }
        break;
      }
      case mxSINGLE_CLASS:
      {
        const float * dims_s = (float *) mxGetData(mxdims);
        for(mwIndex dim = 0; dim < num_dims; dim++)
        {
          dims[dim] = (mwSize) dims_s[dim];
        }
        break;
      }
      default:
      {
        mexErrMsgIdAndTxt(transforms_error_id,
            "Sino dims should be either 'single' or 'double' precision floating points");
        return NULL;
      }
    }

    mxArray * out_array = vol_allocate(num_dims, &dims[0], type);

    // Let's initialize the
    switch(type)
    {
      case mxDOUBLE_CLASS:
      {
#pragma omp parallel
        sino_initialize_sse(out_array, 0.0);
        break;
      }
      case mxSINGLE_CLASS:
      {
#pragma omp parallel
        sino_initialize_sse(out_array, 0.0f);
        break;
      }
      default:
      {
        mexErrMsgIdAndTxt(transforms_error_id,
            "Sinograms should be either 'single' or 'double' precision floating points");
        return NULL;
      }
    }

    return out_array;
  }

////////////////////////////////////////////////////////////////////////////////

  template<typename Type>
  inline void
  do_blobs_to_single_sinogram(mxArray * sino, const mxArray * const blobs,
      const mxArray * const coeffs)
  {
    const mxArray * const sino_offsets_a = mxGetField(coeffs, 0, "sino_offsets");
    const mxArray * const blob_offsets_a = mxGetField(coeffs, 0, "blob_offsets");
    const mxArray * const proj_offsets_a = mxGetField(coeffs, 0, "proj_offsets");
    const mxArray * const proj_coeffs_a = mxGetField(coeffs, 0, "proj_coeffs");

    const double * const sino_offsets = (const double *) mxGetData(sino_offsets_a);
    const double * const blob_offsets = (const double *) mxGetData(blob_offsets_a);
    const double * const proj_offsets = (const double *) mxGetData(proj_offsets_a);
    const double * const proj_coeffs = (const double *) mxGetData(proj_coeffs_a);

    const mwSize * const dims_sino = mxGetDimensions(sino);
    const mwSize & line_length = dims_sino[0];
    const mwSize & lines_num = dims_sino[2];
    Type * const sino_data = ((Type *) mxGetData(sino));

    const mwSize sino_skip = dims_sino[0] * dims_sino[1];

    const mwSize & num_projs = mxGetNumberOfElements(proj_offsets_a);

    std::vector<const Type *> blob_data_bases(num_projs);
    std::vector<Type *> sino_data_bases(num_projs);
    std::vector<mwSize> blob_skips(num_projs);
    std::vector<Type> coeffs_t(num_projs);
    for (mwIndex m = 0; m < num_projs; m++)
    {
      const mwIndex p = (const mwIndex) proj_offsets[m]-1;
      const mwIndex b = (const mwIndex) blob_offsets[m]-1;
      const mwIndex s = (const mwIndex) sino_offsets[m]-1;

      coeffs_t[m] = (const Type) proj_coeffs[m];

      const mwSize sino_initial_offset = line_length * s;

      const mxArray * const cell_blobs = mxGetCell(blobs, b);
      sino_data_bases[m] = sino_data + sino_initial_offset;

      const mwSize blob_initial_offset = line_length * p;
      blob_data_bases[m] = ((const Type *) mxGetData(cell_blobs)) + blob_initial_offset;

      const mwSize * const dims_blob = mxGetDimensions(cell_blobs);
      blob_skips[m] = dims_blob[0] * dims_blob[1];
    }

#pragma omp parallel
    {
      const AccessUnaligned<Type> access;
      const SIMDUnrolling<Type> simd4(4);
      const SIMDUnrolling<Type> simd(1);
      const mwSize line_length_unroll4 = simd4.get_unroll(line_length);
      const mwSize line_length_unroll = simd.get_unroll(line_length);

      typedef typename SIMDUnrolling<Type>::vVvf vVvf;

#pragma omp for
      for (mwIndex line_num = 0; line_num < lines_num; line_num++)
      {
        const mwSize line_sino_skip = line_num * sino_skip;

        for (mwIndex m = 0; m < num_projs; m++)
        {
          const Type & c = coeffs_t[m];
          inner_sum_scale<Type> op(c);

          const Type * const blob_data_line = blob_data_bases[m] + line_num * blob_skips[m];
          Type * const sino_data_line = sino_data_bases[m] + line_sino_skip;

          for (mwIndex elem = 0; elem < line_length_unroll4; elem += simd4.block)
          {
            Type * const sino_line = sino_data_line + elem;
            const Type * const blob_line = blob_data_line + elem;

            const vVvf & a0 = access.load(sino_line + 0 * simd4.shift);
            const vVvf & a1 = access.load(sino_line + 1 * simd4.shift);
            const vVvf & a2 = access.load(sino_line + 2 * simd4.shift);
            const vVvf & a3 = access.load(sino_line + 3 * simd4.shift);

            const vVvf & b0 = access.load(blob_line + 0 * simd4.shift);
            const vVvf & b1 = access.load(blob_line + 1 * simd4.shift);
            const vVvf & b2 = access.load(blob_line + 2 * simd4.shift);
            const vVvf & b3 = access.load(blob_line + 3 * simd4.shift);

            const vVvf c0 = op(a0, b0);
            const vVvf c1 = op(a1, b1);
            const vVvf c2 = op(a2, b2);
            const vVvf c3 = op(a3, b3);

            access.store(sino_line + 0 * simd4.shift, c0);
            access.store(sino_line + 1 * simd4.shift, c1);
            access.store(sino_line + 2 * simd4.shift, c2);
            access.store(sino_line + 3 * simd4.shift, c3);
          }
          for (mwIndex elem = line_length_unroll4; elem < line_length_unroll; elem += simd.block)
          {
            const vVvf & a0 = access.load(sino_data_line + elem);
            const vVvf & b0 = access.load(blob_data_line + elem);
            const vVvf c0 = op(a0, b0);
            access.store(sino_data_line + elem, c0);
          }
          for (mwIndex elem = line_length_unroll; elem < line_length; elem++)
          {
            sino_data_line[elem] = op(sino_data_line[elem], blob_data_line[elem]);
          }
        }
      }
    }
  }

  template<typename Type>
  inline void
  do_single_sinogram_to_blobs(mxArray * blobs, const mxArray * const sino,
      const mxArray * const coeffs)
  {
    const mxArray * const sino_offsets_a = mxGetField(coeffs, 0, "sino_offsets");
    const mxArray * const blob_offsets_a = mxGetField(coeffs, 0, "blob_offsets");
    const mxArray * const proj_offsets_a = mxGetField(coeffs, 0, "proj_offsets");
    const mxArray * const proj_coeffs_a = mxGetField(coeffs, 0, "proj_coeffs");

    const double * const sino_offsets = (const double *) mxGetData(sino_offsets_a);
    const double * const blob_offsets = (const double *) mxGetData(blob_offsets_a);
    const double * const proj_offsets = (const double *) mxGetData(proj_offsets_a);
    const double * const proj_coeffs = (const double *) mxGetData(proj_coeffs_a);

    const mwSize * const dims_sino = mxGetDimensions(sino);
    const mwSize & line_length = dims_sino[0];
    const mwSize & lines_num = dims_sino[2];
    const Type * const sino_data = ((const Type *) mxGetData(sino));

    const mwSize sino_skip = dims_sino[0] * dims_sino[1];

    const mwSize & num_projs = mxGetNumberOfElements(proj_offsets_a);

    std::vector<Type *> blob_data_bases(num_projs);
    std::vector<const Type *> sino_data_bases(num_projs);
    std::vector<mwSize> blob_skips(num_projs);
    std::vector<Type> coeffs_t(num_projs);
    for (mwIndex m = 0; m < num_projs; m++)
    {
      const mwIndex p = (const mwIndex) proj_offsets[m]-1;
      const mwIndex b = (const mwIndex) blob_offsets[m]-1;
      const mwIndex s = (const mwIndex) sino_offsets[m]-1;

      coeffs_t[m] = (const Type) proj_coeffs[m];

      const mwSize sino_initial_offset = line_length * s;

      const mxArray * const cell_blobs = mxGetCell(blobs, b);
      sino_data_bases[m] = sino_data + sino_initial_offset;

      const mwSize blob_initial_offset = line_length * p;
      blob_data_bases[m] = ((Type *) mxGetData(cell_blobs)) + blob_initial_offset;

      const mwSize * const dims_blob = mxGetDimensions(cell_blobs);
      blob_skips[m] = dims_blob[0] * dims_blob[1];
    }

#pragma omp parallel
    {
      const AccessUnaligned<Type> access;
      const SIMDUnrolling<Type> simd4(4);
      const SIMDUnrolling<Type> simd(1);
      const mwSize line_length_unroll4 = simd4.get_unroll(line_length);
      const mwSize line_length_unroll = simd.get_unroll(line_length);

      typedef typename SIMDUnrolling<Type>::vVvf vVvf;

#pragma omp for
      for (mwIndex line_num = 0; line_num < lines_num; line_num++)
      {
        const mwSize line_sino_skip = line_num * sino_skip;

        for (mwIndex m = 0; m < num_projs; m++)
        {
          const Type & c = coeffs_t[m];
          inner_sum_scale<Type> op(c);

          Type * const blob_data_line = blob_data_bases[m] + line_num * blob_skips[m];
          const Type * const sino_data_line = sino_data_bases[m] + line_sino_skip;

          for (mwIndex elem = 0; elem < line_length_unroll4; elem += simd4.block)
          {
            const Type * const sino_line = sino_data_line + elem;
            Type * const blob_line = blob_data_line + elem;

            const vVvf & a0 = access.load(blob_line + 0 * simd4.shift);
            const vVvf & a1 = access.load(blob_line + 1 * simd4.shift);
            const vVvf & a2 = access.load(blob_line + 2 * simd4.shift);
            const vVvf & a3 = access.load(blob_line + 3 * simd4.shift);

            const vVvf & b0 = access.load(sino_line + 0 * simd4.shift);
            const vVvf & b1 = access.load(sino_line + 1 * simd4.shift);
            const vVvf & b2 = access.load(sino_line + 2 * simd4.shift);
            const vVvf & b3 = access.load(sino_line + 3 * simd4.shift);

            const vVvf c0 = op(a0, b0);
            const vVvf c1 = op(a1, b1);
            const vVvf c2 = op(a2, b2);
            const vVvf c3 = op(a3, b3);

            access.store(blob_line + 0 * simd4.shift, c0);
            access.store(blob_line + 1 * simd4.shift, c1);
            access.store(blob_line + 2 * simd4.shift, c2);
            access.store(blob_line + 3 * simd4.shift, c3);
          }
          for (mwIndex elem = line_length_unroll4; elem < line_length_unroll; elem += simd.block)
          {
            const vVvf & a = access.load(blob_data_line + elem);
            const vVvf & b = access.load(sino_data_line + elem);
            const vVvf c = op(a, b);
            access.store(blob_data_line + elem, c);
          }
          for (mwIndex elem = line_length_unroll; elem < line_length; elem++)
          {
            blob_data_line[elem] = op(blob_data_line[elem], sino_data_line[elem]);
          }
        }
      }
    }
  }


////////////////////////////////////////////////////////////////////////////////

  template<typename Type>
  struct ProjCoeffs {
    Type * sino_data_base;
    const Type * blob_data_base;
    mwSize blob_skip;
    std::vector<mwSize> offsets;
    std::vector<Type> coeffs;
  };

  template<typename Type>
  inline void
  do_blobs_to_single_sinogram_struct(mxArray * sino, const mxArray * const blobs,
      const mxArray * const coeffs)
  {
    const mxArray * const proj_struct = mxGetField(coeffs, 0, "proj");
    const mwSize num_slices = mxGetNumberOfElements(proj_struct);

    const mwSize * const dims_sino = mxGetDimensions(sino);
    const mwSize & line_length = dims_sino[0];
    const mwSize & lines_num = dims_sino[2];
    const mwSize line_length_unroll = ROUND_DOWN(line_length, 4);

    const mwSize sino_skip = dims_sino[0] * dims_sino[1];

    Type * const sino_data_base = ((Type *) mxGetData(sino));

    std::vector<ProjCoeffs<Type> > coeffs_str(num_slices);
    for (mwIndex m = 0; m < num_slices; m++)
    {
      const mxArray * const sino_offset_a = mxGetField(proj_struct, m, "sino_offset");
      const mxArray * const blob_offset_a = mxGetField(proj_struct, m, "blob_offset");
      const mxArray * const proj_offsets_a = mxGetField(proj_struct, m, "proj_offsets");
      const mxArray * const proj_coeffs_a = mxGetField(proj_struct, m, "proj_coeffs");

      const double * const proj_offsets = (const double *) mxGetData(proj_offsets_a);
      const double * const proj_coeffs = (const double *) mxGetData(proj_coeffs_a);

      const mwIndex s = mxGetScalar(sino_offset_a) - 1;
      coeffs_str[m].sino_data_base = s * line_length + sino_data_base;

      const mwIndex b = mxGetScalar(blob_offset_a) - 1;
      const mxArray * const cell_blobs = mxGetCell(blobs, b);
      const mwSize * const dims_blob = mxGetDimensions(cell_blobs);
      coeffs_str[m].blob_skip = dims_blob[0] * dims_blob[1];
      coeffs_str[m].blob_data_base = ((const Type *) mxGetData(cell_blobs));

      const mwSize num_projs = mxGetNumberOfElements(proj_offsets_a);
      coeffs_str[m].offsets.resize(num_projs);
      coeffs_str[m].coeffs.resize(num_projs);

      for (mwIndex i = 0; i < num_projs; i++)
      {
        coeffs_str[m].offsets[i] = (const mwIndex) proj_offsets[i]-1;
        coeffs_str[m].coeffs[i] = (const Type) proj_coeffs[i];
      }
    }

#pragma omp parallel
    {
      register Type buff[line_length] __attribute__((aligned(MEMORY_ALIGN)));

#pragma omp for
    for (mwIndex line_num = 0; line_num < lines_num; line_num++)
    {
      const mwSize line_sino_skip = line_num * sino_skip;

      for (mwIndex m = 0; m < num_slices; m++)
      {
        const Type * const blob_data_base = coeffs_str[m].blob_data_base + line_num * coeffs_str[m].blob_skip;

        for (mwIndex elem = 0; elem < line_length_unroll; elem += 4)
        {
          buff[elem+0] = 0;
          buff[elem+1] = 0;
          buff[elem+2] = 0;
          buff[elem+3] = 0;
        }
        for (mwIndex elem = line_length_unroll; elem < line_length; elem++)
        {
          buff[elem] = 0;
        }

        const mwSize & num_projs = coeffs_str[m].offsets.size();
        for (mwIndex i = 0; i < num_projs; i++)
        {
          const mwIndex & p = coeffs_str[m].offsets[i];
          const Type & c = coeffs_str[m].coeffs[i];

          inner_sum_scale<Type> op(c);

          const mwSize blob_initial_offset = line_length * p;
          const Type * const blob_data_line = blob_data_base + blob_initial_offset;

          for (mwIndex elem = 0; elem < line_length_unroll; elem += 4)
          {
            buff[elem+0] = op(buff[elem+0], blob_data_line[elem+0]);
            buff[elem+1] = op(buff[elem+1], blob_data_line[elem+1]);
            buff[elem+2] = op(buff[elem+2], blob_data_line[elem+2]);
            buff[elem+3] = op(buff[elem+3], blob_data_line[elem+3]);
          }
          for (mwIndex elem = line_length_unroll; elem < line_length; elem++)
          {
            buff[elem] = op(buff[elem], blob_data_line[elem]);
          }
        }

        Type * const sino_data_line = coeffs_str[m].sino_data_base + line_sino_skip;

        for (mwIndex elem = 0; elem < line_length_unroll; elem += 4)
        {
          sino_data_line[elem+0] = buff[elem+0];
          sino_data_line[elem+1] = buff[elem+1];
          sino_data_line[elem+2] = buff[elem+2];
          sino_data_line[elem+3] = buff[elem+3];
        }
        for (mwIndex elem = line_length_unroll; elem < line_length; elem++)
        {
          sino_data_line[elem] = buff[elem];
        }
      }
    }
    }
  }

};

#endif /* ZUTIL_CXX_INCLUDE_GT6DBLOBSSINOGRAMSTRANSFORMS_H_ */
