/*
 * gt6DOps.h
 *
 *  Created on: Oct 29, 2014
 *      Author: vigano
 */

#ifndef ZUTIL_CXX_INCLUDE_GT6DOPS_H_
#define ZUTIL_CXX_INCLUDE_GT6DOPS_H_

#include "internal_cell_defs.h"

#include <algorithm>

namespace GT6D {

#define APPLY_FUNC_4FOLD_PRIMAL(shift_val) \
{ \
  vVvf inV11 = access.load(out1 + shift_val * simd_8.shift);\
  vVvf inV21 = access.load(out2 + shift_val * simd_8.shift);\
  const vVvf inV31 = access.load(in3 + shift_val * simd_8.shift);\
  const vVvf inV41 = access.load(in4 + shift_val * simd_8.shift);\
  func(inV11, inV21, inV31, inV41);\
  access.store(out1 + shift_val * simd_8.shift, inV11);\
  access.store(out2 + shift_val * simd_8.shift, inV21);\
}

  const char * primal_error_id = "C_FUN:gt6DUpdatePrimal:wrong_argument";


  template<typename Type>
  class update_primal {
  public:
    typedef typename SIMDUnrolling<Type>::vVvf vVvf;

    update_primal(const Type & _tau) : tau(_tau), tau_v(Coeff<Type>::get(_tau)) { }

    void
    operator()(Type & new_solution, Type & new_enh_solution,
        const Type & old_solution, const Type & correction_tomo) const throw()
    {

      new_solution = non_neg(old_solution + correction_tomo * tau, Type());
      new_enh_solution = new_solution + (new_solution - old_solution);
    }
    void
    operator()(vVvf & new_solution, vVvf & new_enh_solution,
        const vVvf & old_solution, const vVvf & correction_tomo) const throw()
    {
      new_solution = non_neg(old_solution + correction_tomo * tau_v, vVvf());
      new_enh_solution = new_solution + (new_solution - old_solution);
    }
  protected:
    const Type tau;
    const vVvf tau_v;

    inner_non_neg<Type> non_neg;
  };

  inline bool
  check_primal_arguments(const mxArray * const mat_old_solution,
      const mxArray * const mat_correction_tomo)
  {
    const mxClassID type = mxGetClassID(mat_old_solution);
    const mwSize num_elems = mxGetNumberOfElements(mat_old_solution);

    if (type != mxGetClassID(mat_correction_tomo))
    {
      mexErrMsgIdAndTxt(primal_error_id,
              "tomo correction should have the same type as the Primal");
      return false;
    }

    if (num_elems != mxGetNumberOfElements(mat_correction_tomo))
    {
      mexErrMsgIdAndTxt(primal_error_id,
              "tomo correction should have the same number of elements as the Primal");
      return false;
    }

    return true;
  }

  template<typename Type, typename Function, class AccessType>
  inline void
  primal_inner_cycle_sse(Type * const __restrict out_data1,
      Type * const __restrict out_data2,
      const Type * const __restrict in_data3,
      const Type * const __restrict in_data4,
      const mwSize & num_elems, Function & func)
  {
    typedef typename SIMDUnrolling<Type>::vVvf vVvf;

    const SIMDUnrolling<Type> simd_8(8);
    const SIMDUnrolling<Type> simd_1(1);

    const mwSize num_elems_unroll_8 = simd_8.get_unroll(num_elems);
    const mwSize num_elems_unroll_1 = simd_1.get_unroll(num_elems);

    AccessType access;

  #pragma omp for nowait
    for(mwIndex elemIdx = 0; elemIdx < num_elems_unroll_8; elemIdx += simd_8.block)
    {
      Type * const out1 = out_data1 + elemIdx;
      Type * const out2 = out_data2 + elemIdx;
      const Type * const in3 = in_data3 + elemIdx;
      const Type * const in4 = in_data4 + elemIdx;

      APPLY_FUNC_4FOLD_PRIMAL(0);
      APPLY_FUNC_4FOLD_PRIMAL(1);
      APPLY_FUNC_4FOLD_PRIMAL(2);
      APPLY_FUNC_4FOLD_PRIMAL(3);
      APPLY_FUNC_4FOLD_PRIMAL(4);
      APPLY_FUNC_4FOLD_PRIMAL(5);
      APPLY_FUNC_4FOLD_PRIMAL(6);
      APPLY_FUNC_4FOLD_PRIMAL(7);
    }
  #pragma omp for nowait
    for(mwIndex elemIdx = num_elems_unroll_8; elemIdx < num_elems_unroll_1; elemIdx += simd_1.block)
    {
      vVvf inV11 = access.load(&out_data1[elemIdx]);
      vVvf inV21 = access.load(&out_data2[elemIdx]);

      const vVvf inV31 = access.load(&in_data3[elemIdx]);
      const vVvf inV41 = access.load(&in_data4[elemIdx]);

      func(inV11, inV21, inV31, inV41);

      access.store(&out_data1[elemIdx], inV11);
      access.store(&out_data2[elemIdx], inV21);
    }
  #pragma omp for nowait
    for(mwIndex elemIdx = num_elems_unroll_1; elemIdx < num_elems; elemIdx++)
    {
       func(out_data1[elemIdx], out_data2[elemIdx], in_data3[elemIdx], in_data4[elemIdx]);
    }
  }

  template<typename Type, typename Function>
  inline void
  primal_iteration(mxArray * mat_new_solution,
      mxArray * mat_new_enh_solution,
      const mxArray * const mat_old_solution,
      const mxArray * const mat_correction_tomo,
      const Function & func)
  {
    Type * const new_solution = (Type *) mxGetData(mat_new_solution);
    Type * const new_enh_solution = (Type *) mxGetData(mat_new_enh_solution);
    const Type * const old_solution = (const Type *) mxGetData(mat_old_solution);
    const Type * const corr_tomo = (const Type *) mxGetData(mat_correction_tomo);

    const mwSize num_elems = mxGetNumberOfElements(mat_new_solution);

  #pragma omp parallel
    {
      primal_inner_cycle_sse< Type, const Function, AccessAligned<Type> >(
          new_solution, new_enh_solution, old_solution, corr_tomo, num_elems, func);
    }
  }

  inline void
  primal_allocate_output(mxArray * & mat_new_solution,
      mxArray * & mat_new_enh_solution,
      const mxArray * const mat_old_solution)
  {
    const mxClassID type = mxGetClassID(mat_old_solution);
    const mwSize numDims = mxGetNumberOfDimensions(mat_old_solution);
    const mwSize * dims = mxGetDimensions(mat_old_solution);

    mat_new_solution = mxCreateSharedDataCopy(mat_old_solution);
    mat_new_enh_solution = vol_allocate(numDims, dims, type);
  }
};

#endif /* ZUTIL_CXX_INCLUDE_GT6DOPS_H_ */
