/*
 * gtCxxMathsSumCellVolumes.h
 *
 *  Created on: 05 mag 2016
 *      Author: nicola
 */

#ifndef ZUTIL_CXX_INCLUDE_GTCXXMATHSSUMCELLVOLUMES_H_
#define ZUTIL_CXX_INCLUDE_GTCXXMATHSSUMCELLVOLUMES_H_

#include "internal_cell_defs.h"

namespace GT_MATHS {

const char * reduce_volumes_error_id = "gtCxxMathsSumCellVolumes:wrong_argument";

template<typename Type>
void
reduce_volumes(mxArray * const mxOutput, const mxArray * const mxInputCell)
{
  const mwSize num_elements = mxGetNumberOfElements(mxOutput);

  // Deciding the cache size: we want to use maximum 16KB per thread, since
  // latest processors all have 64KB L1 cache size
  const mwSize buffer_size = 16 * 1024 / sizeof(Type);

  const SIMDUnrolling<Type> simd_4(4);

  typedef typename SIMDUnrolling<Type>::vVvf vVvf;

  // Output pointer
  Type * output = (Type *)mxGetData(mxOutput);

  // Taking input pointers
  const mwSize num_cells = mxGetNumberOfElements(mxInputCell);
  std::vector<const Type *> inputs(num_cells);
  for (mwIndex icell = 0; icell < num_cells; icell++)
  {
    const mxArray * mxCell = mxGetCell(mxInputCell, icell);
    inputs[icell] = (const Type *)mxGetData(mxCell);
  }

  const AccessAligned<Type> access;
  const inner_sum<Type> add_op;

#pragma omp parallel for
  for (mwIndex ibase = 0; ibase < num_elements; ibase += buffer_size)
  {
    const mwSize end_of_buffer = std::min(buffer_size, num_elements - ibase);
    const mwSize end_of_unrolled_buffer = simd_4.get_unroll(end_of_buffer);

    register Type buffer[end_of_buffer];

    const Type * const first_cell_data = inputs[0] + ibase;

    // First we initialize with the first volume
    for (mwIndex icache = 0; icache < end_of_unrolled_buffer; icache += simd_4.block)
    {
      const Type * const base_cache_cell = first_cell_data + icache;
      const vVvf & in0 = access.load(base_cache_cell + 0 * simd_4.shift);
      const vVvf & in1 = access.load(base_cache_cell + 1 * simd_4.shift);
      const vVvf & in2 = access.load(base_cache_cell + 2 * simd_4.shift);
      const vVvf & in3 = access.load(base_cache_cell + 3 * simd_4.shift);

      Type * __restrict const base_buff = buffer + icache;
      access.store(base_buff + 0 * simd_4.shift, in0);
      access.store(base_buff + 1 * simd_4.shift, in1);
      access.store(base_buff + 2 * simd_4.shift, in2);
      access.store(base_buff + 3 * simd_4.shift, in3);
    }
    for (mwIndex icache = end_of_unrolled_buffer; icache < end_of_buffer; icache++)
    {
      buffer[icache] = first_cell_data[icache];
    }

    // Now we start summing each other volume
    for (mwIndex icell = 1; icell < num_cells; icell++)
    {
      const Type * const cell_data = inputs[icell] + ibase;

      // First we initialize with the first volume
      for (mwIndex icache = 0; icache < end_of_unrolled_buffer; icache += simd_4.block)
      {
        const Type * const base_cache_cell = cell_data + icache;
        const vVvf & in0_i = access.load(base_cache_cell + 0 * simd_4.shift);
        const vVvf & in1_i = access.load(base_cache_cell + 1 * simd_4.shift);
        const vVvf & in2_i = access.load(base_cache_cell + 2 * simd_4.shift);
        const vVvf & in3_i = access.load(base_cache_cell + 3 * simd_4.shift);

        Type * __restrict const base_buff = buffer + icache;
        const vVvf & in0_b = access.load(base_buff + 0 * simd_4.shift);
        const vVvf & in1_b = access.load(base_buff + 1 * simd_4.shift);
        const vVvf & in2_b = access.load(base_buff + 2 * simd_4.shift);
        const vVvf & in3_b = access.load(base_buff + 3 * simd_4.shift);

        access.store(base_buff + 0 * simd_4.shift, add_op(in0_b, in0_i));
        access.store(base_buff + 1 * simd_4.shift, add_op(in1_b, in1_i));
        access.store(base_buff + 2 * simd_4.shift, add_op(in2_b, in2_i));
        access.store(base_buff + 3 * simd_4.shift, add_op(in3_b, in3_i));
      }
      for (mwIndex icache = end_of_unrolled_buffer; icache < end_of_buffer; icache++)
      {
        buffer[icache] += cell_data[icache];
      }
    }

    Type * const base_output = output + ibase;

    // Finally we save back the result
    for (mwIndex icache = 0; icache < end_of_unrolled_buffer; icache += simd_4.block)
    {
      const Type * const base_buff = buffer + icache;
      const vVvf & in0 = access.load(base_buff + 0 * simd_4.shift);
      const vVvf & in1 = access.load(base_buff + 1 * simd_4.shift);
      const vVvf & in2 = access.load(base_buff + 2 * simd_4.shift);
      const vVvf & in3 = access.load(base_buff + 3 * simd_4.shift);

      Type * __restrict const base_cache_output = base_output + icache;
      access.store(base_cache_output + 0 * simd_4.shift, in0);
      access.store(base_cache_output + 1 * simd_4.shift, in1);
      access.store(base_cache_output + 2 * simd_4.shift, in2);
      access.store(base_cache_output + 3 * simd_4.shift, in3);
    }
    for (mwIndex icache = end_of_unrolled_buffer; icache < end_of_buffer; icache++)
    {
      base_output[icache] = buffer[icache];
    }
  }
}

}  // namespace GT_MATHS

#endif /* ZUTIL_CXX_INCLUDE_GTCXXMATHSSUMCELLVOLUMES_H_ */
