/*
 * internal_array_defs.h
 *
 *  Created on: Jul 17, 2012
 *
 * Nicola Vigano', 2012, ID11 @ ESRF vigano@esrf.eu
 */

#ifndef INTERNAL_ARRAY_DEFS_H_
#define INTERNAL_ARRAY_DEFS_H_

#include <cmath>
#include <omp.h>

//#undef __SSE2__
#ifdef __SSE2__
# include <xmmintrin.h>
# ifdef __GNUC__
#   define AND_V_SINGLE(x, y) __builtin_ia32_andps(x, y)
#   define OR_V_SINGLE(x, y) __builtin_ia32_orps(x, y)
#   define AND_V_DOUBLE(x, y) __builtin_ia32_andpd(x, y)
#   define OR_V_DOUBLE(x, y) __builtin_ia32_orpd(x, y)
# else
#   define AND_V_SINGLE(x, y) _mm_and_ps(x, y)
#   define OR_V_SINGLE(x, y) _mm_or_ps(x, y)
#   define AND_V_DOUBLE(x, y) _mm_and_pd(x, y)
#   define OR_V_DOUBLE(x, y) _mm_or_pd(x, y)
# endif
#elif __SSE__
# include <emmintrin.h>
#endif

#define SOFT_THRESH_SINGLE_VAL(Type, arrayIn, arrayOut, counter, offset) \
    const Type & oldVal##offset = arrayIn[counter + offset]; \
    const Type temp##offset = oldVal##offset - thr; \
    arrayOut[counter + offset] = copysign((temp##offset + fabs(temp##offset)) / 2, oldVal##offset)

#define LOAD_OLDVAL_V(arrayIn, counter, offset) \
  const vVvf oldVal##offset = *(vVvf *)&arrayIn[counter + offset * shift]

#define SOFT_THR_V(offset) \
  const vVvf temp##offset = oldVal##offset - thrV

#define STORE_SOFT_THR_V_DOUBLE(arrayOut, counter, offset) \
  *(vVvf *)&arrayOut[counter + offset * shift] = \
      OR_V_DOUBLE(AND_V_DOUBLE(absMask, ((temp##offset + AND_V_DOUBLE(absMask, temp##offset)) / div2) ), AND_V_DOUBLE(oldVal##offset, signMask))

#define STORE_SOFT_THR_V_SINGLE(arrayOut, counter, offset) \
  *(vVvf *)&arrayOut[counter + offset * shift] = \
      OR_V_SINGLE(AND_V_SINGLE(absMask, ((temp##offset + AND_V_SINGLE(absMask, temp##offset)) / div2) ), AND_V_SINGLE(oldVal##offset, signMask))

# define ROUND_DOWN(x, s) ((x) & ~((s)-1))

using namespace std;

template<typename SizeType, typename Type>
class soft_threshold {
  const Type thr;
public:
  soft_threshold(const Type & _thr) : thr(_thr) { }

  void operator()(Type * const __restrict array, const SizeType & numElems);
  void operator()(Type * const __restrict outArray,
      const Type * const __restrict inArray, const SizeType & numElems);
};

#ifdef __SSE2__
template<typename SizeType>
class soft_threshold<SizeType, double> {
  const double thr;
public:
  soft_threshold(const double & _thr) : thr(_thr) { }

  void operator()(double * const __restrict array, const SizeType & numElems);
  void operator()(double * const __restrict outArray,
      const double * const __restrict inArray, const SizeType & numElems);
};


template<typename SizeType>
inline void
soft_threshold<SizeType, double>::operator()(double * const __restrict array,
    const SizeType & numElems)
{
  typedef double vVvf __attribute__((vector_size(16))) __attribute__((aligned(16)));
  const SizeType unrolling = 8;
  const SizeType shift = 16 / sizeof(double);
  const SizeType block = shift * unrolling;

  const vVvf div2 = _mm_set1_pd(2.0);
  const vVvf thrV = _mm_set1_pd(thr);
  const vVvf signMask = _mm_castsi128_pd(_mm_set1_epi64x(0x8000000000000000L));
  const vVvf absMask  = _mm_castsi128_pd(_mm_set1_epi64x(0x7fffffffffffffffL));

  const SizeType numRoundElems = ROUND_DOWN(numElems, block);
#pragma parallel
  {
#pragma omp for nowait
    for(SizeType count = 0; count < numRoundElems; count += block)
    {
      LOAD_OLDVAL_V(array, count,  0);
      LOAD_OLDVAL_V(array, count,  1);
      LOAD_OLDVAL_V(array, count,  2);
      LOAD_OLDVAL_V(array, count,  3);

      SOFT_THR_V( 0);
      SOFT_THR_V( 1);
      SOFT_THR_V( 2);
      SOFT_THR_V( 3);

      STORE_SOFT_THR_V_DOUBLE(array, count,  0);
      STORE_SOFT_THR_V_DOUBLE(array, count,  1);
      STORE_SOFT_THR_V_DOUBLE(array, count,  2);
      STORE_SOFT_THR_V_DOUBLE(array, count,  3);

      LOAD_OLDVAL_V(array, count,  4);
      LOAD_OLDVAL_V(array, count,  5);
      LOAD_OLDVAL_V(array, count,  6);
      LOAD_OLDVAL_V(array, count,  7);

      SOFT_THR_V( 4);
      SOFT_THR_V( 5);
      SOFT_THR_V( 6);
      SOFT_THR_V( 7);

      STORE_SOFT_THR_V_DOUBLE(array, count,  4);
      STORE_SOFT_THR_V_DOUBLE(array, count,  5);
      STORE_SOFT_THR_V_DOUBLE(array, count,  6);
      STORE_SOFT_THR_V_DOUBLE(array, count,  7);
    }
#pragma omp for nowait
    for(SizeType count = numRoundElems; count < numElems; count++)
    {
      SOFT_THRESH_SINGLE_VAL(double, array, array, count, 0);
    }
  }
}

template<typename SizeType>
inline void
soft_threshold<SizeType, double>::operator()(double * const __restrict outArray,
    const double * const __restrict inArray, const SizeType & numElems)
{
  typedef double vVvf __attribute__((vector_size(16))) __attribute__((aligned(16)));
  const SizeType unrolling = 8;
  const SizeType shift = 16 / sizeof(double);
  const SizeType block = shift * unrolling;

  const vVvf div2 = _mm_set1_pd(2.0);
  const vVvf thrV = _mm_set1_pd(thr);
  const vVvf signMask = _mm_castsi128_pd(_mm_set1_epi64x(0x8000000000000000L));
  const vVvf absMask  = _mm_castsi128_pd(_mm_set1_epi64x(0x7fffffffffffffffL));

  const SizeType numRoundElems = ROUND_DOWN(numElems, block);
#pragma parallel
  {
#pragma omp for nowait
    for(SizeType count = 0; count < numRoundElems; count += block)
    {
      LOAD_OLDVAL_V(inArray, count,  0);
      LOAD_OLDVAL_V(inArray, count,  1);
      LOAD_OLDVAL_V(inArray, count,  2);
      LOAD_OLDVAL_V(inArray, count,  3);

      SOFT_THR_V( 0);
      SOFT_THR_V( 1);
      SOFT_THR_V( 2);
      SOFT_THR_V( 3);

      STORE_SOFT_THR_V_DOUBLE(outArray, count,  0);
      STORE_SOFT_THR_V_DOUBLE(outArray, count,  1);
      STORE_SOFT_THR_V_DOUBLE(outArray, count,  2);
      STORE_SOFT_THR_V_DOUBLE(outArray, count,  3);

      LOAD_OLDVAL_V(inArray, count,  4);
      LOAD_OLDVAL_V(inArray, count,  5);
      LOAD_OLDVAL_V(inArray, count,  6);
      LOAD_OLDVAL_V(inArray, count,  7);

      SOFT_THR_V( 4);
      SOFT_THR_V( 5);
      SOFT_THR_V( 6);
      SOFT_THR_V( 7);

      STORE_SOFT_THR_V_DOUBLE(outArray, count,  4);
      STORE_SOFT_THR_V_DOUBLE(outArray, count,  5);
      STORE_SOFT_THR_V_DOUBLE(outArray, count,  6);
      STORE_SOFT_THR_V_DOUBLE(outArray, count,  7);
    }
#pragma omp for nowait
    for(SizeType count = numRoundElems; count < numElems; count++)
    {
      SOFT_THRESH_SINGLE_VAL(double, inArray, outArray, count, 0);
    }
  }
}
#endif

#ifdef __SSE__
template<typename SizeType>
class soft_threshold<SizeType, float> {
  const float thr;
public:
  soft_threshold(const float & _thr) : thr(_thr) { }

  void operator()(float * const __restrict array, const SizeType & numElems);
  void operator()(float * const __restrict outArray,
      const float * const __restrict inArray, const SizeType & numElems);
};


template<typename SizeType>
inline void
soft_threshold<SizeType, float>::operator()(float * const __restrict array,
    const SizeType & numElems)
{
  typedef float vVvf __attribute__((vector_size(16))) __attribute__((aligned(16)));
  const SizeType unrolling = 8;
  const SizeType shift = 16 / sizeof(float);
  const SizeType block = shift * unrolling;

  const vVvf div2 = _mm_set1_ps(2.0);
  const vVvf thrV = _mm_set1_ps(thr);
  const vVvf signMask = _mm_castsi128_ps(_mm_set1_epi32(0x80000000));
  const vVvf absMask  = _mm_castsi128_ps(_mm_set1_epi32(0x7fffffff));

  const SizeType numRoundElems = ROUND_DOWN(numElems, block);
#pragma parallel
  {
#pragma omp for nowait
    for(SizeType count = 0; count < numRoundElems; count += block)
    {
      LOAD_OLDVAL_V(array, count,  0);
      LOAD_OLDVAL_V(array, count,  1);
      LOAD_OLDVAL_V(array, count,  2);
      LOAD_OLDVAL_V(array, count,  3);

      SOFT_THR_V( 0);
      SOFT_THR_V( 1);
      SOFT_THR_V( 2);
      SOFT_THR_V( 3);

      STORE_SOFT_THR_V_SINGLE(array, count,  0);
      STORE_SOFT_THR_V_SINGLE(array, count,  1);
      STORE_SOFT_THR_V_SINGLE(array, count,  2);
      STORE_SOFT_THR_V_SINGLE(array, count,  3);

      LOAD_OLDVAL_V(array, count,  4);
      LOAD_OLDVAL_V(array, count,  5);
      LOAD_OLDVAL_V(array, count,  6);
      LOAD_OLDVAL_V(array, count,  7);

      SOFT_THR_V( 4);
      SOFT_THR_V( 5);
      SOFT_THR_V( 6);
      SOFT_THR_V( 7);

      STORE_SOFT_THR_V_SINGLE(array, count,  4);
      STORE_SOFT_THR_V_SINGLE(array, count,  5);
      STORE_SOFT_THR_V_SINGLE(array, count,  6);
      STORE_SOFT_THR_V_SINGLE(array, count,  7);
    }
#pragma omp for nowait
    for(SizeType count = numRoundElems; count < numElems; count++)
    {
      SOFT_THRESH_SINGLE_VAL(float, array, array, count, 0);
    }
  }
}

template<typename SizeType>
inline void
soft_threshold<SizeType, float>::operator()(float * const __restrict outArray,
    const float * const __restrict inArray, const SizeType & numElems)
{
  typedef float vVvf __attribute__((vector_size(16))) __attribute__((aligned(16)));
  const SizeType unrolling = 8;
  const SizeType shift = 16 / sizeof(float);
  const SizeType block = shift * unrolling;

  const vVvf div2 = _mm_set1_ps(2.0);
  const vVvf thrV = _mm_set1_ps(thr);
  const vVvf signMask = _mm_castsi128_ps(_mm_set1_epi32(0x80000000));
  const vVvf absMask  = _mm_castsi128_ps(_mm_set1_epi32(0x7fffffff));

  const SizeType numRoundElems = ROUND_DOWN(numElems, block);
#pragma parallel
  {
#pragma omp for nowait
    for(SizeType count = 0; count < numRoundElems; count += block)
    {
      LOAD_OLDVAL_V(inArray, count,  0);
      LOAD_OLDVAL_V(inArray, count,  1);
      LOAD_OLDVAL_V(inArray, count,  2);
      LOAD_OLDVAL_V(inArray, count,  3);

      SOFT_THR_V( 0);
      SOFT_THR_V( 1);
      SOFT_THR_V( 2);
      SOFT_THR_V( 3);

      STORE_SOFT_THR_V_SINGLE(outArray, count,  0);
      STORE_SOFT_THR_V_SINGLE(outArray, count,  1);
      STORE_SOFT_THR_V_SINGLE(outArray, count,  2);
      STORE_SOFT_THR_V_SINGLE(outArray, count,  3);

      LOAD_OLDVAL_V(inArray, count,  4);
      LOAD_OLDVAL_V(inArray, count,  5);
      LOAD_OLDVAL_V(inArray, count,  6);
      LOAD_OLDVAL_V(inArray, count,  7);

      SOFT_THR_V( 4);
      SOFT_THR_V( 5);
      SOFT_THR_V( 6);
      SOFT_THR_V( 7);

      STORE_SOFT_THR_V_SINGLE(outArray, count,  4);
      STORE_SOFT_THR_V_SINGLE(outArray, count,  5);
      STORE_SOFT_THR_V_SINGLE(outArray, count,  6);
      STORE_SOFT_THR_V_SINGLE(outArray, count,  7);
    }
#pragma omp for nowait
    for(SizeType count = numRoundElems; count < numElems; count++)
    {
      SOFT_THRESH_SINGLE_VAL(float, inArray, outArray, count, 0);
    }
  }
}
#endif

template<typename SizeType, typename Type>
inline void
soft_threshold<SizeType, Type>::operator()(Type * const __restrict array,
    const SizeType & numElems)
{
  const SizeType block = 8;
  const SizeType numRoundElems = ROUND_DOWN(numElems, block);
#pragma parallel
  {
#pragma omp for nowait
    for(SizeType count = 0; count < numRoundElems; count += block)
    {
      SOFT_THRESH_SINGLE_VAL(Type, array, array, count,  0);
      SOFT_THRESH_SINGLE_VAL(Type, array, array, count,  1);
      SOFT_THRESH_SINGLE_VAL(Type, array, array, count,  2);
      SOFT_THRESH_SINGLE_VAL(Type, array, array, count,  3);
      SOFT_THRESH_SINGLE_VAL(Type, array, array, count,  4);
      SOFT_THRESH_SINGLE_VAL(Type, array, array, count,  5);
      SOFT_THRESH_SINGLE_VAL(Type, array, array, count,  6);
      SOFT_THRESH_SINGLE_VAL(Type, array, array, count,  7);
      SOFT_THRESH_SINGLE_VAL(Type, array, array, count,  8);
      SOFT_THRESH_SINGLE_VAL(Type, array, array, count,  9);
      SOFT_THRESH_SINGLE_VAL(Type, array, array, count, 10);
      SOFT_THRESH_SINGLE_VAL(Type, array, array, count, 11);
      SOFT_THRESH_SINGLE_VAL(Type, array, array, count, 12);
      SOFT_THRESH_SINGLE_VAL(Type, array, array, count, 13);
      SOFT_THRESH_SINGLE_VAL(Type, array, array, count, 14);
      SOFT_THRESH_SINGLE_VAL(Type, array, array, count, 15);
    }
#pragma omp for nowait
    for(SizeType count = numRoundElems; count < numElems; count++)
    {
      SOFT_THRESH_SINGLE_VAL(Type, array, array, count,  0);
    }
  }
}

template<typename SizeType, typename Type>
inline void
soft_threshold<SizeType, Type>::operator()(Type * const __restrict outArray,
    const Type * const __restrict inArray, const SizeType & numElems)
{
  const SizeType block = 16;
  const SizeType numRoundElems = ROUND_DOWN(numElems, block);
#pragma parallel
  {
#pragma omp for nowait
    for(SizeType count = 0; count < numRoundElems; count += block)
    {
      SOFT_THRESH_SINGLE_VAL(Type, inArray, outArray, count,  0);
      SOFT_THRESH_SINGLE_VAL(Type, inArray, outArray, count,  1);
      SOFT_THRESH_SINGLE_VAL(Type, inArray, outArray, count,  2);
      SOFT_THRESH_SINGLE_VAL(Type, inArray, outArray, count,  3);
      SOFT_THRESH_SINGLE_VAL(Type, inArray, outArray, count,  4);
      SOFT_THRESH_SINGLE_VAL(Type, inArray, outArray, count,  5);
      SOFT_THRESH_SINGLE_VAL(Type, inArray, outArray, count,  6);
      SOFT_THRESH_SINGLE_VAL(Type, inArray, outArray, count,  7);
      SOFT_THRESH_SINGLE_VAL(Type, inArray, outArray, count,  8);
      SOFT_THRESH_SINGLE_VAL(Type, inArray, outArray, count,  9);
      SOFT_THRESH_SINGLE_VAL(Type, inArray, outArray, count, 10);
      SOFT_THRESH_SINGLE_VAL(Type, inArray, outArray, count, 11);
      SOFT_THRESH_SINGLE_VAL(Type, inArray, outArray, count, 12);
      SOFT_THRESH_SINGLE_VAL(Type, inArray, outArray, count, 13);
      SOFT_THRESH_SINGLE_VAL(Type, inArray, outArray, count, 14);
      SOFT_THRESH_SINGLE_VAL(Type, inArray, outArray, count, 15);
    }
#pragma omp for nowait
    for(SizeType count = numRoundElems; count < numElems; count++)
    {
      SOFT_THRESH_SINGLE_VAL(Type, inArray, outArray, count,  0);
    }
  }
}

////////////////////////////////////////////////////////////////////////////////
/// computeAverage

template<typename Type, typename SizeType>
Type
computeAverage(const Type * const inArray, const SizeType & numElems)
{
  typedef Type vVvf __attribute__((vector_size(16))) __attribute__((aligned(16)));
  const SizeType unrolling = 16;
  const SizeType shift = 16 / sizeof(Type);
  const SizeType block = shift * unrolling;

  Type temp_av = 0;
  vVvf accumul;
  switch (shift) {
    case 2: {
      accumul = (vVvf)_mm_set1_pd(0.0);
      break;
    }
    case 4: {
      accumul = (vVvf)_mm_set1_ps(0.0);
      break;
    }
  }

  const SizeType numUnrolledElems = ROUND_DOWN(numElems, block);
#pragma omp parallel
  {
#pragma omp for reduction(+:accumul) nowait
    for(SizeType count = 0; count < numUnrolledElems; count += block) {
      accumul += ( *((vVvf *)&inArray[count +  0 * shift]) + *((vVvf *)&inArray[count +  1 * shift])
                 + *((vVvf *)&inArray[count +  2 * shift]) + *((vVvf *)&inArray[count +  3 * shift])
                 + *((vVvf *)&inArray[count +  4 * shift]) + *((vVvf *)&inArray[count +  5 * shift])
                 + *((vVvf *)&inArray[count +  6 * shift]) + *((vVvf *)&inArray[count +  7 * shift])
                 + *((vVvf *)&inArray[count +  8 * shift]) + *((vVvf *)&inArray[count +  9 * shift])
                 + *((vVvf *)&inArray[count + 10 * shift]) + *((vVvf *)&inArray[count + 11 * shift])
                 + *((vVvf *)&inArray[count + 12 * shift]) + *((vVvf *)&inArray[count + 13 * shift])
                 + *((vVvf *)&inArray[count + 14 * shift]) + *((vVvf *)&inArray[count + 15 * shift]) );
    }
#pragma omp for reduction(+:temp_av) nowait
    for(SizeType count = numUnrolledElems; count < numElems; count++) {
      temp_av += inArray[count];
    }
  }

  const Type * const __restrict accumuld = (Type *)&accumul;
  switch (shift) {
    case 2: {
      return (temp_av + accumuld[0] + accumuld[1]) / numElems;
    }
    case 4: {
      return (temp_av + accumuld[0] + accumuld[1] + accumuld[2] + accumuld[3]
          ) / numElems;
    }
  }
}


#endif /* INTERNAL_ARRAY_DEFS_H_ */
