/*
 * internal_cell_defs.h
 *
 *  Created on: Jul 11, 2012
 *      Author: vigano
 */

#ifndef INTERNAL_CELL_DEFS_H_
#define INTERNAL_CELL_DEFS_H_

#include <mex.h>
#include <emlrt.h>
#include <math.h>
#ifndef DEBUG
# include <omp.h>
#endif

# define ROUND_DOWN(x, s) ((x) & ~((s)-1))

#if defined(__AVX__)
# define MEMORY_ALIGN 32
#else
# define MEMORY_ALIGN 16
#endif

#include <immintrin.h>

#include <algorithm>

extern "C"
{
  bool mxUnshareArray(mxArray *array_ptr, bool noDeepCopy);
  mxArray *mxCreateSharedCopy(const mxArray *pr);
  mxArray *mxCreateSharedDataCopy(const mxArray *pr);
  mxArray *mxUnreference(const mxArray *pr);
}

void
initialize_multithreading(const double & suggested_num_threads = 0)
{
#ifndef DEBUG
  if (suggested_num_threads > 0)
  {
    int num_threads = (const int)suggested_num_threads;
    num_threads = std::min(num_threads, omp_get_num_procs());
    num_threads = std::max(num_threads, 1);
    omp_set_num_threads(num_threads);
  }
  else
  {
    omp_set_num_threads(std::max(omp_get_num_procs()/4*3, 1));
  }
#endif
}

template<typename Type>
class SIMDUnrolling {
public:
  const size_t shift;
  const size_t unrolling;
  const size_t block;

  typedef Type vVvf __attribute__((vector_size(MEMORY_ALIGN))) __attribute__((aligned(MEMORY_ALIGN)));

  SIMDUnrolling(const size_t & _unroll)
  : shift(MEMORY_ALIGN / sizeof(Type)), unrolling(_unroll)
  , block(shift * unrolling)
  { }

  const size_t
  get_unroll(const size_t & tot_size) const
  {
    return ROUND_DOWN(tot_size, block);
  }
};

template<typename Type>
class AccessAligned {
public:
  typedef typename SIMDUnrolling<Type>::vVvf vVvf;

  const vVvf load(const Type * const __restrict in) const
  {
    return *(vVvf *)in;
  }
  void store(Type * const __restrict out, const vVvf & in) const
  {
    *(vVvf *)out = in;
  }
};

template<typename Type>
class AccessStreamed : public AccessAligned<Type> {
public:
  typedef typename AccessAligned<Type>::vVvf vVvf;

  void store(Type * const __restrict out, const vVvf & in) const;
};

template<>
inline void
AccessStreamed<float>::store(float * const __restrict out, const AccessStreamed<float>::vVvf & in) const
{
#if defined(__AVX__)
  _mm256_stream_ps(out, in);
#else
  _mm_stream_ps(out, in);
#endif
}

template<>
inline void
AccessStreamed<double>::store(double * const __restrict out, const AccessStreamed<double>::vVvf & in) const
{
#if defined(__AVX__)
  _mm256_stream_pd(out, in);
#else
  _mm_stream_pd(out, in);
#endif
}

template<typename Type>
class AccessUnaligned {
public:
  typedef typename SIMDUnrolling<Type>::vVvf vVvf;

  const vVvf load(const Type * const __restrict in) const;
  void store(Type * const __restrict out, const vVvf & in) const;
};

template<>
class AccessUnaligned<double> {
public:
  typedef SIMDUnrolling<double>::vVvf vVvf;

  const vVvf load(const double * const __restrict in) const
  {
#if defined(__AVX__)
    return _mm256_loadu_pd(in);
#else
    return _mm_loadu_pd(in);
#endif
  }
  void store(double * const __restrict out, const vVvf & in) const
  {
#if defined(__AVX__)
    _mm256_storeu_pd(out, in);
#else
    _mm_storeu_pd(out, in);
#endif
  }
};

template<>
class AccessUnaligned<float> {
public:
  typedef SIMDUnrolling<float>::vVvf vVvf;

  const vVvf load(const float * const __restrict in) const
  {
#if defined(__AVX__)
    return _mm256_loadu_ps(in);
#else
    return _mm_loadu_ps(in);
#endif
  }
  void store(float * const __restrict out, const vVvf & in) const
  {
#if defined(__AVX__)
    _mm256_storeu_ps(out, in);
#else
    _mm_storeu_ps(out, in);
#endif
  }
};

template<typename Type>
class Coeff {
public:
  typedef typename SIMDUnrolling<Type>::vVvf vVvf;

  static const vVvf get(const Type & coeff);
};

template<>
class Coeff<float> {
public:
  typedef SIMDUnrolling<float>::vVvf vVvf;

  static const vVvf get(const float & coeff)
  {
#if defined(__AVX__)
    return (const vVvf) _mm256_set1_ps(coeff);
#else
    return (const vVvf) _mm_set1_ps(coeff);
#endif
  }
};

template<>
class Coeff<double> {
public:
  typedef SIMDUnrolling<double>::vVvf vVvf;

  static const vVvf get(const double & coeff)
  {
#if defined(__AVX__)
    return (const vVvf) _mm256_set1_pd(coeff);
#else
    return (const vVvf) _mm_set1_pd(coeff);
#endif
  }
};



mxClassID
getMxClassOfCell(const mxArray * cell)
{
  const mwSize numElems = mxGetNumberOfElements(cell);
  if (numElems > 0) {
    const mxArray * const firstArray = mxGetCell(cell, 0);
    const mxClassID firstClass = mxGetClassID(firstArray);
    if (firstClass == mxSINGLE_CLASS || firstClass == mxDOUBLE_CLASS) {
      for(mwIndex count = 1; count < numElems; count++) {
        const mxArray * const array = mxGetCell(cell, count);
        if (firstClass != mxGetClassID(array)) {
          // Damn mixed types!
          return mxUNKNOWN_CLASS;
        }
      }
      // OK it is a float type!
      return firstClass;
    } else {
      // It is still an error!
      return firstClass;
    }
  } else {
    return mxUNKNOWN_CLASS;
  }
}

template<typename Type>
class inner_assign {
public:
  typedef typename SIMDUnrolling<Type>::vVvf vVvf;

  const Type
  operator()(const Type & inData1, const Type & inData2) const throw()
  {
    return (inData2);
  }
  const vVvf
  operator()(const vVvf & inData1, const vVvf & inData2) const throw()
  {
    return (inData2);
  }
};

template<typename Type>
class inner_pow {
  const double exponent;
public:
  inner_pow(const double & _exp) : exponent(_exp) { }

  const Type
  operator()(const Type & inData1, const Type & inData2) const throw()
  {
    return pow(inData2, exponent);
  }
  const __v2df
  operator()(const __v2df & inData1, const __v2df & inData2) const throw()
  {
    const double * const inData1_d = (const double *) &inData1;
    return _mm_set_pd(
      pow(inData1_d[0], exponent), pow(inData1_d[1], exponent)
    );
  }
  const __v4sf
  operator()(const __v4sf & inData1, const __v4sf & inData2) const throw()
  {
    const float * const inData1_s = (const float *) &inData1;
    return _mm_set_ps(
      pow(inData1_s[0], exponent), pow(inData1_s[1], exponent),
      pow(inData1_s[2], exponent), pow(inData1_s[3], exponent)
    );
  }
#if defined(__AVX__)
  const __v4df
  operator()(const __v4df & inData1, const __v4df & inData2) const throw()
  {
    const double * const inData1_d = (const double *) &inData1;
    return _mm256_set_pd(
      pow(inData1_d[0], exponent), pow(inData1_d[1], exponent),
      pow(inData1_d[2], exponent), pow(inData1_d[3], exponent)
    );
  }
  const __v8sf
  operator()(const __v8sf & inData1, const __v8sf & inData2) const throw()
  {
    const float * const inData1_s = (const float *) &inData1;
    return _mm256_set_ps(
      pow(inData1_s[0], exponent), pow(inData1_s[1], exponent),
      pow(inData1_s[2], exponent), pow(inData1_s[3], exponent),
      pow(inData1_s[4], exponent), pow(inData1_s[5], exponent),
      pow(inData1_s[6], exponent), pow(inData1_s[7], exponent)
    );
  }
#endif
};

template<typename Type>
class inner_sqrt {
public:
  typedef typename SIMDUnrolling<Type>::vVvf vVvf;

  const Type
  operator()(const Type & inData1, const Type & inData2) const throw()
  {
    return sqrt(inData2); // __builtin_ia32_sqrtpd
  }
  const __v2df
  operator()(const __v2df & inData1, const __v2df & inData2) const throw()
  {
    return _mm_sqrt_pd(inData2);
  }
  const __v4sf
  operator()(const __v4sf & inData1, const __v4sf & inData2) const throw()
  {
    return _mm_sqrt_ps(inData2);
  }
#if defined(__AVX__)
  const __v4df
  operator()(const __v4df & inData1, const __v4df & inData2) const throw()
  {
    return _mm256_sqrt_pd(inData2);
  }
  const __v8sf
  operator()(const __v8sf & inData1, const __v8sf & inData2) const throw()
  {
    return _mm256_sqrt_ps(inData2);
  }
#endif
};

template<typename Type>
class inner_sum {
public:
  typedef typename SIMDUnrolling<Type>::vVvf vVvf;

  const Type
  operator()(const Type & inData1, const Type & inData2) const throw()
  {
    return (inData1 + inData2);
  }
  const vVvf
  operator()(const vVvf & inData1, const vVvf & inData2) const throw()
  {
    return (inData1 + inData2);
  }
};

template<typename Type>
class inner_prod {
public:
  typedef typename SIMDUnrolling<Type>::vVvf vVvf;

  const Type
  operator()(const Type & inData1, const Type & inData2) const throw()
  {
    return (inData1 * inData2);
  }
  const vVvf
  operator()(const vVvf & inData1, const vVvf & inData2) const throw()
  {
    return (inData1 * inData2);
  }
};

template<typename Type>
class inner_sub {
public:
  typedef typename SIMDUnrolling<Type>::vVvf vVvf;

  const Type
  operator()(const Type & inData1, const Type & inData2) const throw()
  {
    return (inData1 - inData2);
  }
  const vVvf
  operator()(const vVvf & inData1, const vVvf & inData2) const throw()
  {
    return (inData1 - inData2);
  }
};

template<typename Type>
class inner_div {
public:
  typedef typename SIMDUnrolling<Type>::vVvf vVvf;

  const Type
  operator()(const Type & inData1, const Type & inData2) const throw()
  {
    return (inData1 / inData2);
  }
  const vVvf
  operator()(const vVvf & inData1, const vVvf & inData2) const throw()
  {
    return (inData1 / inData2);
  }
};


template<typename Type>
class inner_sum_scale {
public:
  typedef typename SIMDUnrolling<Type>::vVvf vVvf;

  inner_sum_scale(const Type & _scale) : scale(_scale), scale_v(Coeff<Type>::get(_scale)) { }

  const Type
  operator()(const Type & inData1, const Type & inData2) const throw()
  {
    return (inData1 + scale * inData2);
  }
  const vVvf
  operator()(const vVvf & inData1, const vVvf & inData2) const throw()
  {
    return (inData1 + scale_v * inData2);
  }
protected:
  const Type scale;
  const vVvf scale_v;
};

template<typename Type>
class inner_sum_FISTA_scale {
public:
  typedef typename SIMDUnrolling<Type>::vVvf vVvf;

  inner_sum_FISTA_scale(const Type & _scale) : scale(_scale), scale_v(Coeff<Type>::get(_scale)) { }

  const Type
  operator()(const Type & inData1, const Type & inData2) const throw()
  {
    return (inData1 + scale * (inData1 - inData2));
  }
  const vVvf
  operator()(const vVvf & inData1, const vVvf & inData2) const throw()
  {
    return (inData1 + scale_v * (inData1 - inData2));
  }
protected:
  const Type scale;
  const vVvf scale_v;
};

template<typename Type>
class inner_non_neg {
public:
  typedef typename SIMDUnrolling<Type>::vVvf vVvf;

  inner_non_neg()
  : zero_v(Coeff<Type>::get(0.0f))
  { }

  const Type
  operator()(const Type & inData1, const Type & inData2) const throw()
  {
    return inData1 * (inData1 >= 0);
  }

  const vVvf
  operator()(const vVvf & inData1, const vVvf & inData2) const throw();
protected:
  const vVvf zero_v;
};

template<>
inline const inner_non_neg<float>::vVvf
inner_non_neg<float>::operator ()(
    const inner_non_neg<float>::vVvf & inData1,
    const inner_non_neg<float>::vVvf & inData2)
const throw()
{
  typedef inner_non_neg<float>::vVvf vVvf;
#if defined(__AVX__)
  return _mm256_and_ps(inData1, _mm256_cmp_ps(inData1, zero_v, _CMP_GE_OS));
#else
  return _mm_and_ps(inData1, _mm_cmpge_ps(inData1, zero_v));
#endif
}

template<>
inline const inner_non_neg<double>::vVvf
inner_non_neg<double>::operator ()(
    const inner_non_neg<double>::vVvf & inData1,
    const inner_non_neg<double>::vVvf & inData2)
const throw()
{
  typedef inner_non_neg<double>::vVvf vVvf;
#if defined(__AVX__)
  return _mm256_and_pd(inData1, _mm256_cmp_pd(inData1, zero_v, _CMP_GE_OS));
#else
  return _mm_and_pd(inData1, _mm_cmpge_pd(inData1, zero_v));
#endif
}

template<typename Type>
class inner_sum_FISTA_scale_non_neg {
public:
  typedef typename SIMDUnrolling<Type>::vVvf vVvf;

  inner_sum_FISTA_scale_non_neg(const Type & _scale)
  : scale(_scale), scale_v(Coeff<Type>::get(_scale)), zero_v(Coeff<Type>::get(0.0f))
  { }

  const Type
  operator()(const Type & inData1, const Type & inData2) const throw()
  {
    const Type intermed = (inData1 + scale * (inData1 - inData2));
    return intermed * (intermed >= 0);
  }
  const vVvf
  operator()(const vVvf & inData1, const vVvf & inData2) const throw();
protected:
  const Type scale;
  const vVvf scale_v;
  const vVvf zero_v;
};

template<>
inline const inner_sum_FISTA_scale_non_neg<float>::vVvf
inner_sum_FISTA_scale_non_neg<float>::operator ()(
    const inner_sum_FISTA_scale_non_neg<float>::vVvf & inData1,
    const inner_sum_FISTA_scale_non_neg<float>::vVvf & inData2)
const throw()
{
  typedef inner_sum_FISTA_scale_non_neg<float>::vVvf vVvf;
  const vVvf intermed = (inData1 + scale_v * (inData1 - inData2));
#if defined(__AVX__)
  return _mm256_and_ps(intermed, _mm256_cmp_ps(intermed, zero_v, _CMP_GE_OS));
#else
  return _mm_and_ps(intermed, _mm_cmpge_ps(intermed, zero_v));
#endif
}

template<>
inline const inner_sum_FISTA_scale_non_neg<double>::vVvf
inner_sum_FISTA_scale_non_neg<double>::operator ()(
    const inner_sum_FISTA_scale_non_neg<double>::vVvf & inData1,
    const inner_sum_FISTA_scale_non_neg<double>::vVvf & inData2)
const throw()
{
  typedef inner_sum_FISTA_scale_non_neg<double>::vVvf vVvf;
  const vVvf intermed = (inData1 + scale_v * (inData1 - inData2));
#if defined(__AVX__)
  return _mm256_and_pd(intermed, _mm256_cmp_pd(intermed, zero_v, _CMP_GE_OS));
#else
  return _mm_and_pd(intermed, _mm_cmpge_pd(intermed, zero_v));
#endif
}

template<typename Type>
class inner_sub_scale {
public:
  typedef typename SIMDUnrolling<Type>::vVvf vVvf;

  inner_sub_scale(const Type & _scale) : scale(_scale), scale_v(Coeff<Type>::get(_scale)) { }

  const Type
  operator()(const Type & inData1, const Type & inData2) const throw()
  {
    return (inData1 - inData2 * scale);
  }
  const vVvf
  operator()(const vVvf & inData1, const vVvf & inData2) const throw()
  {
    return (inData1 - inData2 * scale_v);
  }
protected:
  const Type scale;
  const vVvf scale_v;
};


template<typename Type>
class inner_sum_scalar {
public:
  typedef typename SIMDUnrolling<Type>::vVvf vVvf;

  inner_sum_scalar(const Type & _scalar) : scalar(_scalar), scalar_v(Coeff<Type>::get(_scalar)) { }

  const Type
  operator()(const Type & inData1, const Type & inData2) const throw()
  {
    return (inData2 + scalar);
  }
  const vVvf
  operator()(const vVvf & inData1, const vVvf & inData2) const throw()
  {
    return (inData2 + scalar_v);
  }
protected:
  const Type scalar;
  const vVvf scalar_v;
};

template<typename Type>
class inner_prod_scalar {
public:
  typedef typename SIMDUnrolling<Type>::vVvf vVvf;

  inner_prod_scalar(const Type & _scalar) : scalar(_scalar), scalar_v(Coeff<Type>::get(_scalar)) { }

  const Type
  operator()(const Type & inData1, const Type & inData2) const throw()
  {
    return (inData2 * scalar);
  }
  const vVvf
  operator()(const vVvf & inData1, const vVvf & inData2) const throw()
  {
    return (inData2 * scalar_v);
  }
protected:
  const Type scalar;
  const vVvf scalar_v;
};

inline mxArray *
vol_allocate(const mwSize & num_dims, const mwSize * const dims, const mxClassID & type)
{
  #if defined(EMLRT_VERSION_INFO) && EMLRT_VERSION_INFO >= 0x2015a && !defined(MX_COMPAT_32)
      return mxCreateUninitNumericArray(num_dims, (mwSize *)dims, type, mxREAL);
  #else
      const mwSize zero_dims[2] = {0, 0};
      mxArray * const xOutput = mxCreateNumericArray(2, zero_dims, type, mxREAL);
      mxSetDimensions(xOutput, dims, num_dims);
      const mwSize num_elems = mxGetNumberOfElements(xOutput);
      const mwSize elem_size = mxGetElementSize(xOutput);
      mxSetData(xOutput, mxMalloc(elem_size * num_elems));
      return xOutput;
  #endif
}

inline void
cell_allocate(mxArray * & outArray, const mxArray * const inArray)
{
  const mwSize numCells = mxGetNumberOfElements(inArray);
  const mwSize numCellDims = mxGetNumberOfDimensions(inArray);
  const mwSize * cellDims = mxGetDimensions(inArray);

  outArray = mxCreateCellArray(numCellDims, cellDims);

  for(mwIndex cellIdx = 0; cellIdx < numCells; cellIdx++)
  {
    const mxArray * inCell = mxGetCell(inArray, cellIdx);
    const mxClassID type = mxGetClassID(inCell);
    const mwSize num_dims = mxGetNumberOfDimensions(inCell);
    const mwSize * dims = mxGetDimensions(inCell);

    mxArray * const outCell = vol_allocate(num_dims, dims, type);
    mxSetCell(outArray, cellIdx, outCell);
  }
}

inline void
cell_allocate_single_vol_from_dims(mxArray * & out_array,
    const mxArray * const mxdims, const mxClassID & type)
{
  const mwSize num_dims = mxGetNumberOfElements(mxdims);
  const double * dims_d = (double *) mxGetData(mxdims);
  mwSize * dims = new mwSize[num_dims];
  mwSize num_elems = 1;
  for(mwIndex dim = 0; dim < num_dims; dim++)
  {
    dims[dim] = (mwSize) dims_d[dim];
    num_elems *= dims[dim];
  }
  out_array = vol_allocate(num_dims, dims, type);

  delete [] dims;
}

inline void
cell_allocate_from_dims(mxArray * & out_array, const mxArray * const dims,
    const mxClassID & type)
{
  const mwSize num_cells = mxGetNumberOfElements(dims);
  const mwSize num_cell_dims = mxGetNumberOfDimensions(dims);
  const mwSize * cell_dims = mxGetDimensions(dims);

  out_array = mxCreateCellArray(num_cell_dims, cell_dims);

  for(mwIndex cell_idx = 0; cell_idx < num_cells; cell_idx++)
  {
    const mxArray * inCell = mxGetCell(dims, cell_idx);
    const mwSize num_dims = mxGetNumberOfElements(inCell);
    const double * dims_d = (double *) mxGetData(inCell);
    mwSize * dims = new mwSize[num_dims];
    mwSize num_elems = 1;
    for(mwIndex dim = 0; dim < num_dims; dim++)
    {
      dims[dim] = (mwSize) dims_d[dim];
      num_elems *= dims[dim];
    }

    mxArray * const out_cell = vol_allocate(num_dims, dims, type);
    mxSetCell(out_array, cell_idx, out_cell);

    delete [] dims;
  }
}

template<typename Type, typename Function, class AccessType>
inline void
cell_inner_cycle_sse(Type * const __restrict outData,
    const Type * const __restrict inData1,
    const Type * const __restrict inData2,
    const mwSize & numElems, Function & func)
{
  typedef typename SIMDUnrolling<Type>::vVvf vVvf;

  const SIMDUnrolling<Type> simd_8(8);
  const SIMDUnrolling<Type> simd_1(1);

  AccessType access;

#pragma omp for nowait
  for(mwIndex elemIdx = 0; elemIdx < simd_8.get_unroll(numElems);
      elemIdx += simd_8.block)
  {
    const Type * const in1 = inData1 + elemIdx;
    const Type * const in2 = inData2 + elemIdx;
    Type * const out = outData + elemIdx;

    const vVvf inV11 = access.load(in1 + 0 * simd_8.shift);
    const vVvf inV12 = access.load(in1 + 1 * simd_8.shift);
    const vVvf inV13 = access.load(in1 + 2 * simd_8.shift);
    const vVvf inV14 = access.load(in1 + 3 * simd_8.shift);

    const vVvf inV21 = access.load(in2 + 0 * simd_8.shift);
    const vVvf inV22 = access.load(in2 + 1 * simd_8.shift);
    const vVvf inV23 = access.load(in2 + 2 * simd_8.shift);
    const vVvf inV24 = access.load(in2 + 3 * simd_8.shift);

    access.store(out + 0 * simd_8.shift, func(inV11, inV21));
    access.store(out + 1 * simd_8.shift, func(inV12, inV22));
    access.store(out + 2 * simd_8.shift, func(inV13, inV23));
    access.store(out + 3 * simd_8.shift, func(inV14, inV24));

    const vVvf inV31 = access.load(in1 + 4 * simd_8.shift);
    const vVvf inV32 = access.load(in1 + 5 * simd_8.shift);
    const vVvf inV33 = access.load(in1 + 6 * simd_8.shift);
    const vVvf inV34 = access.load(in1 + 7 * simd_8.shift);

    const vVvf inV41 = access.load(in2 + 4 * simd_8.shift);
    const vVvf inV42 = access.load(in2 + 5 * simd_8.shift);
    const vVvf inV43 = access.load(in2 + 6 * simd_8.shift);
    const vVvf inV44 = access.load(in2 + 7 * simd_8.shift);

    access.store(out + 4 * simd_8.shift, func(inV31, inV41));
    access.store(out + 5 * simd_8.shift, func(inV32, inV42));
    access.store(out + 6 * simd_8.shift, func(inV33, inV43));
    access.store(out + 7 * simd_8.shift, func(inV34, inV44));
  }
#pragma omp for nowait
  for(mwIndex elemIdx = simd_8.get_unroll(numElems);
      elemIdx < simd_1.get_unroll(numElems); elemIdx += simd_1.block)
  {
    const vVvf inV11 = access.load(&inData1[elemIdx]);
    const vVvf inV21 = access.load(&inData2[elemIdx]);

    access.store(&outData[elemIdx], func(inV11, inV21));
  }
#pragma omp for nowait
  for(mwIndex elemIdx = simd_1.get_unroll(numElems); elemIdx < numElems; elemIdx++)
  {
    outData[elemIdx] = func(inData1[elemIdx], inData2[elemIdx]);
  }
}

template<typename Type, typename Function, class AccessType>
inline void
cell_inner_cycle_scalar_sse(Type * const __restrict outData,
    const Type * const __restrict inData1,
    const Type inData2,
    const mwSize & numElems, Function & func)
{
  typedef typename SIMDUnrolling<Type>::vVvf vVvf;

  const mwSize unrolling = 8;
  const SIMDUnrolling<Type> simd_unroll(unrolling);
  const SIMDUnrolling<Type> simd(1);

  const vVvf inData2_v = Coeff<Type>::get(inData2);

  AccessType access;

#pragma omp for nowait
  for(mwIndex elemIdx = 0; elemIdx < simd_unroll.get_unroll(numElems);
      elemIdx += simd_unroll.block)
  {
    const vVvf inV11 = access.load(&inData1[elemIdx + 0 * simd_unroll.shift]);
    const vVvf inV12 = access.load(&inData1[elemIdx + 1 * simd_unroll.shift]);
    const vVvf inV13 = access.load(&inData1[elemIdx + 2 * simd_unroll.shift]);
    const vVvf inV14 = access.load(&inData1[elemIdx + 3 * simd_unroll.shift]);

    access.store(&outData[elemIdx + 0 * simd_unroll.shift], func(inV11, inData2_v));
    access.store(&outData[elemIdx + 1 * simd_unroll.shift], func(inV12, inData2_v));
    access.store(&outData[elemIdx + 2 * simd_unroll.shift], func(inV13, inData2_v));
    access.store(&outData[elemIdx + 3 * simd_unroll.shift], func(inV14, inData2_v));

    const vVvf inV31 = access.load(&inData1[elemIdx + 4 * simd_unroll.shift]);
    const vVvf inV32 = access.load(&inData1[elemIdx + 5 * simd_unroll.shift]);
    const vVvf inV33 = access.load(&inData1[elemIdx + 6 * simd_unroll.shift]);
    const vVvf inV34 = access.load(&inData1[elemIdx + 7 * simd_unroll.shift]);

    access.store(&outData[elemIdx + 4 * simd_unroll.shift], func(inV31, inData2_v));
    access.store(&outData[elemIdx + 5 * simd_unroll.shift], func(inV32, inData2_v));
    access.store(&outData[elemIdx + 6 * simd_unroll.shift], func(inV33, inData2_v));
    access.store(&outData[elemIdx + 7 * simd_unroll.shift], func(inV34, inData2_v));
  }
#pragma omp for nowait
  for(mwIndex elemIdx = simd_unroll.get_unroll(numElems);
      elemIdx < simd.get_unroll(numElems); elemIdx += simd.block)
  {
    const vVvf inV11 = access.load(&inData1[elemIdx]);

    access.store(&outData[elemIdx], func(inV11, inData2_v));
  }
#pragma omp for nowait
  for(mwIndex elemIdx = simd.get_unroll(numElems); elemIdx < numElems; elemIdx++)
  {
    outData[elemIdx] = func(inData1[elemIdx], inData2);
  }
}

template<typename Type, const bool allocate, typename Function>
inline void
cell_iteration(mxArray * & outArray, const mxArray * const inArray,
    const Function & func)
{
  if (allocate) {
    cell_allocate(outArray, inArray);
  }

  const mwSize numCells = mxGetNumberOfElements(inArray);
#pragma omp parallel
  {
    for(mwIndex cellIdx = 0; cellIdx < numCells; cellIdx++) {
      const mxArray * const inCell = mxGetCell(inArray, cellIdx);
      mxArray * const outCell = mxGetCell(outArray, cellIdx);

      const mwSize numElems = mxGetNumberOfElements(inCell);
      const Type * const inData = (const Type *) mxGetData(inCell);
      Type * const outData = (Type *) mxGetData(outCell);

#if defined(__SSE2__)
      if (mxGetNumberOfElements(inCell) == 1)
      {
        cell_inner_cycle_scalar_sse< Type, const Function, AccessAligned<Type> >(outData, outData, *inData, numElems, func);
      }
      else
      {
        cell_inner_cycle_sse< Type, const Function, AccessAligned<Type> >(outData, outData, inData, numElems, func);
      }
#else
#pragma omp for
      if (mxGetNumberOfElements(inCell) == 1)
      {
        for(mwIndex elemIdx = 0; elemIdx < numElems; elemIdx++)
        {
          outData[elemIdx] = func(outData[elemIdx], *inData);
        }
      }
      else
      {
        for(mwIndex elemIdx = 0; elemIdx < numElems; elemIdx++)
        {
          outData[elemIdx] = func(outData[elemIdx], inData[elemIdx]);
        }
      }
#endif
    }
  }
}

template<typename Type, typename Function>
inline void
cell_iteration_copy(mxArray * & outArray, const mxArray * const inArray1,
    const mxArray * const inArray2, const Function & func)
{
  cell_allocate(outArray, inArray1);

  const mwSize numCells = mxGetNumberOfElements(inArray1);
#pragma omp parallel
  {
    for(mwIndex cellIdx = 0; cellIdx < numCells; cellIdx++) {
      const mxArray * const inCell1 = mxGetCell(inArray1, cellIdx);
      const mxArray * const inCell2 = mxGetCell(inArray2, cellIdx);
      mxArray * const outCell = mxGetCell(outArray, cellIdx);

      const mwSize numElems = mxGetNumberOfElements(inCell1);
      const Type * const inData1 = (const Type *) mxGetData(inCell1);
      const Type * const inData2 = (const Type *) mxGetData(inCell2);
      Type * const __restrict outData = (Type *) mxGetData(outCell);

#if defined(__SSE2__)
      if (mxGetNumberOfElements(inCell2) == 1)
      {
        cell_inner_cycle_scalar_sse< Type, const Function, AccessAligned<Type> >(outData, inData1, *inData2, numElems, func);
      }
      else
      {
        cell_inner_cycle_sse< Type, const Function, AccessAligned<Type> >(outData, inData1, inData2, numElems, func);
      }
#else
#pragma omp for
      if (mxGetNumberOfElements(inCell2) == 1)
      {
        for(mwIndex elemIdx = 0; elemIdx < numElems; elemIdx++)
        {
          outData[elemIdx] = func(inData1[elemIdx], *inData2);
        }
      }
      else
      {
        for(mwIndex elemIdx = 0; elemIdx < numElems; elemIdx++)
        {
          outData[elemIdx] = func(inData1[elemIdx], inData2[elemIdx]);
        }
      }
#endif
    }
  }
}

//----------------------------------------------------------------------------//

static const char error_slice_scaled_id[] = "C_FUN:sum_slice_scaled_sse:wrong_argument";

static const char error_slice_scaled_wrong_dest_dims[] = "One of the destination arrays is not 3D";
static const char error_slice_scaled_wrong_src_dims[] = "One of the source arrays is not 3D";

static const char error_slice_scaled_wrong_dim_0[] = "Blobs and sinos should have same length along 1st dimension";
static const char error_slice_scaled_wrong_dim_2[] = "Blobs and sinos should have same length along 3rd dimension";

static const char error_slice_scaled_sino_slice_over_hlim[] = "One of destination slices is too large!";
static const char error_slice_scaled_blob_slice_over_hlim[] = "One of source slices is too large!";

static const char error_slice_scaled_sino_slice_over_llim[] = "One of destination slices is less than 1!";
static const char error_slice_scaled_blob_slice_over_llim[] = "One of source slices is less than 1!";

inline bool
check_blobs_and_single_sinogram(const mxArray * const sino,
    const mxArray * const blobs, const mxArray * const coeff)
{
  const mxArray * const sino_offsets_a = mxGetField(coeff, 0, "sino_offsets");
  const mxArray * const blob_offsets_a = mxGetField(coeff, 0, "blob_offsets");
  const mxArray * const proj_offsets_a = mxGetField(coeff, 0, "proj_offsets");
  const mxArray * const proj_coeffs_a = mxGetField(coeff, 0, "proj_coeffs");

  const double * const sino_offsets = (const double *) mxGetData(sino_offsets_a);
  const double * const blob_offsets = (const double *) mxGetData(blob_offsets_a);
  const double * const proj_offsets = (const double *) mxGetData(proj_offsets_a);
  const double * const proj_coeffs = (const double *) mxGetData(proj_coeffs_a);

  const mwSize num_dims_sino = mxGetNumberOfDimensions(sino);
  if (num_dims_sino != 3)
  {
    mexErrMsgIdAndTxt(error_slice_scaled_id, error_slice_scaled_wrong_dest_dims);
    return false;
  }
  const mwSize * const dims_sino = mxGetDimensions(sino);

  const mwSize num_projs = mxGetNumberOfElements(proj_offsets_a);
  for (mwIndex m = 0; m < num_projs; m++)
  {
    const mwIndex p = (const mwIndex) proj_offsets[m]-1;
    const mwIndex b = (const mwIndex) blob_offsets[m]-1;
    const mwIndex s = (const mwIndex) sino_offsets[m]-1;

    const mxArray * const cell_blobs = mxGetCell(blobs, b);

    const mwSize num_dims_blob = mxGetNumberOfDimensions(cell_blobs);
    if (num_dims_blob != 3)
    {
      mexErrMsgIdAndTxt(error_slice_scaled_id, error_slice_scaled_wrong_src_dims);
      return false;
    }

    // Sino and Blob have the same size along 1st and 3rd dimensions
    const mwSize * const dims_blob = mxGetDimensions(cell_blobs);

    if (dims_blob[0] != dims_sino[0])
    {
      mexErrMsgIdAndTxt(error_slice_scaled_id, error_slice_scaled_wrong_dim_0);
      return false;
    }
    if (dims_blob[2] != dims_sino[2])
    {
      mexErrMsgIdAndTxt(error_slice_scaled_id, error_slice_scaled_wrong_dim_2);
      return false;
    }

    if (s >= dims_sino[1])
    {
      mexErrMsgIdAndTxt(error_slice_scaled_id, error_slice_scaled_sino_slice_over_hlim);
      return false;
    }
    if (p >= dims_blob[1])
    {
      mexErrMsgIdAndTxt(error_slice_scaled_id, error_slice_scaled_blob_slice_over_hlim);
      return false;
    }
    if (s < 0)
    {
      mexErrMsgIdAndTxt(error_slice_scaled_id, error_slice_scaled_sino_slice_over_llim);
      return false;
    }
    if (p < 0)
    {
      mexErrMsgIdAndTxt(error_slice_scaled_id, error_slice_scaled_blob_slice_over_llim);
      return false;
    }
  }
  return true;
}

inline bool
check_blobs_and_sinograms(const mxArray * const sinos,
    const mxArray * const blobs, const mxArray * const coeffs)
{
  const mwSize num_cells = mxGetNumberOfElements(sinos);
  for(mwIndex cell_idx = 0; cell_idx < num_cells; cell_idx++)
  {
    mxArray * const cell_sinos = mxGetCell(sinos, cell_idx);
    const mxArray * const cell_coeffs = mxGetCell(coeffs, cell_idx);

    if (!check_blobs_and_single_sinogram(cell_sinos, blobs, cell_coeffs))
    {
      return false;
    }
  }
  return true;
}

template<typename Type, typename AccessDest, typename AccessSrc>
inline void
sum_slice_scaled_do_line_sse(const Type * const src_data_line,
    Type * const dest_data_line, const mwSize & line_length,
    inner_sum_scale<Type> & op,
    const SIMDUnrolling<Type> & simd_unroll, const SIMDUnrolling<Type> & simd,
    AccessDest & access_dest, AccessSrc & access_src)
{
  typedef typename SIMDUnrolling<Type>::vVvf vVvf;

  for (mwIndex elem = 0; elem < simd_unroll.get_unroll(line_length);
      elem += simd_unroll.block)
  {
    const vVvf src_0 = access_src.load(&src_data_line[elem + 0*simd_unroll.shift]);
    const vVvf src_1 = access_src.load(&src_data_line[elem + 1*simd_unroll.shift]);
    const vVvf src_2 = access_src.load(&src_data_line[elem + 2*simd_unroll.shift]);
    const vVvf src_3 = access_src.load(&src_data_line[elem + 3*simd_unroll.shift]);

    const vVvf dest_0 = access_dest.load(&dest_data_line[elem + 0*simd_unroll.shift]);
    const vVvf dest_1 = access_dest.load(&dest_data_line[elem + 1*simd_unroll.shift]);
    const vVvf dest_2 = access_dest.load(&dest_data_line[elem + 2*simd_unroll.shift]);
    const vVvf dest_3 = access_dest.load(&dest_data_line[elem + 3*simd_unroll.shift]);

    access_dest.store(&dest_data_line[elem + 0*simd_unroll.shift], op(dest_0, src_0));
    access_dest.store(&dest_data_line[elem + 1*simd_unroll.shift], op(dest_1, src_1));
    access_dest.store(&dest_data_line[elem + 2*simd_unroll.shift], op(dest_2, src_2));
    access_dest.store(&dest_data_line[elem + 3*simd_unroll.shift], op(dest_3, src_3));
  }
  for (mwIndex elem = simd_unroll.get_unroll(line_length);
      elem < simd.get_unroll(line_length); elem += simd.block)
  {
    const vVvf src_0 = access_src.load(&src_data_line[elem]);
    const vVvf dest_0 = access_dest.load(&dest_data_line[elem]);
    access_dest.store(&dest_data_line[elem], op(dest_0, src_0));
  }
  for (mwIndex elem = simd.get_unroll(line_length); elem < line_length; elem++)
  {
    dest_data_line[elem] = op(dest_data_line[elem], src_data_line[elem]);
  }
}

template<typename Type>
inline void
sum_slice_scaled_sse(mxArray * dest, const mwIndex & dest_slice,
    const mxArray * const src, const mwIndex & src_slice, const Type & c)
{
  const mwSize num_dims_dest = mxGetNumberOfDimensions(dest);
  const mwSize num_dims_src = mxGetNumberOfDimensions(src);

  const mwSize * const dims_dest = mxGetDimensions(dest);
  const mwSize * const dims_src = mxGetDimensions(src);

  // Sino and Blob have the same size along 1st and 3rd dimensions
  const mwSize & line_length = dims_dest[0];
  const mwSize & lines_num = dims_dest[2];

  const mwSize dest_initial_offset = line_length * dest_slice;
  const mwSize src_initial_offset = line_length * src_slice;

  const mwSize dest_skip = dims_dest[0] * dims_dest[1];
  const mwSize src_skip = dims_src[0] * dims_src[1];

  const Type * const src_data_base = ((const Type *) mxGetData(src)) + src_initial_offset;
  Type * const dest_data_base = ((Type *) mxGetData(dest)) + dest_initial_offset;

  inner_sum_scale<Type> op(c);

  typedef typename SIMDUnrolling<Type>::vVvf vVvf;

  const mwSize unrolling = 4;
  const SIMDUnrolling<Type> simd_unroll(unrolling);
  const SIMDUnrolling<Type> simd(1);

  AccessAligned<Type> align;
  AccessUnaligned<Type> unalign;

  for (mwIndex line_num = 0; line_num < lines_num; line_num++)
  {
    const Type * const src_data_line = src_data_base + line_num * src_skip;
    Type * const dest_data_line = dest_data_base + line_num * dest_skip;

    sum_slice_scaled_do_line_sse<Type, AccessUnaligned<Type>, AccessUnaligned<Type> >(
        src_data_line, dest_data_line, line_length,
        op, simd_unroll, simd, unalign, unalign);
  }
}

template<typename Type>
inline void
do_blobs_to_single_sinogram_sse(mxArray * sino, const mxArray * const blobs,
    const mxArray * const coeffs)
{
  const mxArray * const sino_offsets_a = mxGetField(coeffs, 0, "sino_offsets");
  const mxArray * const blob_offsets_a = mxGetField(coeffs, 0, "blob_offsets");
  const mxArray * const proj_offsets_a = mxGetField(coeffs, 0, "proj_offsets");
  const mxArray * const proj_coeffs_a = mxGetField(coeffs, 0, "proj_coeffs");

  const double * const sino_offsets = (const double *) mxGetData(sino_offsets_a);
  const double * const blob_offsets = (const double *) mxGetData(blob_offsets_a);
  const double * const proj_offsets = (const double *) mxGetData(proj_offsets_a);
  const double * const proj_coeffs = (const double *) mxGetData(proj_coeffs_a);

  const mwSize num_projs = mxGetNumberOfElements(proj_offsets_a);
#pragma omp for
  for (mwIndex m = 0; m < num_projs; m++)
  {
    const mwIndex p = (const mwIndex) proj_offsets[m]-1;
    const mwIndex b = (const mwIndex) blob_offsets[m]-1;
    const mwIndex s = (const mwIndex) sino_offsets[m]-1;

    const Type & c = (const Type) proj_coeffs[m];

    const mxArray * const cell_blobs = mxGetCell(blobs, b);

    sum_slice_scaled_sse<Type>(sino, s, cell_blobs, p, c);
  }
}

template<typename Type>
inline void
do_cell_blobs_to_sinograms_sse(mxArray * sinos, const mxArray * const blobs,
    const mxArray * const coeffs)
{
  const mwSize num_cells = mxGetNumberOfElements(sinos);
  for(mwIndex index = 0; index < num_cells; index++)
  {
    mxArray * const sino = mxGetCell(sinos, index);
    const mxArray * const coeff = mxGetCell(coeffs, index);

    do_blobs_to_single_sinogram_sse<Type>(sino, blobs, coeff);
  }
}

template<typename Type>
inline void
do_single_sinogram_to_blobs_sse(mxArray * blobs, const mxArray * const sino,
    const mxArray * const coeffs)
{
  const mxArray * const sino_offsets_a = mxGetField(coeffs, 0, "sino_offsets");
  const mxArray * const blob_offsets_a = mxGetField(coeffs, 0, "blob_offsets");
  const mxArray * const proj_offsets_a = mxGetField(coeffs, 0, "proj_offsets");
  const mxArray * const proj_coeffs_a = mxGetField(coeffs, 0, "proj_coeffs");

  const double * const sino_offsets = (const double *) mxGetData(sino_offsets_a);
  const double * const blob_offsets = (const double *) mxGetData(blob_offsets_a);
  const double * const proj_offsets = (const double *) mxGetData(proj_offsets_a);
  const double * const proj_coeffs = (const double *) mxGetData(proj_coeffs_a);

  const mwSize num_projs = mxGetNumberOfElements(proj_offsets_a);
#pragma omp for
  for (mwIndex m = 0; m < num_projs; m++)
  {
    const mwIndex p = (const mwIndex) proj_offsets[m]-1;
    const mwIndex b = (const mwIndex) blob_offsets[m]-1;
    const mwIndex s = (const mwIndex) sino_offsets[m]-1;

    const Type & c = (const Type) proj_coeffs[m];

    mxArray * const cell_blobs = mxGetCell(blobs, b);

    sum_slice_scaled_sse<Type>(cell_blobs, p, sino, s, c);
  }
}

template<typename Type>
inline void
do_cell_sinograms_to_blobs_sse(mxArray * blobs, const mxArray * const sinos,
    const mxArray * const coeffs)
{
  const mwSize num_cells = mxGetNumberOfElements(sinos);
  for(mwIndex index = 0; index < num_cells; index++)
  {
    const mxArray * const sino = mxGetCell(sinos, index);
    const mxArray * const coeff = mxGetCell(coeffs, index);

    do_single_sinogram_to_blobs_sse<Type>(blobs, sino, coeff);
  }
}

template<typename Type>
inline void
cell_initialize_single_sse(mxArray * const array, const Type & val)
{
  typedef typename SIMDUnrolling<Type>::vVvf vVvf;

  const mwSize unrolling = 8;
  const SIMDUnrolling<Type> simd_unroll(unrolling);
  const SIMDUnrolling<Type> simd(1);

  const vVvf vec_val = Coeff<Type>::get(val);

  AccessAligned<Type> access;
  //AccessStreamed<Type> access;

  Type * __restrict data = (Type *) mxGetData(array);
  const mwSize num_elems = mxGetNumberOfElements(array);

#pragma omp for nowait
  for (mwIndex elem_count = 0; elem_count < simd_unroll.get_unroll(num_elems);
      elem_count += simd_unroll.block)
  {
    access.store(&data[elem_count + 0 * simd_unroll.shift], vec_val);
    access.store(&data[elem_count + 1 * simd_unroll.shift], vec_val);
    access.store(&data[elem_count + 2 * simd_unroll.shift], vec_val);
    access.store(&data[elem_count + 3 * simd_unroll.shift], vec_val);

    access.store(&data[elem_count + 4 * simd_unroll.shift], vec_val);
    access.store(&data[elem_count + 5 * simd_unroll.shift], vec_val);
    access.store(&data[elem_count + 6 * simd_unroll.shift], vec_val);
    access.store(&data[elem_count + 7 * simd_unroll.shift], vec_val);
  }
#pragma omp for nowait
  for (mwIndex elem_count = simd_unroll.get_unroll(num_elems);
      elem_count < simd.get_unroll(num_elems); elem_count += simd.block)
  {
    access.store(&data[elem_count], vec_val);
  }
#pragma omp for nowait
  for (mwIndex elem_count = simd.get_unroll(num_elems); elem_count < num_elems; elem_count++)
  {
    data[elem_count] = val;
  }
}

template<typename Type>
inline void
cell_initialize_sse(mxArray * const array, const Type & val)
{
  typedef typename SIMDUnrolling<Type>::vVvf vVvf;

  const mwSize unrolling = 8;
  const SIMDUnrolling<Type> simd_unroll(unrolling);
  const SIMDUnrolling<Type> simd(1);

  const vVvf vec_val = Coeff<Type>::get(val);

  AccessAligned<Type> access;
  //AccessStreamed<Type> access;

  const mwSize num_cells = mxGetNumberOfElements(array);
#pragma omp for
  for(mwIndex cell_idx = 0; cell_idx < num_cells; cell_idx++)
  {
    mxArray * cell = mxGetCell(array, cell_idx);
    Type * __restrict data = (Type *) mxGetData(cell);
    const mwSize num_elems = mxGetNumberOfElements(cell);

    for (mwIndex elem_count = 0; elem_count < simd_unroll.get_unroll(num_elems);
        elem_count += simd_unroll.block)
    {
      access.store(&data[elem_count + 0 * simd_unroll.shift], vec_val);
      access.store(&data[elem_count + 1 * simd_unroll.shift], vec_val);
      access.store(&data[elem_count + 2 * simd_unroll.shift], vec_val);
      access.store(&data[elem_count + 3 * simd_unroll.shift], vec_val);

      access.store(&data[elem_count + 4 * simd_unroll.shift], vec_val);
      access.store(&data[elem_count + 5 * simd_unroll.shift], vec_val);
      access.store(&data[elem_count + 6 * simd_unroll.shift], vec_val);
      access.store(&data[elem_count + 7 * simd_unroll.shift], vec_val);
    }
    for (mwIndex elem_count = simd_unroll.get_unroll(num_elems);
        elem_count < simd.get_unroll(num_elems); elem_count += simd.block)
    {
      access.store(&data[elem_count], vec_val);
    }
    for (mwIndex elem_count = simd.get_unroll(num_elems); elem_count < num_elems; elem_count++)
    {
      data[elem_count] = val;
    }
  }
}


#endif /* INTERNAL_CELL_DEFS_H_ */
