classdef Gt6DReconstructionAlgorithmFactory < handle
    properties
        data_type = 'single';

        det_index = 1;

        volume_downscaling = 1;
        rspace_super_sampling = 1;
        rspace_oversize = 1.2;

        use_predicted_scatter_ints = false;

        % Different types of shape functions:
        %   'none'   : no shape functions are used
        %   'w'      : Only omega is considered
        %   'nw'     : (eta, omega) shape functions
        shape_functions_type = 'none';

        % Different types of num_interp:
        %   'flat'    : all blobs are inteprolated with the num_interp
        %   'lorentz' : num_interp is rescaled by the Lorentz factor
        num_interp_type = 'lorentz';

        parameters;
    end

    methods (Access = public)
        function self = Gt6DReconstructionAlgorithmFactory(parameters, varargin)
            self = self@handle();

            self = parse_pv_pairs(self, varargin);

            self.parameters = parameters;
        end

        function [algo, blobs] = getReconstructionAlgo(self, sampler, num_interp, varargin)
            if (numel(sampler) == 1)
                [algo, blobs] = self.get6DAlgorithmSingleRegion(sampler, num_interp, varargin{:});
            else
                [algo, blobs] = self.get6DAlgorithmMultiRegion(sampler, num_interp, varargin{:});
            end
        end

        function [algo, blobs] = get6DAlgorithmSingleRegion(self, sampler, num_interp, varargin)
            % Build Empty volumes
            ref_gr = sampler.get_reference_grain();
            volume_size = self.get_volume_size(ref_gr);

            % Deciding the type of num_interp type
            [blobs_w_interp, num_interp] = self.compute_num_interp(num_interp, sampler);

            shape_funcs = self.get_shape_functions(sampler, num_interp, blobs_w_interp);

            algo_params = self.get_algorithm_params(sampler, blobs_w_interp, shape_funcs);

            blobs = algo_params.blobs;
            or_groups = [1, sampler.get_total_num_orientations()];

            algo = Gt6DBlobReconstructor(volume_size, blobs, ...
                'data_type', self.data_type, ...
                'geometries', algo_params.geometries, ...
                'offsets', algo_params.offsets, ...
                'ss_geometries', algo_params.geometries_ss, ...
                'ss_offsets', algo_params.offsets_ss, ...
                'volume_ss', self.volume_downscaling, ...
                'rspace_ss', self.rspace_super_sampling, ...
                'psf', algo_params.psf, ...
                'orientation_groups', or_groups, ...
                varargin{:} );
        end

        function [algo, blobs] = get6DAlgorithmMultiRegion(self, sampler, num_interp, varargin)
            num_regions = numel(sampler);

            % Build Empty volumes
            ref_gr = sampler(1).get_reference_grain();
            volume_size = self.get_volume_size(ref_gr);

            ref_inc = ref_gr.proj(self.det_index).included;
            ref_sel = ref_gr.proj(self.det_index).selected;

            % This vector will tell us where we will find in the blobs, the
            % ones that were selected among the included, because the
            % information we have in the twins about the spots that
            % coincide are about the included and not the selected
            ref_inc_to_sel = zeros(size(ref_inc));
            ref_inc_to_sel(ref_sel) = 1:numel(find(ref_sel));

            blobs_w_interp = cell(num_regions, 1);

            for ii_o_reg = 1:num_regions
                gr = sampler(ii_o_reg).get_reference_grain();
                fprintf('%d) Grainid %d:\n', ii_o_reg, gr.id)

                % Deciding the type of num_interp type
                [blobs_w_interp{ii_o_reg}, num_interp] = self.compute_num_interp(num_interp, sampler(ii_o_reg));

                % assigning to the twin the blob_w_interp of the parent for
                % the shared reflections
                if (ii_o_reg > 1)
                    [shared, parent_pos_shared_bls] = ...
                        self.get_shared_bls(ref_inc_to_sel, sampler, ii_o_reg);

                    blobs_w_interp{ii_o_reg}(shared) = blobs_w_interp{1}(parent_pos_shared_bls);

                    w_interp_mismatch = blobs_w_interp{1}(parent_pos_shared_bls) ~= blobs_w_interp{ii_o_reg}(shared);
                    if (any(w_interp_mismatch))
                        % If this happens, there is a bug in the code somewhere
                        warning('Gt6DReconstructionAlgorithmFactory:wrong_interpolation', ...
                            ['For some reason, some of the num_interp used', ...
                            ' for the main orientation-space region and ', ...
                            'the region n.%d, are different.\n', ...
                            'The reconstruction will not be correct!!'], ...
                            ii_o_reg)
                        fprintf(' - Parent blob position: %d num-interp: %d, twin blob position: %d num-interp: %d\n', ...
                            [parent_pos_shared_bls, blobs_w_interp{1}(parent_pos_shared_bls), ...
                            find(shared), blobs_w_interp{ii_o_reg}(shared)]')
                    end
                end

                shape_funcs = self.get_shape_functions(sampler(ii_o_reg), num_interp, blobs_w_interp{ii_o_reg});

                algo_params(ii_o_reg) = self.get_algorithm_params( ...
                    sampler(ii_o_reg), blobs_w_interp{ii_o_reg}, shape_funcs); %#ok<AGROW>
                fprintf('\n')
            end

            blobs = cat(1, algo_params(:).blobs);

            psf = cat(1, algo_params(:).psf);
%             psf = fspecial('gaussian', 21, 0.75) + 2 * fspecial('gaussian', 21, 4);
%             psf = psf / sum(psf(:));
%             psf = { permute(psf, [1 3 2]) };

            geometries = cat(1, algo_params(:).geometries);
            geometries_ss = cat(1, algo_params(:).geometries_ss);

            offsets = algo_params(1).offsets;
            offsets_ss = algo_params(1).offsets_ss;

            num_parent_blobs = numel(algo_params(1).blobs);

            % We should first go through all the shared blobs in each
            % orientation and extend them if needed, and then extend the
            % positions pointed by the orientations in the shared blobs
            all_shared_blobs_sel = find(ref_gr.proj(self.det_index).shared_bls_with_twins(ref_sel));

            % The reference num_interps
            shared_blobs_w_interp = zeros(num_parent_blobs, 1);
            shared_blobs_w_interp(all_shared_blobs_sel) = blobs_w_interp{1}(all_shared_blobs_sel);

            % A record for the W limits of shared blobs between parent and
            % twins
            shared_bls_w_lims = zeros(num_parent_blobs, 2, num_regions);
            shared_bls_w_lims(all_shared_blobs_sel, :, 1) = algo_params(1).blobs_w_lims(all_shared_blobs_sel, :) ./ shared_blobs_w_interp(all_shared_blobs_sel, [1 1]);
            rec_shared_bls_w_lims = shared_bls_w_lims(:, :, 1);

            for ii_o_reg = 2:num_regions
                [shared, parent_pos_shared_bls] = ...
                    self.get_shared_bls(ref_inc_to_sel, sampler, ii_o_reg);

                shared_bls_w_lims(parent_pos_shared_bls, :, ii_o_reg) = algo_params(ii_o_reg).blobs_w_lims(shared, :) ...
                    ./ shared_blobs_w_interp(parent_pos_shared_bls, [1 1]);

                rec_shared_bls_w_lims(parent_pos_shared_bls, :) = [ ...
                    min(rec_shared_bls_w_lims(parent_pos_shared_bls, 1), shared_bls_w_lims(parent_pos_shared_bls, 1, ii_o_reg)), ...
                    max(rec_shared_bls_w_lims(parent_pos_shared_bls, 2), shared_bls_w_lims(parent_pos_shared_bls, 2, ii_o_reg)) ];
            end

            to_be_changed = any(rec_shared_bls_w_lims(all_shared_blobs_sel, :) ~= shared_bls_w_lims(all_shared_blobs_sel, :, 1), 2);

            rec_rescaled_shared_blobs_sizes = rec_shared_bls_w_lims(all_shared_blobs_sel, 2) - rec_shared_bls_w_lims(all_shared_blobs_sel, 1) + 1;

            fprintf('Shared blobs:\n')
            % And now let's extend those blobs that need it!
            for ii_b = 1:numel(all_shared_blobs_sel)
                shared_blob_pos_inc = all_shared_blobs_sel(ii_b);

                % Collecting IDs of other regions using the said blobs
                reg_ids = zeros(num_regions, 1);
                for ii_o_reg = 2:num_regions
                    [~, parent_pos_shared_bls] = ...
                        self.get_shared_bls(ref_inc_to_sel, sampler, ii_o_reg);

                    if (~isempty(find(parent_pos_shared_bls == shared_blob_pos_inc, 1)))
                        curr_or = sampler(ii_o_reg).get_reference_grain();
                        reg_ids(ii_o_reg) = curr_or.id;
                    end
                end
                reg_ids = reg_ids(reg_ids > 0);

                twins_string = sprintf(' %d', reg_ids);
                fprintf('%02d) Blob in parent: %d, shared with twins:%s\n', ii_b, shared_blob_pos_inc, twins_string)

                % if they changed, let's enlarge them
                if (to_be_changed(ii_b))
                    proj_shift = shared_bls_w_lims(shared_blob_pos_inc, 1, 1) ...
                        - rec_shared_bls_w_lims(shared_blob_pos_inc, 1);

                    curr_blob_size = size(blobs{shared_blob_pos_inc});
                    new_size = [curr_blob_size(1), rec_rescaled_shared_blobs_sizes(ii_b), curr_blob_size(3)];

                    fprintf('  * Enlarging blob, depth: %d -> %d, with shift: %d\n', ...
                        curr_blob_size(2), new_size(2), proj_shift)

                    % Creating the new blob
                    new_blob = zeros(new_size, 'like', blobs{shared_blob_pos_inc});
                    new_blob = gtPlaceSubVolume(new_blob, blobs{shared_blob_pos_inc}, [0, proj_shift, 0]);
                    blobs{shared_blob_pos_inc} = new_blob;

                    % Fixing the shift in parent's projection
                    if (~isempty(geometries_ss))
                        for ii_o = 1:numel(offsets_ss)
                            for ii_ss = 1:numel(offsets_ss{ii_o})
                                inds_of_shared_blobs = offsets_ss{ii_o}{ii_ss}.blob_offsets == shared_blob_pos_inc;

                                offsets_ss{ii_o}{ii_ss}.proj_offsets(inds_of_shared_blobs) ...
                                    = offsets_ss{ii_o}{ii_ss}.proj_offsets(inds_of_shared_blobs) + proj_shift;
                            end
                        end
                    else
                        for ii_o = 1:numel(offsets)
                            inds_of_shared_blobs = offsets{ii_o}.blob_offsets == shared_blob_pos_inc;

                            offsets{ii_o}.proj_offsets(inds_of_shared_blobs) ...
                                = offsets{ii_o}.proj_offsets(inds_of_shared_blobs) + proj_shift;
                        end
                    end
                end
            end
            fprintf('\n')

            base_offset_shift = num_parent_blobs;
            for ii_o_reg = 2:num_regions
                [shared, parent_pos_shared_bls, shared_pos] = ...
                    self.get_shared_bls(ref_inc_to_sel, sampler, ii_o_reg);

                % This shift is related to the new size of the shared
                % blobs, to accomodate every other sampled region in
                % orientation-space
                proj_shifts = ...
                    shared_bls_w_lims(parent_pos_shared_bls, 1, ii_o_reg) ...
                    - rec_shared_bls_w_lims(parent_pos_shared_bls, 1);
                blobs_with_projs_to_be_shifted = find(proj_shifts > 0);

                % Shifting projection to the concatenated blobs, unless that
                % blob was shared between parent and twin
                if (~isempty(geometries_ss))
                    proj_coeffs_ss = algo_params(ii_o_reg).offsets_ss;
                    for ii_o = 1:numel(proj_coeffs_ss)
                        for ii_ss = 1:numel(proj_coeffs_ss{ii_o})
                            % Indices of the blobs, that we project to, in
                            % curr_or
                            inds_of_blobs = proj_coeffs_ss{ii_o}{ii_ss}.blob_offsets;

                            % to_be_shifted contains the indices to the
                            % blobs belonging only to the current region
                            % identified by ii_o_reg
                            to_be_shifted = ~shared(inds_of_blobs);

                            % The indices of the given blobs belonging to
                            % the current orientation will be shifted by
                            % the number of blobs accumulated already
                            inds_of_blobs(to_be_shifted) ...
                                = inds_of_blobs(to_be_shifted) + base_offset_shift;

                            % Treating the shared blobs
                            inds_of_blobs(~to_be_shifted) ...
                                = ref_inc_to_sel(shared_pos(inds_of_blobs(~to_be_shifted)));

                            proj_coeffs_ss{ii_o}{ii_ss}.blob_offsets = inds_of_blobs;

                            % And now taking care of the positions inside
                            % those blobs
                            for ii_b = 1:numel(blobs_with_projs_to_be_shifted)
                                ind_of_parent_blob = parent_pos_shared_bls(blobs_with_projs_to_be_shifted(ii_b));
                                inds_of_shared_blobs = inds_of_blobs == ind_of_parent_blob;

                                proj_coeffs_ss{ii_o}{ii_ss}.proj_offsets(inds_of_shared_blobs) = ...
                                    proj_coeffs_ss{ii_o}{ii_ss}.proj_offsets(inds_of_shared_blobs) + proj_shifts(blobs_with_projs_to_be_shifted(ii_b));
                            end
                        end
                    end
                    offsets_ss = cat(1, offsets_ss, proj_coeffs_ss);
                else
                    proj_coeffs = algo_params(ii_o_reg).offsets;
                    for ii_o = 1:numel(proj_coeffs)
                        % Indices of the blobs, that we project to, in
                        % curr_or
                        inds_of_blobs = proj_coeffs{ii_o}.blob_offsets;

                        % to_be_shifted contains the indices to the
                        % blobs belonging only to the current region
                        % identified by ii_o_reg
                        to_be_shifted = ~shared(inds_of_blobs);

                        % The indices of the given blobs belonging to
                        % the current orientation will be shifted by
                        % the number of blobs accumulated already
                        inds_of_blobs(to_be_shifted) ...
                            = inds_of_blobs(to_be_shifted) + base_offset_shift;

                        % Treating the shared blobs
                        inds_of_blobs(~to_be_shifted) ...
                            = ref_inc_to_sel(shared_pos(inds_of_blobs(~to_be_shifted)));

                        proj_coeffs{ii_o}.blob_offsets = inds_of_blobs;

                        % And now taking care of the positions inside
                        % those blobs
                        for ii_b = 1:numel(blobs_with_projs_to_be_shifted)
                            ind_of_parent_blob = parent_pos_shared_bls(blobs_with_projs_to_be_shifted(ii_b));
                            inds_of_shared_blobs = inds_of_blobs == ind_of_parent_blob;

                            proj_coeffs{ii_o}.proj_offsets(inds_of_shared_blobs) = ...
                                proj_coeffs{ii_o}.proj_offsets(inds_of_shared_blobs) + proj_shifts(blobs_with_projs_to_be_shifted(ii_b));
                        end
                    end
                    offsets = cat(1, offsets, proj_coeffs);
                end

                base_offset_shift = base_offset_shift + numel(algo_params(ii_o_reg).blobs);
            end

            or_groups = zeros(num_regions, 2);
            or_groups(1, 1) = 1;
            for ii_o_reg = 2:num_regions
                or_groups(ii_o_reg, 1) = or_groups(ii_o_reg-1) + sampler(ii_o_reg-1).get_total_num_orientations();
            end
            or_groups(1:end-1, 2) = or_groups(2:end, 1) - 1;
            or_groups(end, 2) = or_groups(end, 1) + sampler(end).get_total_num_orientations() - 1;

            algo = Gt6DBlobReconstructor(volume_size, blobs, ...
                'data_type', self.data_type, ...
                'geometries', geometries, ...
                'offsets', offsets, ...
                'ss_geometries', geometries_ss, ...
                'ss_offsets', offsets_ss, ...
                'volume_ss', self.volume_downscaling, ...
                'psf', psf, ...
                'orientation_groups', or_groups, ...
                varargin{:} );
        end
    end

    methods (Access = protected)
        function shape_funcs = get_shape_functions(self, sampler, num_interp, blobs_w_interp)
            switch (lower(self.shape_functions_type))
                case 'none'
                    shape_funcs = {};
                case 'n'
%                     shape_funcs = gtDefShapeFunctionsCreateNW(sampler);
                    shape_funcs = gtDefShapeFunctionsFwdProj(sampler, ...
                        'shape_function_type', 'nw', ...
                        'num_interp', num_interp, ...
                        'blobs_w_interp', blobs_w_interp);
                    shape_funcs = gtDefShapeFunctionsNW2N(shape_funcs);
                case 'w'
%                     shape_funcs = gtDefShapeFunctionsCreateW(sampler);
                    shape_funcs = gtDefShapeFunctionsFwdProj(sampler,  ...
                        'shape_function_type', 'w', ...
                        'num_interp', num_interp, ...
                        'blobs_w_interp', blobs_w_interp);
                case 'nw'
%                     shape_funcs = gtDefShapeFunctionsCreateNW(sampler);
                    shape_funcs = gtDefShapeFunctionsFwdProj(sampler, ...
                        'shape_function_type', 'nw', ...
                        'num_interp', num_interp, ...
                        'blobs_w_interp', blobs_w_interp);
                otherwise
                    error('Gt6DReconstructionAlgorithmFactory:wrong_argument', ...
                        'Shape functions of type: %s, not supported yet!', ...
                        self.shape_functions_type)
            end
        end

        function [shared, parent_corresp_shared_blobs, shared_pos] = get_shared_bls(self, selected_pos_in_included, sampler, ii_o_reg)
            curr_or = sampler(ii_o_reg).get_reference_grain();
            curr_sel = sampler(ii_o_reg).selected;
            % This contains the position of the shared blobs in
            % parent's numbering
            shared_pos = curr_or.proj(self.det_index).shared_bls_with_parent(curr_sel);
            % This indicates the ones that are actually shared among
            % the twin's blobs
            shared = shared_pos > 0;

            parent_corresp_shared_blobs = selected_pos_in_included(shared_pos(shared));
        end

        function algo_params = get_algorithm_params(self, sampler, blobs_w_interp, shape_funcs)
            fprintf('Extracting blobs on detector: ')
            c = tic();
            sel_bls = sampler.selected;
            bl = sampler.bl(sel_bls);
            fprintf('\b\b (%f s).\n', toc(c));
            tot_blobs = numel(bl);
            sel_incl_indx = find(sel_bls);

            ref_gr = sampler.get_reference_grain();
            proj = ref_gr.proj(self.det_index);

            sel_allb_idxs = sampler.ondet(sampler.included(sel_bls));
            ref_omegas = ref_gr.allblobs.omega(sel_allb_idxs);

            [orients, orients_ss] = sampler.get_orientations();

            num_ospace_oversampling = prod(sampler.ss_factor);
            if (num_ospace_oversampling == 1)
                ors_struct = [orients{:}];
            else
                ors_struct = cat(4, orients_ss{:});
                ors_struct = [ors_struct{:}];
            end
            % Array of structures that contain all the info relative to
            % each orientation, for each blob
            ors_allblobs = [ors_struct(:).allblobs];

            % Finding the projection limits in W, and taking care of the
            % 360 degrees wrapping
            switch (lower(self.shape_functions_type))
                case {'none', 'n'}
                    [w_tab_lims, w_tab, ref_ws] = self.get_w_tab_numinterp(ors_allblobs, ref_omegas);
                case {'w', 'nw'}
                    [w_tab_lims, ref_ws] = self.get_w_tab_shapefuncs(shape_funcs, ref_omegas);
            end

            [blobs_w_lims_pad, projs_w_lims, paddings] = ...
                self.get_blob_lims(w_tab_lims, bl, ref_ws, blobs_w_interp);

            blobs = self.pad_blobs(bl, paddings, blobs_w_interp);
            if (self.use_predicted_scatter_ints)
                blobs = self.renormalize_blobs(sampler, blobs);
            end

            % Let's restrict blobs to the reached regions
            for ii_b = 1:tot_blobs
                fprintf('Blob: %03d (n. %03d) --> Blob w lims %4d:%4d -> %4d:%4d, Projs w lims %4d:%4d, Paddings %2d:%2d, Size 1:%2d, Numinterp %2d -> Intensity: %3.3f\n', ...
                    ii_b, sel_incl_indx(ii_b), ...
                    bl(ii_b).bbwim, blobs_w_lims_pad(ii_b, :),...
                    round(projs_w_lims(ii_b, :)), ...
                    paddings(ii_b, :), size(blobs{ii_b}, 2), ...
                    blobs_w_interp(ii_b), gtMathsSumNDVol(blobs{ii_b}));
            end

            fprintf('Computing ASTRA proj geometry and blobs <-> sinograms coefficients: ')
            c = tic();
            % ASTRA Geometry
            switch (lower(self.shape_functions_type))
                case 'none'
                    geometries = self.compute_proj_geom_numinterp(bl, ...
                        ref_gr, ors_allblobs);

                    offsets = self.compute_proj_coeffs_numinterp(...
                        blobs_w_lims_pad, w_tab, blobs_w_interp);
                case 'n'
                    error('Gt6DReconstructionAlgorithmFactory:wrong_argument', ...
                        'Projection geometry with only N-shape-functions, is not implemented, yet!')
                case 'w'
                    [geometries, offsets] = self.compute_proj_geom_shapefuncs_w( ...
                        bl, ref_gr, ors_allblobs, blobs_w_lims_pad, shape_funcs, blobs_w_interp);
                case 'nw'
                    [geometries, offsets] = self.compute_proj_geom_shapefuncs_nw( ...
                        bl, ref_gr, ors_allblobs, blobs_w_lims_pad, shape_funcs, blobs_w_interp);
            end
            fprintf('Done (%f s).\n', toc(c));

            % Taking care of the orientation-space super-sampling
            tot_orient = sampler.get_total_num_orientations();
            if (num_ospace_oversampling == 1)
                geometries_ss = {};
                offsets_ss = {};
            else
                geometries = reshape(geometries, num_ospace_oversampling, []);
                offsets = reshape(offsets, num_ospace_oversampling, []);

                geometries_ss = cell(tot_orient, 1);
                offsets_ss = cell(tot_orient, 1);
                for ii_or = 1:tot_orient
                    geometries_ss{ii_or} = geometries(:, ii_or);

                    % merging all the orientation-space super-sampling
                    % projection coefficients into one structure per
                    % orientation. This is needed in order to fwd/bwd-project
                    % all the different sub-orientations together.
                    offsets_ss{ii_or} = self.merge_offsets(offsets(:, ii_or));
                end

                geometries = {};
                offsets = {};
            end

            if (isfield(bl, 'psf'))
                psf = arrayfun(@(x){ permute(x.psf, [1 3 2]) }, bl);
            elseif (isfield(proj, 'psf'))
                psf = { permute(proj.psf, [1 3 2]) };
            else
                psf = {};
            end

            algo_params = struct( ...
                'blobs', {blobs}, ...
                'geometries', {geometries}, ...
                'offsets', {offsets}, ...
                'geometries_ss', {geometries_ss}, ...
                'offsets_ss', {offsets_ss}, ...
                'psf', {psf}, ...
                'blobs_w_lims', {blobs_w_lims_pad} );
        end

        function volume_size = get_volume_size(self, ref_gr)
            proj = ref_gr.proj(self.det_index);

            spacing = mean([proj.vol_size_y, proj.vol_size_x, proj.vol_size_z]) * (self.rspace_oversize - 1);
            volume_size = ceil([proj.vol_size_y, proj.vol_size_x, proj.vol_size_z] + spacing);

            if (self.volume_downscaling > 1)
                volume_size = ceil(volume_size / self.volume_downscaling);
            end
        end

        function [blobs_w_interp, num_interp] = compute_num_interp(self, num_interp, sampler)
            ref_gr = sampler.get_reference_grain();
            selected = sampler.ondet(sampler.included(sampler.selected));
            etas = ref_gr.allblobs.eta(selected);

            if (num_interp < 0)
                num_interp = abs(num_interp) * sampler.minimum_num_interp();
            elseif (~num_interp)
                num_interp = sampler.suggest_num_interp();
            end

            if (strcmpi(self.num_interp_type, 'lorentz'))
                lorentz_factor = 1 ./ abs(sind(etas));
                % Let's try to not penalize rounding errors and being a bit
                % tollerant with small deviations from integer numbers
                % (rounding in [-0.75 0.25), instead of [-0.5 0.5) ), but
                % still trying to avoid gaps in the projections
                blobs_w_interp = ceil(num_interp .* (lorentz_factor - 0.25));
            else
                blobs_w_interp = num_interp(ones(numel(etas), 1), 1);
            end
        end

        function pad_interp_blobs = pad_blobs(self, blobs, paddings, slices_interp)
            num_blobs = numel(blobs);
            pad_interp_blobs = cell(num_blobs, 1);

            blob_original_sizes = cat(1, blobs(:).bbsize);

            % nums_pad_slices : number of Ws in the padded blob
            % nums_interp_slices : number of groupped slices in the exit
            %                      blobs
            nums_pad_slices = blob_original_sizes(:, 3) + paddings(:, 1) + paddings(:, 2);
            nums_interp_slices = (nums_pad_slices -1) ./ slices_interp + 1;

            blobs_sizes = [ ...
                blob_original_sizes(:, 1), nums_interp_slices, blob_original_sizes(:, 2)];

            fprintf('Creating padded and interpolated blobs: ')
            c = tic();
            for ii = 1:num_blobs
                num_chars = fprintf('%03d/%03d', ii, num_blobs);

                blob = blobs(ii).intm;
                if (self.use_predicted_scatter_ints)
                    blob = blob ./ sum(blob(:)) .* blobs(ii).intensity;
                end
                blob(blob < 0) = 0;

                pad_interp_blobs{ii} = zeros(blobs_sizes(ii, :), self.data_type);

                % Let's build the interpolated slices
                for n = 1:nums_interp_slices(ii)
                    coeffs = abs( (slices_interp(ii) * (n-1) +1) - (1:nums_pad_slices(ii)));
                    coeffs = 1 - coeffs / slices_interp(ii);

                    % these will be the blob slices that relate to the
                    % given ii interpolated slice
                    good_indxes = find(coeffs > 0);
                    % The corresponding coefficients
                    coeffs = coeffs(good_indxes);

                    blob_indexes = good_indxes - paddings(ii, 1);
                    safe_indexes_pos = (blob_indexes <= blob_original_sizes(ii, 3)) & (blob_indexes >= 1);
                    safe_indexes = blob_indexes(safe_indexes_pos);
                    safe_coeffs = coeffs(safe_indexes_pos);

                    % We now produce the slice
                    slice = zeros(blob_original_sizes(ii, 1:2), self.data_type);
                    for w = 1:numel(safe_coeffs)
                        slice = slice + safe_coeffs(w) .* blob(:, :, safe_indexes(w));
                    end
                    pad_interp_blobs{ii}(:, n, :) = slice;
                end

                fprintf(repmat('\b', [1 num_chars]));
            end
            fprintf('\b\b (%f s).\n', toc(c));
        end

        function [w_tab_lims, w_tab, ref_w] = get_w_tab_numinterp(self, or_allblobs, ref_ws)
            omega_step = gtGetOmegaStepDeg(self.parameters);
            % Tabular of omegas: horizontal direction the n geometries,
            % vertical direction the omegas relative to each blob
            w_tab = [or_allblobs(:).omega];
            % Images numbers
            w_tab = w_tab / omega_step;

            ref_w = ref_ws / omega_step;

            w_tab = gtGrainAnglesTabularFix360deg(w_tab, ref_w, self.parameters);
            w_tab_lims = cat(3, floor(w_tab), ceil(w_tab));
        end

        function [w_tab_lims, ref_w] = get_w_tab_shapefuncs(self, shape_funcs, ref_ws)
            omega_step = self.parameters.labgeo.omstep;

            num_ors = numel(shape_funcs);
            num_blobs = numel(shape_funcs{1});
            w_tab_lims = zeros(num_blobs, num_ors, 2);
            % Tabular of omegas: horizontal direction the n geometries,
            % vertical direction the omegas relative to each blob
            for ii_or = 1:num_ors
                w_tab_lims(:, ii_or, :) = cat(1, shape_funcs{ii_or}(:).bbwim);
            end

            ref_w = ref_ws / omega_step;

            w_tab_lims(:, :, 1) = gtGrainAnglesTabularFix360deg(w_tab_lims(:, :, 1), ref_w, self.parameters);
            w_tab_lims(:, :, 2) = gtGrainAnglesTabularFix360deg(w_tab_lims(:, :, 2), ref_w, self.parameters);
        end

        function [lims_blobs_w_pad, lims_projs_w, paddings] = get_blob_lims(self, w_tab_lims, bl, ref_int_ws, slices_interp)
        % Finding limits for the shape functions

            % Let's add extreme values
            lims_projs_w = [ ...
                min([w_tab_lims(:, :, 1), w_tab_lims(:, :, 2)], [], 2), ...
                max([w_tab_lims(:, :, 1), w_tab_lims(:, :, 2)], [], 2) ];

            lims_blobs_w_orig = vertcat( bl(:).bbwim );

            % Let's treat those blobs at the w edge 360->0
            % (from the blobs perspective)
            lims_blobs_w_orig = gtGrainAnglesTabularFix360deg(lims_blobs_w_orig, ref_int_ws, self.parameters);

            paddings(:, 1) = max(lims_blobs_w_orig(:, 1) - lims_projs_w(:, 1), 0);
            paddings(:, 2) = max(lims_projs_w(:, 2) - lims_blobs_w_orig(:, 2), 0);

            % Adding the padding
            lims_blobs_w_pad = [ ...
                lims_blobs_w_orig(:, 1) - paddings(:, 1), ...
                lims_blobs_w_orig(:, 2) + paddings(:, 2) ];

            % computing the needed extension for the interpolation
            lims_blobs_w_pad = [ ...
                floor(lims_blobs_w_pad(:, 1) ./ slices_interp) .* slices_interp, ...
                ceil(lims_blobs_w_pad(:, 2) ./ slices_interp) .* slices_interp ];
            % The upper limit is actually one slice too many, but it is
            % supposed to be zero anyway

            paddings = [ ...
                lims_blobs_w_orig(:, 1) - lims_blobs_w_pad(:, 1), ...
                lims_blobs_w_pad(:, 2) - lims_blobs_w_orig(:, 2) ];
        end

        function geoms = compute_proj_geom_numinterp(self, bl, ref_gr, ors_allblobs)
            bls_bbuim = cat(1, bl.bbuim);
            bls_bbvim = cat(1, bl.bbvim);
            bls_bbsize = cat(1, bl.bbsize);
            bb_bls_uv = [bls_bbuim(:, 1), bls_bbvim(:, 1), bls_bbsize(:, [1 2])];

            num_orients = numel(ors_allblobs);
            bb_bls_uv_oi = repmat(bb_bls_uv, [num_orients, 1]);

            dvec_sam = cat(1, ors_allblobs(:).dvecsam);
            rot_w_l2s = cat(3, ors_allblobs(:).srot);

            proj_geom = gtGeoProjForReconstruction(...
                dvec_sam, rot_w_l2s, ref_gr.center, bb_bls_uv_oi, [], ...
                self.parameters.detgeo(self.det_index), ...
                self.parameters.labgeo, ...
                self.parameters.samgeo, ...
                self.parameters.recgeo(self.det_index), ...
                'ASTRA_grain');

            if (self.volume_downscaling > 1)
                proj_geom(:, 4:12) = proj_geom(:, 4:12) / self.volume_downscaling;
            end

            geoms = cell(num_orients, 1);
            count = 1;
            for ii = 1:num_orients
                num_vecs = size(ors_allblobs(ii).dvecsam, 1);
                geoms{ii} = double(proj_geom(count:(count+num_vecs-1), :));
                count = count + num_vecs;
            end
        end

        function offsets = compute_proj_coeffs_numinterp(~, lims_blobs_w, w_tab, slices_interp)
            num_blobs = size(w_tab, 1);
            num_orients = size(w_tab, 2);

            rescaled_w_tab = w_tab ./ slices_interp(:, ones(num_orients, 1));

            w_int_tab(:, :, 1) = floor(rescaled_w_tab);
            w_int_tab(:, :, 2) = ceil(rescaled_w_tab);

            slice_coeffs_in_blob(:, :, 1) = w_int_tab(:, :, 2) - rescaled_w_tab;
            slice_coeffs_in_blob(:, :, 2) = 1 - slice_coeffs_in_blob(:, :, 1);

            rescaled_low_lims_blobs_w = lims_blobs_w(:, 1) ./ slices_interp;
            slice_pos_in_blob = w_int_tab - repmat(rescaled_low_lims_blobs_w, [1 num_orients 2]) + 1;

            slice_coeffs_in_blob = permute(slice_coeffs_in_blob, [3 1 2]);
            slice_coeffs_in_blob = reshape(slice_coeffs_in_blob, 2 * num_blobs, num_orients);
            slice_pos_in_blob = permute(slice_pos_in_blob, [3 1 2]);
            slice_pos_in_blob = reshape(slice_pos_in_blob, 2 * num_blobs, num_orients);

            inds = 1:num_blobs;
            inds = reshape(inds([1 1], :), 1, []);

            % Assign offsets
            offsets = cell(num_orients, 1);
            for ii_o = 1:num_orients
                offsets{ii_o} = struct( ...
                    'sino_offsets', inds, ...
                    'blob_offsets', inds, ...
                    'proj_offsets', slice_pos_in_blob(:, ii_o)', ...
                    'proj_coeffs', slice_coeffs_in_blob(:, ii_o)' );
            end
        end

        function [geometries, offsets] = compute_proj_geom_shapefuncs_w(...
                self, bl, ref_gr, ors_allblobs, lims_blobs_w, sf_w, blobs_w_interp)
        % We here produce ASTRA's projection geometry and blob assempling
        % coefficients
            bls_bbuim = cat(1, bl.bbuim);
            bls_bbvim = cat(1, bl.bbvim);
            bls_bbsize = cat(1, bl.bbsize);
            bb_bls_uv = [bls_bbuim(:, 1), bls_bbvim(:, 1), bls_bbsize(:, [1 2])];

            num_blobs = numel(bl);
            num_orients = numel(ors_allblobs);

            rot_comp_w = gtMathsRotationMatrixComp(self.parameters.labgeo.rotdir', 'col');

            om_step = gtGetOmegaStepDeg(self.parameters);

            geometries = cell(num_orients, 1);
            offsets = cell(num_orients, 1);
            for ii_o = 1:num_orients
                num_chars = fprintf('%03d/%03d', ii_o, num_orients);

                sf = sf_w{ii_o};

                % Assign offsets
                ind_of_blob = cell(1, num_blobs);
                slice_coeff = cell(1, num_blobs);
                ws = cell(1, num_blobs);
                for ii_b = 1:num_blobs
                    cs = reshape(sf(ii_b).intm, 1, []);
                    iws = sf(ii_b).bbwim(1):blobs_w_interp(ii_b):sf(ii_b).bbwim(2);

                    valid_coeffs = cs > eps('single');
                    slice_coeff{ii_b} = cs(valid_coeffs);
                    ws{ii_b} = iws(valid_coeffs);

                    ind_of_blob{ii_b} = ii_b * ones(1, sum(valid_coeffs));
                end

                ws = [ws{:}];
                pos_in_sino = 1:numel(ws);
                ind_of_blob = [ind_of_blob{:}];
                pos_in_blob = (ws - lims_blobs_w(ind_of_blob, 1)') ./ blobs_w_interp(ind_of_blob, 1)' + 1;
                slice_coeff = [slice_coeff{:}];

                offsets{ii_o} = struct( ...
                    'sino_offsets', pos_in_sino, ...
                    'blob_offsets', ind_of_blob, ...
                    'proj_offsets', pos_in_blob, ...
                    'proj_coeffs', slice_coeff );

                % Computing geometries
                rot_w_l2s = gtMathsRotationTensor(ws' .* om_step, rot_comp_w);

                dvec_lab = ors_allblobs(ii_o).dvec(ind_of_blob, :);
                dvec_sam = gtGeoLab2Sam(dvec_lab, rot_w_l2s, ...
                    self.parameters.labgeo, self.parameters.samgeo, true);

                proj_geom = gtGeoProjForReconstruction(...
                    dvec_sam, rot_w_l2s, ref_gr.center, ...
                    bb_bls_uv(ind_of_blob, :), [], ...
                    self.parameters.detgeo(self.det_index), ...
                    self.parameters.labgeo, ...
                    self.parameters.samgeo, ...
                    self.parameters.recgeo(self.det_index), ...
                    'ASTRA_grain');

                if (self.volume_downscaling > 1)
                    proj_geom(:, 4:12) = proj_geom(:, 4:12) / self.volume_downscaling;
                end

                geometries{ii_o} = double(proj_geom);

                fprintf(repmat('\b', [1 num_chars]));
            end
        end

        function [geometries, offsets] = compute_proj_geom_shapefuncs_nw(...
                self, bl, ref_gr, ors_allblobs, lims_blobs_w, sf_nw, blobs_w_interp)
        % We here produce ASTRA's projection geometry and blob assempling
        % coefficients
            bls_bbuim = cat(1, bl.bbuim);
            bls_bbvim = cat(1, bl.bbvim);
            bls_bbsize = cat(1, bl.bbsize);
            bb_bls_uv = [bls_bbuim(:, 1), bls_bbvim(:, 1), bls_bbsize(:, [1 2])];

            num_blobs = numel(bl);
            num_orients = numel(ors_allblobs);

            rot_comp_w = gtMathsRotationMatrixComp(self.parameters.labgeo.rotdir', 'col');

            om_step = gtGetOmegaStepDeg(self.parameters);

            geometries = cell(num_orients, 1);
            offsets = cell(num_orients, 1);
            for ii_o = 1:num_orients
                num_chars = fprintf('%03d/%03d', ii_o, num_orients);

                sf = sf_nw{ii_o};

                bl_bbsizes = cat(1, sf.bbsize);

                % Assign offsets
                ind_of_blob = cell(1, num_blobs);
                slice_coeff = cell(1, num_blobs);
                ns = cell(1, num_blobs);
                ws = cell(1, num_blobs);
                for ii_b = 1:num_blobs
                    cs = sf(ii_b).intm;
                    cs = reshape(cs, 1, []);
                    % Omegas of each column of the shape-function
                    iws = sf(ii_b).bbwim(1):blobs_w_interp(ii_b):sf(ii_b).bbwim(2);

                    % Etas of each row of the shape-function
                    ins = linspace(sf(ii_b).bbnim(1), sf(ii_b).bbnim(2), bl_bbsizes(ii_b, 1));

                    ins = reshape(ins, [], 1);
                    ins = ins(:, ones(numel(iws), 1));
                    ins = reshape(ins, [], 1);

                    iws = iws(ones(sf(ii_b).bbsize(1), 1), :);
                    iws = reshape(iws, [], 1);

                    % ins and iws have been expanded to be of the same
                    % size, but there is some redundancy going on there.
                    % This could impact on the perforance of the projection
                    % geometry determination later in this same function,
                    % but this is the only way to keep consistency with the
                    % sinogram<->blobs transformation coefficients.
                    valid_coeffs = cs > eps('single');
                    ns{ii_b} = ins(valid_coeffs);
                    ws{ii_b} = iws(valid_coeffs);
                    slice_coeff{ii_b} = cs(valid_coeffs);

                    ind_of_blob{ii_b} = ii_b * ones(1, sum(valid_coeffs));
                end

                ws = cat(1, ws{:});
                ns = cat(1, ns{:});
                pos_in_sino = 1:numel(ws);
                ind_of_blob = [ind_of_blob{:}];
                pos_in_blob = ((ws - lims_blobs_w(ind_of_blob, 1)) ./ blobs_w_interp(ind_of_blob, 1))' + 1;
                slice_coeff = [slice_coeff{:}];

                offsets{ii_o} = struct( ...
                    'sino_offsets', pos_in_sino, ...
                    'blob_offsets', ind_of_blob, ...
                    'proj_offsets', pos_in_blob, ...
                    'proj_coeffs', slice_coeff );

                % Computing geometries
                rot_w_l2s = gtMathsRotationTensor(ws .* om_step, rot_comp_w);

                % Building the projection geometry from the etas and omegas
                t = ors_allblobs(ii_o).theta(ind_of_blob);
                pl_lab = gtGeoPlLabFromThetaEta(t, ns, self.parameters.labgeo);
                dvec_lab = gtFedPredictDiffVecMultiple(pl_lab', self.parameters.labgeo.beamdir')';
                dvec_sam = gtGeoLab2Sam(dvec_lab, rot_w_l2s, ...
                    self.parameters.labgeo, self.parameters.samgeo, true);

                proj_geom = gtGeoProjForReconstruction(...
                    dvec_sam, rot_w_l2s, ref_gr.center, ...
                    bb_bls_uv(ind_of_blob, :), [], ...
                    self.parameters.detgeo(self.det_index), ...
                    self.parameters.labgeo, ...
                    self.parameters.samgeo, ...
                    self.parameters.recgeo(self.det_index), ...
                    'ASTRA_grain');

                if (self.volume_downscaling > 1)
                    proj_geom(:, 4:12) = proj_geom(:, 4:12) / self.volume_downscaling;
                end

                geometries{ii_o} = double(proj_geom);

                fprintf(repmat('\b', [1 num_chars]));
            end
        end

        function blobs = renormalize_blobs(self, sampler, blobs)
            ref_gr = sampler.get_reference_grain();
            if (isfield(self.parameters.cryst(ref_gr.phaseid), 'int_exp'))
                cryst_scatter_ints = self.parameters.cryst(ref_gr.phaseid).int_exp;
            else
                cryst_scatter_ints = self.parameters.cryst(ref_gr.phaseid).int;
            end

            sel = sampler.ondet(sampler.included(sampler.selected));
            thetatypes = ref_gr.allblobs.thetatype(sel);
            scatter_ints = cryst_scatter_ints(thetatypes);
            lorentz_fact = ref_gr.allblobs.lorentzfac(sel);
            for ii_b = 1:numel(blobs)
                blobs{ii_b} = blobs{ii_b} ./ scatter_ints(ii_b) ./ lorentz_fact(ii_b);
            end
        end
    end

    methods (Access = protected, Static)
        function new_offsets_ss = merge_offsets(offsets_ss)
            new_offsets_ss = offsets_ss{1};
            cumulative_offs = max(offsets_ss{1}.sino_offsets);

            for ii_ss = 2:numel(offsets_ss)
                new_offsets_ss.sino_offsets ...
                    = cat(2, new_offsets_ss.sino_offsets, offsets_ss{ii_ss}.sino_offsets + cumulative_offs);
                new_offsets_ss.blob_offsets ...
                    = cat(2, new_offsets_ss.blob_offsets, offsets_ss{ii_ss}.blob_offsets);
                new_offsets_ss.proj_offsets ...
                    = cat(2, new_offsets_ss.proj_offsets, offsets_ss{ii_ss}.proj_offsets);
                new_offsets_ss.proj_coeffs ...
                    = cat(2, new_offsets_ss.proj_coeffs, offsets_ss{ii_ss}.proj_coeffs);

                num_projs = max(offsets_ss{ii_ss}.sino_offsets);
                cumulative_offs = cumulative_offs + num_projs;
            end
        end
    end
end