classdef Gt6DVolumeToBlobProjector < Gt6DVolumeProjector
    properties
        offsets = {};
        ss_offsets = {};
        blobs_depths = [];

        % Weights
        fwd_weights = {};
        bwd_weights = {};

        blob_sinogram_c_functions = true;
    end

    methods (Access = public)
        function self = Gt6DVolumeToBlobProjector(vols_size, detector_size, blobs_depths, varargin)
            self = self@Gt6DVolumeProjector(vols_size, detector_size, varargin{:});

            self.blobs_depths = blobs_depths;
        end

        function fwd_weights = getRowsSum(self)
            fwd_weights = gtMathsGetSameSizeZeros(self.blobs);

            chunk_size = self.num_gpus * self.jobs_bunch_size;

            vol_ones = ones(self.volume_geometry, self.data_type);
            vols_ones = cell(chunk_size, 1);
            for n = 1:chunk_size
                vols_ones{n} = vol_ones;
            end

            num_geoms = self.get_number_geometries();

            if (chunk_size > 1)
                chunk_end = num_geoms - rem(num_geoms, chunk_size);

                for n = 1:chunk_size:chunk_end
                    inds = n:(n+chunk_size-1);
                    fwd_weights = self.fwd_project_volume( ...
                        fwd_weights, vols_ones, inds);
                end
            else
                chunk_end = 0;
            end

            for n = (chunk_end+1):num_geoms
                fwd_weights = self.fwd_project_volume( ...
                    fwd_weights, vol_ones, n);
            end
        end

        function bwd_weights = getColumnsSum(self)
            num_blobs = numel(self.blobs_depths);
            blobs_ones = cell(1, num_blobs);
            for n = 1:num_blobs
                blobs_ones{n} = ones(self.proj_size(1), ...
                    self.blobs_depths(n), self.proj_size(2));
            end

            num_geoms = self.get_number_geometries();
            bwd_weights = cell(1, num_geoms);
            for n = 1:num_geoms
                bwd_weights{n} = self.bwd_project_volumes(blobs_ones, n);
            end
        end
    end

    methods (Access = protected)
        function [blobs, fp_time, bs_time] = fwd_project_volume(self, blobs, volume, n)
        % We get volumes in input and we want the blobs as output, but the
        % output from the base function is stacks of sinograms relative to
        % the geometry of the volumes, so we need to actually decompose the
        % output and recompose the slices into blobs

            c = tic();
            sino = self.fwd_project_single_volume(volume, n);
            fp_time = toc(c);

            num_orients = numel(n);
            if (num_orients > 1)
                bs_time = 0;
                for ii_o = 1:num_orients
                    [blobs, bs_time_o] = self.process_sino_fwd(blobs, sino{ii_o}, n(ii_o));
                    bs_time = bs_time + bs_time_o;
                end
            else
                [blobs, bs_time] = self.process_sino_fwd(blobs, sino, n);
            end
        end

        function [volume, bp_time, bs_time] = bwd_project_volume(self, blobs, n)
        % Similar problem to the projection function, with the only
        % difference that in this case the input is blobs and we want to
        % transform them into stack of sinograms

            num_orients = numel(n);
            if (num_orients > 1)
                sino = cell(num_orients, 1);
                bs_time = 0;
                for ii_o = 1:num_orients
                    [sino{ii_o}, bs_time_o] = self.process_sino_bwd(blobs, n(ii_o));
                    bs_time = bs_time + bs_time_o;
                end
            else
                [sino, bs_time] = self.process_sino_bwd(blobs, n);
            end

            c = tic();
            volume = self.bwd_project_single_volume(sino, n);
            bp_time = toc(c);
        end

        function [blobs, bs_time] = process_sino_fwd(self, blobs, sino, n)
        % Here we add the individual sinograms to the blobs

            if (self.using_super_sampling())
                offs = self.ss_offsets{n};
            else
                offs = self.offsets{n};
            end

            c = tic();
            try
                if (self.blob_sinogram_c_functions)
                    blobs = gt6DSingleSinoToBlobs_c(...
                        blobs, sino, offs, self.num_threads);
                else
                    blobs = self.single_sinogram_to_blobs(...
                        blobs, sino, offs);
                end
            catch mexc
                fprintf('Error raised by orientation: %d\n', n)
                rethrow(mexc)
            end
            bs_time = toc(c);
        end

        function [sino, bs_time] = process_sino_bwd(self, blobs, n)
        % Here we extract the individual sinograms from the blobs

            if (self.using_super_sampling())
                offs = self.ss_offsets{n};
                geom = self.ss_geometries{n};
            else
                offs = self.offsets{n};
                geom = self.geometries{n};
            end

            sinogram_size = [ ...
                self.proj_size(1), size(geom, 1), self.proj_size(2)];

            c = tic();
            try
                if (self.blob_sinogram_c_functions)
                    sino = gt6DBlobsToSingleSino_c( ...
                        sinogram_size, blobs, offs, self.num_threads);
                else
                    sino = self.blobs_to_single_sinogram(...
                        sinogram_size, blobs, offs);
                end
            catch mexc
                fprintf('Error raised by orientation: %d\n', n)
                rethrow(mexc)
            end
            bs_time = toc(c);
        end
    end

    methods (Access = public, Static)
        function blobs = single_sinogram_to_blobs(blobs, sino, coeffs)
            try
                sino_offsets = coeffs.sino_offsets(:);
                blob_offsets = coeffs.blob_offsets(:);
                proj_offsets = coeffs.proj_offsets(:);
                proj_coeffs = coeffs.proj_coeffs(:);
                for m = 1:numel(proj_offsets)
                    p = proj_offsets(m);
                    b = blob_offsets(m);
                    c = proj_coeffs(m);
                    s = sino_offsets(m);

                    blobs{b}(:, p, :) = blobs{b}(:, p, :) + c * sino(:, s, :);
                end
            catch mexc
                fprintf('\n\nError!!\n')
                fprintf('  indices: b %d, p %d, s %d, m %d\n', ...
                    b, p, s, m)
                fprintf('  size blob (%d %d %d), size sino (%d %d %d)\n\n', ...
                    size(blobs{b}), size(sino))
                fprintf('  Limits over p: [%d, %d]\n', ...
                    min(proj_offsets), max(proj_offsets))
                fprintf('  Limits over s: [%d, %d]\n\n', ...
                    min(sino_offsets), max(sino_offsets))
                rethrow(mexc)
            end
        end

        function sino = blobs_to_single_sinogram(sinogram_size, blobs, coeffs)
            try
                sino = zeros(sinogram_size, 'single');

                sino_offsets = coeffs.sino_offsets(:);
                blob_offsets = coeffs.blob_offsets(:);
                proj_offsets = coeffs.proj_offsets(:);
                proj_coeffs = coeffs.proj_coeffs(:);
                for m = 1:numel(proj_offsets)
                    p = proj_offsets(m);
                    b = blob_offsets(m);
                    c = proj_coeffs(m);
                    s = sino_offsets(m);

                    sino(:, s, :) = sino(:, s, :) + c * blobs{b}(:, p, :);
                end
            catch mexc
                fprintf('\n\nError!!\n')
                fprintf('  indices: b %d, p %d, s %d, m %d\n', ...
                    b, p, s, m)
                fprintf('  size blob (%d %d %d), size sino (%d %d %d)\n\n', ...
                    size(blobs{b}), size(sino))
                fprintf('  Limits over p: [%d, %d]\n', ...
                    min(proj_offsets), max(proj_offsets))
                fprintf('  Limits over s: [%d, %d]\n\n', ...
                    min(sino_offsets), max(sino_offsets))
                rethrow(mexc)
            end
        end
    end
end