function bl = gtDefFwdProjGvdm2W(grain, ref_sel, gv, fedpars, parameters, det_ind, verbose)

    if (~exist('det_ind', 'var'))
        det_ind = 1;
    end
    if (~exist('verbose', 'var'))
        verbose = true;
    end

    o = GtConditionalOutput(verbose);

    o.fprintf('Forward projection (%s):\n', mfilename)

    nbl = numel(ref_sel);
    uinds = gv.used_ind;
    nv = numel(uinds);

    labgeo = parameters.labgeo;
    samgeo = parameters.samgeo;
    om_step = gtGetOmegaStepDeg(parameters, det_ind);

    g = gtMathsRod2OriMat(grain.R_vector);

    if (strcmpi(fedpars.defmethod, 'rod_rightstretch'))
        if (isfield(grain.allblobs, 'plcry'))
            pls_orig = grain.allblobs.plcry(ref_sel, :);
        else
            % We bring the plane normals back to the status of pl_cry
            pls_orig = grain.allblobs.plorig(ref_sel, :);
            pls_orig = gtVectorLab2Cryst(pls_orig, g);
        end
    else
        pls_orig = grain.allblobs.plorig(ref_sel, :);
    end

    if (isfield(fedpars, 'detector') ...
        && isfield(fedpars.detector(det_ind), 'blobs_w_interp') ...
        && ~isempty(fedpars.detector(det_ind).blobs_w_interp))
        blobs_w_interp = fedpars.detector(det_ind).blobs_w_interp;
        if (numel(blobs_w_interp) ~= nbl)
            error('gtDefFwdProjGvdm2W:wrong_argument', ...
                'Number of "blobs_w_interp" (%d) doesn''t match with the number of blobs (%d)', ...
                numel(blobs_w_interp), nbl)
        end
    else
        blobs_w_interp = ones(nbl, 1);
    end

    sinths = grain.allblobs.sintheta(ref_sel);
    ominds = grain.allblobs.omind(ref_sel);

    if (isfield(fedpars, 'ref_omega'))
        ref_oms = fedpars.ref_omega(ref_sel);
    else
        ref_oms = grain.allblobs.omega(ref_sel);
    end
    w_shifts = round(ref_oms ./ om_step ./ blobs_w_interp);

    % The plane normals need to be brought in the Lab reference where the
    % beam direction and rotation axis are defined.
    % Use the Sample -> Lab orientation transformation assuming omega=0;
    % (vector length preserved for free vectors)
    pls_orig = gtGeoSam2Lab(pls_orig, eye(3), labgeo, samgeo, true, false);

    linear_interp = ~(isfield(fedpars, 'projector') ...
        && strcmpi(fedpars.projector, 'nearest'));

    gvpow = gv.pow(1, uinds);

    gvd = gv.d(:, uinds);
    % Deformation tensor (relative to reference state) from its components
    defT = gtFedDefTensorFromComps(gvd, fedpars.dcomps, fedpars.defmethod, 0);

    %%% Computation of indices
    o.fprintf('   * Computing indices and bbsizes: ')
    t = tic();

    w = cell(nbl, 1);
    w_min = zeros(nbl, 1);
    w_max = zeros(nbl, 1);

    valid_voxels = false(nv, nbl);

    for ii_b = 1:nbl
        num_chars = o.fprintf('%03d/%03d', ii_b, nbl);
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Calculate new detector coordinates
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        % New deformed plane normals and relative elongations (relative to
        % reference state)
        pl_orig = pls_orig(ii_b, :)';  % ! use plcry and not plsam to keep omega order below!

        [pl_samd, drel] = gtStrainPlaneNormals(pl_orig, defT); % unit column vectors

        % New sin(theta)
        sinth = sinths(ii_b) ./ drel;

        % Predict omega angles: 4 for each plane normal
        om = gtFedPredictOmegaMultiple(pl_samd, sinth, labgeo.beamdir', ...
            labgeo.rotdir', labgeo.rotcomp, ominds(ii_b));

        valid_voxels(:, ii_b) = ~isnan(om);

        % Delete those where no reflection occurs
        if (any(isnan(om)))
            inds_bad = find(isnan(om));
            gvd(:, inds_bad(1:min(10, numel(inds_bad))))
            warning('gtFedFwdProjExact:bad_R_vectors', ...
                'No diffraction from some elements after deformation (%d over %d) for blob %d.', ...
                numel(inds_bad), numel(om), ii_b)
        end

        om = gtGrainAnglesTabularFix360deg(om, ref_oms(ii_b));

        w_bl = om ./ om_step;
        w_bl = w_bl / blobs_w_interp(ii_b);
        w_bl = w_bl - w_shifts(ii_b);

        w{ii_b} = w_bl;

        w_min(ii_b, :) = min(w{ii_b});
        w_max(ii_b, :) = max(w{ii_b});

        o.fprintf(repmat('\b', [1, num_chars]));
    end
    o.fprintf('Done in %g s\n', toc(t));

    o.fprintf('   * Computing max BBox size and feasibilities:\n')

    if (linear_interp)
        blob_w_sizes = ceil(w_max) - floor(w_min) + 1;
        blob_orig_w_shifts = 1 - floor(w_min);
    else
        blob_w_sizes = round(w_max) - round(w_min) + 1;
        blob_orig_w_shifts = 1 - round(w_min);
    end

    o.fprintf('     Computed Blob W sizes: [%s]\n', sprintf(' %d', blob_w_sizes));

    bl = gtFwdSimBlobDefinition('sf_w', nbl);

    %%% Blob projection
    o.fprintf('   * Projecting volumes: ')
    t = tic();
    for ii_b = 1:nbl
        num_chars = o.fprintf('%03d/%03d', ii_b, nbl);

        % Detector coordinates U,V in blob
        w_bl = w{ii_b} + blob_orig_w_shifts(ii_b * ones(1, nv))';

        % Let's now filter valid voxels
        w_bl = w_bl(:, valid_voxels(:, ii_b))';
        gvpow_v = reshape(gvpow(valid_voxels(:, ii_b)), [], 1);

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Sum intensities
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Distribute value over 8 pixels

        if (linear_interp)
            [inds_w, ints_w] = gtMathsGetInterpolationIndices(w_bl, ...
                gvpow_v, fedpars.bltype);
        else
            inds_w = round(w_bl);
            ints_w = gvpow_v;
        end

        % Accumulate all intensities in the blob voxel-wise
        %   'uvw' needs to be nx1; 'ints' is now 1xnx8
        try
            bl(ii_b).intm = accumarray(inds_w, ints_w, [blob_w_sizes(ii_b), 1]);
        catch mexc
            fprintf(['\n\nERROR\nThe error is probably caused by the' ...
                ' projected intensities falling outside the blob volume.' ...
                '\nTry increasing the blob volume padding:' ...
                ' fedpars.detector(det_ind).blobsizeadd, or FIXING' ...
                ' YOUR CODE!!\n\n'])
            disp('Blob ID:')
            disp(ii_b)
            disp('Blob size:')
            disp(blob_w_sizes(ii_b))
            disp('Min projected W coordinate:')
            fprintf('\t%g \t(rounded: %d )\n', ...
                min(w_bl, [], 1), min(inds_w, [], 1))
            disp('Max projected W coordinate:')
            fprintf('\t%g \t(rounded: %d )\n', ...
                max(w_bl, [], 1), max(inds_w, [], 1))
            new_exc = GtFedExceptionFwdProj(mexc, det_ind, [0 0 0]);
            throw(new_exc)
        end

        bl(ii_b).bbsize = blob_w_sizes(ii_b);

        im_low_lim = w_shifts(ii_b) - blob_orig_w_shifts(ii_b) + 1;
        bl(ii_b).bbwim = [im_low_lim, im_low_lim + bl(ii_b).bbsize - 1] * blobs_w_interp(ii_b);
        bl(ii_b).interpw = blobs_w_interp(ii_b);

        o.fprintf(repmat('\b', [1, num_chars]));
    end
    o.fprintf('Done in %g s\n', toc(t));
end
