function [samXYZ, lab2sam] = gtGeoLab2Sam(labXYZ, omega, labgeo, samgeo, ...
                             freevec, rot_flag, element_wise)
% GTGEOLAB2SAM  LAB -> SAMPLE reference coordinate transformation
%
% [samXYZ, lab2sam] = gtGeoLab2Sam(labXYZ, omega, labgeo, ...
%                                  samgeo, freevec, rot_flag)
%
% ----------------------------------------------------------------------
%
% Given the LAB coordinates of a set of points, transforms them 
% into SAMPLE coordinates according to an omega rotation. Omega is usually
% positive.
% It optionally creates the transformation matrix.
%
% INPUT  
%   labXYZ         - coordinates in the Lab reference in pixels (n,3)
%   omega          - omega angles in degrees (n,1)
%   labgeo.rotpos  - rotation axis position in Lab coordinates (1,3)
%   labgeo.rotdir  - rotation axis direction (right-handed rotation)
%   samgeo.orig    - origin of the sample reference in Lab coordinates (1,3)
%   samgeo.voxsize - [x,y,z] voxel sizes in lab units
%   freevec        - 0 for fixed vectors (e.g. location);
%                    1 for free vectors (e.g. orientation/direction)
%   rot_flag       - if true, only applies rotation so the norm of the 
%                    input vectors will be unchanged; 
%                    i.e. no scaling applied according to the 
%                    voxel sizes (only works if 'samgeo.lab2sam' is undefined)
%
% OUTPUT 
%   samXYZ         - coordinates in the Sample reference in no. of voxels (n,3)
%   lab2sam        - coordinate transformation matrix
%
% OPTIONAL, INPUT:
%   omega          - can be rotation tensors ('col') from omega (3,3,n) or
%                    simply (3,3,1) which would then be used for all the
%                    lab vectors.
%

if (~exist('rot_flag', 'var') || isempty(rot_flag))
    rot_flag = false;
end

if (~exist('element_wise', 'var') || isempty(element_wise))
    element_wise = true;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Change of reference from Lab to Sample
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Transformation matrix from Lab to Sample coordinates:
%   Note that Lab is an orthonormal basis by definition, but Sample may not be !!
%   Therefore needs calculation of the inverse, and not only transpose.
if (~isfield(samgeo,'lab2sam') || (isfield(samgeo,'lab2sam') && isempty(samgeo.lab2sam)))
    
    if (rot_flag)
        % Keep norm of input vectors -> keep unit lengths in the rows of 
        % the transformation matrix 
        sam2lab = [samgeo.dirx; samgeo.diry; samgeo.dirz];
    else
        sam2lab = [ ...
            samgeo.dirx * samgeo.voxsize(1); ...
            samgeo.diry * samgeo.voxsize(2); ...
            samgeo.dirz * samgeo.voxsize(3) ];
    end

    lab2sam = inv(sam2lab);

elseif (rot_flag)
    error('Transformation matrix already defined. Set ''rot_flag'' to false.')
else
    lab2sam = samgeo.lab2sam;   
end

%%%%%%%%%%%%%%%%%%%%%
%%% Rotation in Lab
%%%%%%%%%%%%%%%%%%%%%

ones_in = ones(size(labXYZ, 1), 1);

[size_omega(1), size_omega(2), size_omega(3)] = size(omega);
% Tensor which rotates with omega from Lab back to Sample:
if (all(size_omega(1:2) == [3 3]))
    % This means that the rotation tensors were passed already!
    if (size_omega(3) == 1)
        rottensors = omega(:, :, ones_in);
    else
        rottensors = omega;
    end
else
    rotcomp    = gtMathsRotationMatrixComp(labgeo.rotdir', 'col');
    rottensors = gtMathsRotationTensor(omega, rotcomp);
end

if (~freevec)
    % Vectors relative to rotation axis position
    labXYZ = labXYZ - labgeo.rotpos(ones_in, :);
end

rottensors_t = reshape(rottensors, 3, []);

if (element_wise)
    % Multiply element-wise to avoid loop (applying rottensors to labXYZ)
    labXYZ_t = [labXYZ, labXYZ, labXYZ]';
    labXYZ_t = reshape(labXYZ_t, 3, []);
    pro = labXYZ_t .* rottensors_t;

    % Sum, reshape, transpose to get rotated vector
    labXYZ_rot = reshape(sum(pro, 1), 3, [])';
else
    % output wll be groupped by (all rotations per center) x number centers
    labXYZ_rot = labXYZ * rottensors_t;
    labXYZ_rot = reshape(labXYZ_rot', 3, [])';
end

if (~freevec)
    ones_out = ones(size(labXYZ_rot, 1), 1);
    % Set offset back to get rotated vector
    labXYZ_rot = labXYZ_rot + labgeo.rotpos(ones_out, :) - samgeo.orig(ones_out, :);
end

samXYZ = labXYZ_rot * lab2sam;

end % of function
