function [labXYZ, sam2lab] = gtGeoSam2Lab(samXYZ, omega, labgeo, samgeo, ...
                             freevec, rot_flag, element_wise)
% GTGEOSAM2LAB  SAMPLE -> LAB reference coordinate transformation.
%
% [labXYZ, sam2lab] = gtGeoSam2Lab(samXYZ, om, labgeo, samgeo, ...
%                              freevec, rot_flag)
%
% ----------------------------------------------------------------------- 
%
% Given the SAMPLE coordinates of a set of points, transforms them 
% into LAB coordinates according to an omega rotation. Omega is usually
% positive.
% It optionally creates the transformation matrix.
%
% INPUT  
%   samXYZ         - coordinates in the Sample reference in no. of voxels (n,3)
%   omega          - omega angles in degrees (n,1)
%   labgeo.rotpos  - rotation axis position in Lab coordinates (1,3)
%   labgeo.rotdir  - rotation axis direction (right-handed rotation)
%   samgeo.orig    - origin of the sample reference in Lab coordinates (1,3)
%   samgeo.voxsize - [x,y,z] voxel sizes in lab units
%   freevec        - 0 for fixed vectors (e.g. location);
%                    1 for free vectors (e.g. orientation)
%   rot_flag       - if true, only applies rotation and the norm of the 
%                    input vectors will be unchanged; i.e. no scaling 
%                    applied according to the voxel sizes (only works if 
%                    'samgeo.sam2lab' is undefined)
%
% OUTPUT 
%   labXYZ         - row vector of SAMPLE coordinates (n,3)
%   sam2lab        - coordinate transformation matrix
%
% OPTIONAL, INPUT:
%   omega          - can be rotation tensors ('col') from -omega (3,3,n) or
%                    simply (3,3,1) which would then be used for all the
%                    lab vectors.
%

if (~exist('rot_flag', 'var') || isempty(rot_flag))
    rot_flag = false;
end

if (~exist('element_wise', 'var') || isempty(element_wise))
    element_wise = true;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Change of reference from Sample to Lab
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Transformation matrix from Sample to Lab coordinates:
%   (Lab is an orthonormal basis by definition.)
if (~isfield(samgeo,'sam2lab') || (isfield(samgeo,'sam2lab') && isempty(samgeo.sam2lab)))
    
    if (rot_flag)
        % Keep norm of input vectors -> keep unit lengths in the rows of
        % the transformation matrix
        sam2lab = [samgeo.dirx; samgeo.diry; samgeo.dirz];
    else
        sam2lab = [ ...
            samgeo.dirx * samgeo.voxsize(1); ...
            samgeo.diry * samgeo.voxsize(2); ...
            samgeo.dirz * samgeo.voxsize(3) ];
    end
    
elseif (rot_flag)
    error('Transformation matrix already defined. Set ''rot_flag'' to false.')
else
    sam2lab = samgeo.sam2lab;
end

%%%%%%%%%%%%%%%%%%%%%
%%% Rotation in Lab
%%%%%%%%%%%%%%%%%%%%%

ones_in = ones(size(samXYZ, 1), 1);

[size_omega(1), size_omega(2), size_omega(3)] = size(omega);
% Tensor which rotates with omega from Sample to Lab
if (all(size_omega(1:2) == [3 3]))
    % This means that the rotation tensors were passed already!
    if (size_omega(3) == 1)
        rottensors = omega(:, :, ones_in);
    else
        rottensors = omega;
    end
else
    rotcomp    = gtMathsRotationMatrixComp(labgeo.rotdir', 'col');
    rottensors = gtMathsRotationTensor(-omega, rotcomp);
end

% Vectors in Lab reference at the origin before rotation
labXYZprerot = samXYZ * sam2lab;

if (~freevec)
    % Vectors relative to rot. axis position in Lab reference
    vv           = samgeo.orig - labgeo.rotpos;
    labXYZprerot = labXYZprerot + vv(ones_in,:);
end

rottensors_t = reshape(rottensors, 3, []);

if (element_wise)
    % Multiply element-wise to avoid loop
    labXYZprerot_t = [labXYZprerot, labXYZprerot, labXYZprerot]';
    labXYZprerot_t = reshape(labXYZprerot_t, 3, []);
    pro = labXYZprerot_t .* rottensors_t;

    % Sum, reshape, transpose
    labXYZ = reshape(sum(pro, 1), 3, [])';
else
    % output wll be groupped by (all rotations per center) x number centers
    labXYZ = labXYZprerot * rottensors_t;
    labXYZ = reshape(labXYZ', 3, [])';
end

if (~freevec)
    ones_out = ones(size(labXYZ, 1), 1);
    % Set offset back
    labXYZ = labXYZ + labgeo.rotpos(ones_out, :);
end

end % of function
