function updateOar(queryname, action)
% UPDATEOAR
%     updateOar(queryname, action)
%     ----------------------------
%
%     Check OAR arrayjobs from OAR and update oarparameters.mat
%
%     INPUT:
%       queryname = arrayid <double> | function name <string>
%       action    = 'delete' | {'update'} | ('err','out','log','params','oar')
%                                           Use of logviewOar
%
%     Sub-functions:
%[sub]- deteleOarArray
%[sub]- updateArray
%[sub]- hasAllFields
%[sub]- clean


    if (~exist('queryname', 'var') || isempty(queryname))
        queryname = 'gt*';
    end
    if (~exist('action', 'var') || isempty(action))
        action = 'update';
    end

    oarparameters = gtOarLoadParameters('oarparameters');
    % Needed for compatibility with old files
    oarparameters = clean(oarparameters);

    if ischar(queryname)
        lookfunc = true;
        funcname = queryname;
    else
        lookfunc = false;
        arrayid = queryname;
    end

    fnames = fieldnames(oarparameters);

    if (lookfunc)
        %%%%%%%%%%%%%%%%%%%%%%%%%%
        % look for function name %
        %%%%%%%%%%%%%%%%%%%%%%%%%%
        if (~isempty(funcname))
            fprintf('Looking for functions that match regular expression: %s\n', ...
                funcname)
            match = regexpi(fnames, funcname, 'once');
            match = ~cellfun(@isempty, match);
            fnames = fnames(match);
        end

        if (ismember(lower(action), {'delete','del','d'}))
            % loop over functions
            for jj = 1:length(fnames)
                disp(['Deleting function ' fnames{jj} '...']);
                arrayField = fieldnames(oarparameters.(fnames{jj}));

                questionStr = ['Are you sure to remove the function ' ...
                    fnames{jj} ' from OAR and oarparameters.mat? [y/n]'];
                check = inputwdefault(questionStr, 'y');
                if strcmpi(check, 'y')
                    fprintf('Deleting jobs from OAR... ')
                    cellfun(@(array) deleteOarArray(array(6:end)), arrayField);
                    fprintf('Done.\nRemoving information about the jobs... ')
                    oarparameters = rmfield(oarparameters, fnames{jj});
                    fprintf('Done.\n')
                end
            end
        elseif (ismember(lower(action), {'update','upd','u'}))
            % loop over functions
            for jj = 1:length(fnames)
                arrays = fieldnames(oarparameters.(fnames{jj}));
                arrayName = fnames{jj};

                for kk = 1:length(arrays)
                    arrayField = arrays{kk};
                    oarparameters = updateArray(oarparameters, arrayName, arrayField);
                end % end for k
            end % end for j
        elseif ismember(lower(action), {'err','out','log','params','oar'})
            logviewOar(funcname, action);
        else
            % Fail :)
            gtError( 'OAR:wrong_argument', ...
                    'Action %s is not valid!', action)
        end
    else
        %%%%%%%%%%%%%%%%%%%%%%%%%
        % look for array number %
        %%%%%%%%%%%%%%%%%%%%%%%%%
        if (ismember(lower(action), {'err', 'out', 'log', 'params', 'oar'}))
            fprintf('Looking for job number "%d"...\n', arrayid)
            logviewOar(arrayid, action);
        else
            % loop over functions
            fprintf('Looking for array number "%d"...\n', arrayid)

            for jj = 1:length(fnames)
                arrayName = fnames{jj};

                % search array id(s) for each function
                arrays = logviewOar(arrayName, [], false);
                % check if arrayid exists
                if ~isempty(find(arrays.(arrayName).arrays == arrayid, 1))
                    arrayField = sprintf('array%d', arrayid);
                    % Do we want to delete?
                    if ismember(lower(action), {'delete','del','d'})
                        disp(['Deleting array n. ' num2str(arrayid) '...']);
                        questionStr = ['Are you sure to remove array ' ...
                            num2str(arrayid) ' from oarparameters.mat? [y/n]'];
                        check = inputwdefault(questionStr, 'y');
                        if strcmpi(check, 'y')
                            fprintf('Deleting jobs from OAR... ')
                            deleteOarArray(arrayid);
                            fprintf('Done.\nRemoving information about the jobs... ')

                            oarparameters.(arrayName) = rmfield(oarparameters.(arrayName), arrayField);
                            fprintf('Done.\n')
                        end
                    % Do we want some update?
                    elseif ismember(lower(action), {'update','upd','u'})
                        oarparameters = updateArray(oarparameters, arrayName, arrayField);
                    else
                        % Fail :)
                        gtError( 'OAR:wrong_argument', ...
                                'Action %s is not valid!', action)
                    end
                end % end if arrayid exists
            end % end for jj
        end
    end % end lookfunc

    gtOarSaveParameters('oarparameters', oarparameters);
end % end of function

%%
% Sub-functions

function deleteOarArray(array)
% DELETEOARARRAY
% deleteOarArray(array)

    if ~ischar(array)
        array = num2str(array);
    end
    [~, message] = unix(['oardel --array ' array]);

    if strfind(message, '[ERROR]')
        error('OAR:error_in_deleting', message);
    end
end

function hasFields = hasAllFields(structToCheck, fields)
% HASALLFIELDS
% hasFields = hasAllFields(structToCheck, fields)

    for n = 1:numel(fields)
        if (~isfield(structToCheck, fields{n}))
            hasFields = false;
            return
        end
    end
    hasFields = true;
end

function oarparams = clean(oarparams)
% CLEAN
% oarparams = clean(oarparams)

    if (~isempty(oarparams))
        funcNames = fieldnames(oarparams);
        for funcName = funcNames'
            func = oarparams.(funcName{1});
            arrayNames = fieldnames(func);
            for arrayName = arrayNames'
                if (isempty(func.(arrayName{1})))
                    func = rmfield(func, arrayName{1});
                end
            end
            oarparams.(funcName{1}) = func;
            if (isempty(fieldnames(func)))
                oarparams = rmfield(oarparams, funcName{1});
            end
        end
    end
end

function oarparameters = updateArray(oarparameters, arrayName, arrayField)
% UPDATEARRAY
% oarparameters = updateArray(oarparameters, arrayName, arrayField)

    vars = {'job_array_index', 'state', 'exit_code', 'submissionTime', 'startTime', 'stopTime'};

    jobs = oarparameters.(arrayName).(arrayField).job;
    njobs = numel(jobs);

    fprintf('Quering OAR to get fresh information about array: %s ...', arrayField(6:end))
    oarStats = gtOarGetStats([jobs.jobId], vars);
    fprintf(' Done.\n')

    h_name = length(arrayName)+2;
    h_job  = length(num2str(jobs(1).jobId))+2;
    h_other = length(datestr(now(),'yyyy-mm-dd HH:MM:SS'))+2;

    % Update each job record
    for m = 1 : njobs
        % Checking presence of fields
        if (~hasAllFields(oarStats(m), vars))
            fprintf('Skipping incomplete record for Job: %d (consider deleting it)\n', ...
                jobs(m).jobId)
            continue;
        end

        % added job_array_index to output
        jobs(m).(vars{1}) = oarStats(m).(vars{1});
        jobs(m).(vars{2}) = oarStats(m).(vars{2});
        fprintf('%*s %*d %*s %*s ', h_name, arrayName, h_job, jobs(m).jobId, ...
            3+2, oarStats(m).(vars{1}), h_other-7, oarStats(m).(vars{2}))
        for l = 3 : length(vars)
            jobs(m).(vars{l}) = oarStats(m).(vars{l});
            fprintf('%*s ', h_other, jobs(m).(vars{l}))
        end
        fprintf('\n')
    end

    matchesStates = strcmp({jobs(:).state}, 'Terminated');
    matchesCodes = strcmp({jobs(:).exit_code}, '0 (0,0,0)');
    matches = matchesStates & matchesCodes;
    if any(matches)
        fprintf('Removed terminated jobs from array: %s\n', arrayField(6:end))
        jobs = jobs(~matches);
    end

    oarparameters.(arrayName).(arrayField).job = jobs;

    if (isempty(oarparameters.(arrayName).(arrayField).job))
        fprintf('All the jobs from array: "%s" are terminated with exit_code "0 (0,0,0)"\n', ...
            arrayField(6:end))
        oarparameters.(arrayName) = rmfield(oarparameters.(arrayName), arrayField);
    end

    if (isempty(oarparameters.(arrayName)))
        fprintf('Removed "%s" from oarparameters.[xml|mat]\n', arrayName)
        oarparameters = rmfield(oarparameters, arrayName);
    end
end % end subfunction updateArray

