function volume = gtVolRead(filename, infoVOL, skip, outFtype)
% GTVOLREAD  Reads a volume in raw format into the matrix image.
%     -------------------------------------------------------------------------
%     volume = gtVolRead(filename, infoVOL, skip, outFtype)
%
%     INPUT:
%       filename = <string>  Name of the input file
%       infoVOL  = <struct>  Data structure with the following fields:
%                            - ftype: datatype, see fread for possibilities
%                            - volSizeX: first dimension (rows in Matlab)
%                            - volSizeY: second dimension (columns in Matlab)
%                            - volSizeZ: third dimension (default is 1)
%                            - fbyteorder: 'b' or 'l' (default); big endian or
%                              little endian
%
%     OPTIONAL INPUT:
%       skip     = <int>     Number of bytes skipped at file start
%                            default is 0
%       outFtype = <string>  Can be used to force the output datatype
%
%     OUTPUT:
%       volume   = <3Dimage> Output volume with the original type, if not 
%                            enforced by the user in the field ftype
%
%     Version 003 10-02-2012 by NVigano,   nicola.vigano@esrf.fr   
%       Imported and changed API to use structure, added better error
%       checking/reporting
%
%     Version 002 XX-03-2006 by GJohnson
%       Output singles instead of doubles (requires half the memory, useful 
%       for large volumes).
%
%     Version 001            by WLudwig,   wolfgang.ludwig@esrf.fr 
%                               PReischig, peter.reischig@esrf.fr 

    if ~exist('infoVOL', 'var')
        mexc = MException('VOL:missing_parameter', ...
                          'The volume info structure is required');
        throw(mexc);
    else
        if ~isfield(infoVOL, 'fbyteorder')
            [~, ~, infoVOL.fbyteorder] = computer();
        end
        if ~isfield(infoVOL, 'volSizeX')
            mexc = MException('VOL:missing_parameter', ...
                              'Information on the X dimension is required');
            throw(mexc);
        end
        if ~isfield(infoVOL, 'volSizeY')
            mexc = MException('VOL:missing_parameter', ...
                              'Information on the Y dimension is required');
            throw(mexc);
        end
        if ~isfield(infoVOL, 'volSizeZ')
            infoVOL.volSizeZ = 1;
        end
    end

    if ~exist('skip', 'var')
        skip = 0;
    end

    if ~any(find(infoVOL.ftype == '>'))
        if (exist('outFtype', 'var'))
            datatype = [infoVOL.ftype '=>' outFtype];
        else
            datatype = [infoVOL.ftype '=>' infoVOL.ftype];
        end
    else
        datatype = infoVOL.ftype;
    end

    fid = fopen(filename, 'r', infoVOL.fbyteorder);
    if (fid == -1)
        mexc = MException('VOL:wrong_file_name', ...
                          ['File ''' filename ''' does not exist!']);
        throw(mexc);
    end

    if (fseek(fid, skip, 'bof') == -1)
        fclose(fid);
        error('VOL:wrong_argument', 'Problems reading file, file size ???');
    end

    [volume, count] = fread(fid, infoVOL.volSizeX*infoVOL.volSizeY*infoVOL.volSizeZ, datatype);
    fclose(fid);

    if (count ~= (infoVOL.volSizeX * infoVOL.volSizeY * infoVOL.volSizeZ))
        mexc = MException('VOL:wrong_parameters', ...
                          'The volume contains less data than expected');
        throw(mexc);
    end

    volume = reshape(volume, [infoVOL.volSizeX infoVOL.volSizeY infoVOL.volSizeZ]);
end
